@@ -67,7 +67,7 @@ void KrashConfig :: readConfig()
   m_safeMode = args->isSet("safer");
   m_execname = args->getOption( "appname" );
   if ( !args->getOption( "apppath" ).isEmpty() )
-    m_execname.prepend( args->getOption( "apppath" ) + "/" );
+    m_execname.prepend( args->getOption( "apppath" ) + '/' );
 
   QCString programname = args->getOption("programname");
   if (programname.isEmpty())
@@ -131,6 +131,7 @@ void KrashConfig :: readConfig()
     m_showbacktrace = preset.readBoolEntry("ShowBacktraceButton", true);
     m_showdebugger = preset.readBoolEntry("ShowDebugButton", true);
   }
+  m_disablechecks = preset.readBoolEntry("DisableChecks", false);
 
   bool b = preset.readBoolEntry("SignalDetails", true);
 
