@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    ---
    Copyright (C) 2004, Anders Lund <anders@alweb.dk>
@@ -25,6 +25,7 @@
 #include "kateexternaltools.h"
 #include "kateexternaltools.moc"
 #include "katedocmanager.h"
+#include "kateviewmanager.h"
 #include "kateapp.h"
 
 #include "katemainwindow.h"
@@ -40,6 +41,8 @@
 #include <kconfig.h>
 #include <krun.h>
 #include <kicondialog.h>
+#include <kpopupmenu.h>
+#include <kdebug.h>
 
 #include <qbitmap.h>
 #include <qcombobox.h>
@@ -57,10 +60,6 @@
 
 #include <stdlib.h>
 #include <unistd.h>
-
-#include <kdebug.h>
-#include <kapplication.h>
-#include <kmdi/mainwindow.h>
 //END Includes
 
 KateExternalToolsCommand *KateExternalToolsCommand::s_self=0;
@@ -186,13 +185,13 @@ void KateExternalToolsCommand::reload () {
   else m_inited=true;
 }
 
-bool KateExternalToolsCommand::exec (Kate::View *view, const QString &cmd, QString &msg) {
+bool KateExternalToolsCommand::exec (Kate::View *view, const QString &cmd, QString &) {
 	QWidget *wv=dynamic_cast<QWidget*>(view);
 	if (!wv) {
 // 		kdDebug(13001)<<"KateExternalToolsCommand::exec: Could not get view widget"<<endl;
 		return false;
 	}
-  KMDI::MainWindow *dmw=dynamic_cast<KMDI::MainWindow*>(wv->topLevelWidget());
+  KateMDI::MainWindow *dmw=dynamic_cast<KateMDI::MainWindow*>(wv->topLevelWidget());
 	if (!dmw) {
 // 		kdDebug(13001)<<"KateExternalToolsCommand::exec: Could not get main window"<<endl;
 		return false;
@@ -212,7 +211,7 @@ bool KateExternalToolsCommand::exec (Kate::View *view, const QString &cmd, QStri
 	return true;
 }
 
-bool KateExternalToolsCommand::help (Kate::View *view, const QString &cmd, QString &msg) {
+bool KateExternalToolsCommand::help (Kate::View *, const QString &, QString &) {
 	return false;
 }
 //END KateExternalToolsCommand
@@ -310,7 +309,6 @@ KateExternalToolsMenuAction::KateExternalToolsMenuAction( const QString &text,
 void KateExternalToolsMenuAction::reload()
 {
   m_actionCollection->clear ();
-
   popupMenu()->clear();
 
   // load all the tools, and create a action for each of them
@@ -326,7 +324,7 @@ void KateExternalToolsMenuAction::reload()
   config->setReadDefaults( false );
 
   int ver = config->readNumEntry( "version" );
-  if ( ver < gver )
+  if ( ver <= gver )
   {
     QStringList removed = config->readListEntry( "removed" );
     bool sepadded = false;
