@@ -26,6 +26,7 @@
 #include <kglobal.h>
 #include <kconfig.h>
 #include <kinstance.h>
+#include <kstartupinfo.h>
 #include <dcopclient.h>
 #include <dcopref.h>
 #include <kdebug.h>
@@ -98,6 +99,7 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
   // command line args init and co
   KCmdLineArgs::init (argc, argv, &aboutData);
   KCmdLineArgs::addCmdLineOptions (options);
+  KCmdLineArgs::addTempFileOption();
   KateApp::addCmdLineOptions ();
 
   // get our command line args ;)
@@ -121,6 +123,22 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
       if (client.isApplicationRegistered(tryApp))
         kateApp = tryApp;
     }
+    else if ( args->isSet("start") )
+    {
+      for (unsigned int i=0; i < allClients.count(); i++)
+      {
+        if (allClients[i] == "kate" || allClients[i].left(5) == "kate-")
+        {
+          DCOPRef ref( allClients[i], "KateApplication" );
+          QString s = ref.call( "session" );
+          if ( QString(args->getOption("start")) == s )
+          {
+            kateApp = allClients[i];
+            break;
+          }
+        }
+      }
+    }
     else
     {
       for (unsigned int i=0; i < allClients.count(); ++i)
@@ -137,6 +155,8 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
     if (!kateApp.isEmpty())
     {
       kdDebug () << "kate app: " << kateApp << endl;
+      // make kdeinit happy
+      client.registerAs( "kate" );
 
       DCOPRef kRef (kateApp, "KateApplication");
 
@@ -145,8 +165,10 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
 
       QString enc = args->isSet("encoding") ? args->getOption("encoding") : QCString("");
 
+      bool tempfileSet = KCmdLineArgs::isTempFileSet();
+
       for (int z=0; z<args->count(); z++)
-        kRef.call( "openURL", args->url(z), enc );
+        kRef.call( "openURL", args->url(z), enc, tempfileSet );
 
       if( args->isSet( "stdin" ) )
       {
@@ -189,6 +211,24 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
       if (nav)
          kRef.call( "setCursor", line, column );
 
+      // since the user tried to open a document, let us assume [s]he
+      // wants to see that document.
+      // ### what to do about the infamous focus stealing prevention?
+      uint mwn = kRef.call("activeMainWindowNumber");
+      QCString smwn;
+      DCOPRef wRef( kateApp, QCString( "__KateMainWindow#") + smwn.setNum(mwn) );
+      if ( wRef.call("minimized") )
+      {
+        if ( wRef.call( "maximized" ) )
+          wRef.call( "maximize" );
+        else
+          wRef.call("restore");
+      }
+      wRef.call( "raise" );
+
+      // stop startup notification
+      KStartupInfo::appStarted(  );
+
       return 0;
     }
   }
@@ -198,8 +238,12 @@ extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
 
   // app execution should already end :)
   if (app.shouldExit())
+  {
     return 0;
+  }
 
   // execute ourself ;)
   return app.exec();
 }
+
+// kate: space-indent on; indent-width 2; replace-tabs on; mixed-indent off;
