@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "katepluginmanager.h"
@@ -24,6 +24,8 @@
 #include "kateapp.h"
 #include "katemainwindow.h"
 
+#include "../interfaces/application.h"
+
 #include <kconfig.h>
 #include <qstringlist.h>
 #include <kmessagebox.h>
@@ -34,7 +36,7 @@ KatePluginManager::KatePluginManager(QObject *parent) : QObject (parent)
 {
   m_pluginManager = new Kate::PluginManager (this);
   setupPluginList ();
-  
+
   loadConfig ();
   loadAllEnabledPlugins ();
 }
@@ -43,16 +45,14 @@ KatePluginManager::~KatePluginManager()
 {
   // first write config
   writeConfig ();
-  
+
   // than unload the plugins
   unloadAllPlugins ();
-  m_pluginList.setAutoDelete(true);
-  m_pluginList.clear();
 }
 
 KatePluginManager *KatePluginManager::self()
 {
-  return KateApp::self()->katePluginManager ();
+  return KateApp::self()->pluginManager ();
 }
 
 void KatePluginManager::setupPluginList ()
@@ -62,88 +62,93 @@ void KatePluginManager::setupPluginList ()
   for(KTrader::OfferList::Iterator it(traderList.begin()); it != traderList.end(); ++it)
   {
     KService::Ptr ptr = (*it);
-    
-    QString pVersion = ptr->property("X-Kate-Version").toString();
-    
-    if ((pVersion >= "2.2") && (pVersion <= KATE_VERSION))
-    {
-      KatePluginInfo *info = new KatePluginInfo;
-      
-      info->load = false;
-      info->service = ptr;
-      info->plugin = 0L;
 
-      m_pluginList.append(info);
+    QString pVersion = ptr->property("X-Kate-Version").toString();
+
+//    if ((pVersion >= "2.5") && (pVersion <= KateApp::kateVersion(false)))
+    if (pVersion == "2.5")
+    {
+      KatePluginInfo info;
+
+      info.load = false;
+      info.service = ptr;
+      info.plugin = 0L;
+
+      m_pluginList.push_back (info);
     }
   }
 }
 
 void KatePluginManager::loadConfig ()
 {
-  kapp->config()->setGroup("Kate Plugins");
+  KateApp::self()->config()->setGroup("Kate Plugins");
 
-  for (uint i=0; i<m_pluginList.count(); i++)
-    m_pluginList.at(i)->load =  kapp->config()->readBoolEntry(m_pluginList.at(i)->service->library(), false) ||
-	kapp->config()->readBoolEntry(m_pluginList.at(i)->service->property("X-Kate-PluginName").toString(),false);
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
+    m_pluginList[i].load =  KateApp::self()->config()->readBoolEntry (m_pluginList[i].service->library(), false) ||
+                            KateApp::self()->config()->readBoolEntry (m_pluginList[i].service->property("X-Kate-PluginName").toString(),false);
 }
 
 void KatePluginManager::writeConfig ()
 {
-  kapp->config()->setGroup("Kate Plugins");
+  KateApp::self()->config()->setGroup("Kate Plugins");
 
-  for (uint i=0; i<m_pluginList.count(); i++) {
-	KatePluginInfo *info=m_pluginList.at(i);
-	QString saveName=info->service->property("X-Kate-PluginName").toString();
-	if (saveName.isEmpty())
-		saveName=info->service->library();
-    	kapp->config()->writeEntry(saveName, m_pluginList.at(i)->load);
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
+  {
+    QString saveName=m_pluginList[i].service->property("X-Kate-PluginName").toString();
+
+    if (saveName.isEmpty())
+      saveName = m_pluginList[i].service->library();
+
+    KateApp::self()->config()->writeEntry (saveName, m_pluginList[i].load);
   }
 }
 
 void KatePluginManager::loadAllEnabledPlugins ()
 {
-  for (uint i=0; i<m_pluginList.count(); i++)
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
   {
-    if  (m_pluginList.at(i)->load)
-      loadPlugin (m_pluginList.at(i));
+    if  (m_pluginList[i].load)
+      loadPlugin (&m_pluginList[i]);
     else
-      unloadPlugin (m_pluginList.at(i)); 
+      unloadPlugin (&m_pluginList[i]);
   }
 }
 
 void KatePluginManager::unloadAllPlugins ()
 {
-  for (uint i=0; i<m_pluginList.count(); i++)
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
   {
-    if  (m_pluginList.at(i)->plugin)
-      unloadPlugin (m_pluginList.at(i));
+    if  (m_pluginList[i].plugin)
+      unloadPlugin (&m_pluginList[i]);
   }
 }
 
 void KatePluginManager::enableAllPluginsGUI (KateMainWindow *win)
 {
-  for (uint i=0; i<m_pluginList.count(); i++)
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
   {
-    if  (m_pluginList.at(i)->load)
-      enablePluginGUI (m_pluginList.at(i), win);
+    if  (m_pluginList[i].load)
+      enablePluginGUI (&m_pluginList[i], win);
   }
 }
 
 void KatePluginManager::disableAllPluginsGUI (KateMainWindow *win)
 {
-  for (uint i=0; i<m_pluginList.count(); i++)
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
   {
-    if  (m_pluginList.at(i)->load)
-      disablePluginGUI (m_pluginList.at(i), win);
+    if  (m_pluginList[i].load)
+      disablePluginGUI (&m_pluginList[i], win);
   }
 }
 
 void KatePluginManager::loadPlugin (KatePluginInfo *item)
 {
   QString pluginName=item->service->property("X-Kate-PluginName").toString();
+
   if (pluginName.isEmpty())
-       pluginName=item->service->library();
-  item->load = (item->plugin = Kate::createPlugin (QFile::encodeName(item->service->library()), Kate::application(),0,pluginName));
+    pluginName=item->service->library();
+
+  item->load = (item->plugin = Kate::createPlugin (QFile::encodeName(item->service->library()), Kate::application(), 0, pluginName));
 }
 
 void KatePluginManager::unloadPlugin (KatePluginInfo *item)
@@ -167,9 +172,9 @@ void KatePluginManager::enablePluginGUI (KatePluginInfo *item)
   if (!item->plugin) return;
   if (!Kate::pluginViewInterface(item->plugin)) return;
 
-  for (uint i=0; i< ((KateApp*)parent())->mainWindows(); i++)
+  for (uint i=0; i< KateApp::self()->mainWindows(); i++)
   {
-    Kate::pluginViewInterface(item->plugin)->addView(((KateApp*)parent())->mainWindow(i));
+    Kate::pluginViewInterface(item->plugin)->addView(KateApp::self()->mainWindow(i)->mainWindow());
   }
 }
 
@@ -186,17 +191,17 @@ void KatePluginManager::disablePluginGUI (KatePluginInfo *item)
   if (!item->plugin) return;
   if (!Kate::pluginViewInterface(item->plugin)) return;
 
-  for (uint i=0; i< ((KateApp*)parent())->mainWindows(); i++)
+  for (uint i=0; i< KateApp::self()->mainWindows(); i++)
   {
-    Kate::pluginViewInterface(item->plugin)->removeView(((KateApp*)parent())->mainWindow(i));
+    Kate::pluginViewInterface(item->plugin)->removeView(KateApp::self()->mainWindow(i)->mainWindow());
   }
 }
 
 Kate::Plugin *KatePluginManager::plugin(const QString &name)
 {
- for (uint i=0; i<m_pluginList.count(); i++)
+  for (unsigned int i=0; i < m_pluginList.size(); ++i)
   {
-    KatePluginInfo *info=m_pluginList.at(i);
+    KatePluginInfo *info = &m_pluginList[i];
     QString pluginName=info->service->property("X-Kate-PluginName").toString();
     if (pluginName.isEmpty())
        pluginName=info->service->library();
