@@ -45,9 +45,9 @@ static ModifierKey modifierKeys[] = {
       I18N_NOOP("The Shift key is now active."),
       I18N_NOOP("The Shift key is now inactive.") },
     { ControlMask, 0, "Control",
-      I18N_NOOP("The Control key has been locked and is now active for all of the following keypresses."),
-      I18N_NOOP("The Control key is now active."),
-      I18N_NOOP("The Control key is now inactive.") },
+      I18N_NOOP("The Ctrl key has been locked and is now active for all of the following keypresses."),
+      I18N_NOOP("The Ctrl key is now active."),
+      I18N_NOOP("The Ctrl key is now inactive.") },
     { 0, XK_Alt_L, "Alt",
       I18N_NOOP("The Alt key has been locked and is now active for all of the following keypresses."),
       I18N_NOOP("The Alt key is now active."),
@@ -69,9 +69,9 @@ static ModifierKey modifierKeys[] = {
       I18N_NOOP("The Hyper key is now active."),
       I18N_NOOP("The Hyper key is now inactive.") },
     { 0, 0, "Alt Graph",
-      I18N_NOOP("The Alt Graph key has been locked and is now active for all of the following keypresses."),
-      I18N_NOOP("The Alt Graph key is now active."),
-      I18N_NOOP("The Alt Graph key is now inactive.") },
+      I18N_NOOP("The Alt Gr key has been locked and is now active for all of the following keypresses."),
+      I18N_NOOP("The Alt Gr key is now active."),
+      I18N_NOOP("The Alt Gr key is now inactive.") },
     { 0, XK_Num_Lock, "Num Lock",
       I18N_NOOP("The Num Lock key has been activated."),
       "",
@@ -787,12 +787,12 @@ void KAccessApp::notifyChanges() {
    unsigned int disabled = features & ~requestedFeatures;
 
    if (enabled & XkbSlowKeysMask)
-      KNotifyClient::event (0, "slowkeys", i18n("Slow keys has been enabled. From now on, you need to press each key a certain time before it gets acceted."));
+      KNotifyClient::event (0, "slowkeys", i18n("Slow keys has been enabled. From now on, you need to press each key for a certain length of time before it is accepted."));
    else if (disabled & XkbSlowKeysMask)
       KNotifyClient::event (0, "slowkeys", i18n("Slow keys has been disabled."));
 
    if (enabled & XkbBounceKeysMask)
-      KNotifyClient::event (0, "bouncekeys", i18n("Bounce keys has been enabled. From now on, each key will be blocked a certain time after it was used."));
+      KNotifyClient::event (0, "bouncekeys", i18n("Bounce keys has been enabled. From now on, each key will be blocked for a certain length of time after it is used."));
    else if (disabled & XkbBounceKeysMask)
       KNotifyClient::event (0, "bouncekeys", i18n("Bounce keys has been disabled."));
 
