@@ -978,7 +978,7 @@ int KVirtualBGRenderer::hash()
         fp += m_renderer[i]->fingerprint();
     }
     int h = QHash(fp);
-    kdDebug() << k_funcinfo << " fp=\""<<fp<<"\" h="<<h<<endl;
+    //kdDebug() << k_funcinfo << " fp=\""<<fp<<"\" h="<<h<<endl;
     return QHash(fp);
 }
 
