@@ -1,4 +1,5 @@
 /* vi: ts=8 sts=4 sw=4
+ * kate: space-indent on; indent-width 4; indent-mode cstyle;
  *
  * This file is part of the KDE project, module kdesktop.
  * Copyright (C) 1999 Geert Jansen <g.t.jansen@stud.tue.nl>
@@ -21,6 +22,7 @@
 
 #include <dcopclient.h>
 #include <kapplication.h>
+#include <kdebug.h>
 #include <kglobalsettings.h>
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
@@ -28,27 +30,10 @@
 #include <konq_defaults.h>
 
 #include "bgdefaults.h"
+#include "bghash.h"
 #include "bgsettings.h"
 
 
-/*
- * QString -> int hash. From Qt's QGDict::hashKeyString().
- */
-
-static int QHash(QString key)
-{
-    int g, h = 0;
-    const QChar *p = key.unicode();
-    for (unsigned i=0; i < key.length(); i++) {
-        h = (h << 4) + p[i].cell();
-        if ((g = (h & 0xf0000000)))
-            h ^= (g >> 24);
-        h &= ~g;
-    }
-    return h;
-}
-
-
 /**** KBackgroundPattern ****/
 
 
@@ -430,12 +415,14 @@ QStringList KBackgroundProgram::list()
 /**** KBackgroundSettings ****/
 
 
-KBackgroundSettings::KBackgroundSettings(int desk, KConfig *config)
+KBackgroundSettings::KBackgroundSettings(int desk, int screen, bool drawBackgroundPerScreen, KConfig *config)
     : KBackgroundPattern(),
       KBackgroundProgram()
 {
     dirty = false; hashdirty = true;
+	m_bDrawBackgroundPerScreen = drawBackgroundPerScreen;
     m_Desk = desk;
+    m_Screen = screen;
     m_bEnabled = true;
 
     // Default values.
@@ -561,9 +548,11 @@ void KBackgroundSettings::copyConfig(const KBackgroundSettings *settings)
 }
 
 
-void KBackgroundSettings::load(int desk, bool reparseConfig)
+void KBackgroundSettings::load(int desk, int screen, bool drawBackgroundPerScreen, bool reparseConfig)
 {
     m_Desk = desk;
+    m_Screen = screen;	
+    m_bDrawBackgroundPerScreen = drawBackgroundPerScreen;
     readSettings(reparseConfig);
 }
 
@@ -676,7 +665,8 @@ void KBackgroundSettings::setWallpaperList(QStringList list)
     for(QStringList::ConstIterator it = list.begin();
         it != list.end(); ++it)
     {
-       m_WallpaperList.append(d->relativeLocation("wallpaper", *it));
+       QString rpath = d->relativeLocation("wallpaper", *it);
+       m_WallpaperList.append( !rpath.isEmpty() ? rpath : *it );
     }
     updateWallpaperFiles();
     // Try to keep the current wallpaper
@@ -736,12 +726,20 @@ void KBackgroundSettings::setUseShm(bool use)
     m_bShm = use;
 }
 
+QString KBackgroundSettings::configGroupName() const
+{
+    QString screenName;
+    if (m_bDrawBackgroundPerScreen)
+        screenName = QString("Screen%1").arg(QString::number(m_Screen));
+    return QString("Desktop%1%2").arg(m_Desk).arg(screenName);
+}
+
 void KBackgroundSettings::readSettings(bool reparse)
 {
     if (reparse)
         m_pConfig->reparseConfiguration();
 
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
+    m_pConfig->setGroup(configGroupName());
 
     // Background mode (Flat, div. Gradients, Pattern or Program)
     m_ColorA = m_pConfig->readColorEntry("Color1", &defColorA);
@@ -783,7 +781,6 @@ void KBackgroundSettings::readSettings(bool reparse)
 
     // Multiple wallpaper config
     m_WallpaperList = m_pConfig->readPathListEntry("WallpaperList");
-    updateWallpaperFiles();
 
     m_Interval = m_pConfig->readNumEntry("ChangeInterval", 60);
     m_LastChange = m_pConfig->readNumEntry("LastChange", 0);
@@ -796,6 +793,8 @@ void KBackgroundSettings::readSettings(bool reparse)
 	m_MultiMode = mode;
     }
 
+    updateWallpaperFiles();
+
     // Wallpaper mode (NoWallpaper, div. tilings)
     m_WallpaperMode = defWallpaperMode;
     m_Wallpaper = m_pConfig->readPathEntry("Wallpaper");
@@ -823,7 +822,7 @@ void KBackgroundSettings::writeSettings()
     if (!dirty)
         return;
 
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
+    m_pConfig->setGroup(configGroupName());
     m_pConfig->writeEntry("Color1", m_ColorA);
     m_pConfig->writeEntry("Color2", m_ColorB);
     m_pConfig->writePathEntry("Pattern", KBackgroundPattern::name());
@@ -927,8 +926,11 @@ QStringList KBackgroundSettings::wallpaperFiles() const
  */
 void KBackgroundSettings::changeWallpaper(bool init)
 {
-    if (m_WallpaperFiles.count() == 0)
+    if (m_WallpaperFiles.count() == 0) {
+        if( init )
+	    m_CurrentWallpaper = 0;
         return;
+    }
 
     switch (m_MultiMode) {
     case InOrder:
@@ -951,7 +953,7 @@ void KBackgroundSettings::changeWallpaper(bool init)
     }
 
     m_LastChange = (int) time(0L);
-    m_pConfig->setGroup(QString("Desktop%1").arg(m_Desk));
+    m_pConfig->setGroup(configGroupName());
     m_pConfig->writeEntry("CurrentWallpaper", m_CurrentWallpaper);
     m_pConfig->writeEntry("LastChange", m_LastChange);
     m_pConfig->sync();
@@ -1100,12 +1102,42 @@ void KGlobalBackgroundSettings::setLimitCache(bool limit)
 }
 
 
-void KGlobalBackgroundSettings::setCommonBackground(bool common)
+bool KGlobalBackgroundSettings::drawBackgroundPerScreen(int desk) const
 {
-    if (common == m_bCommon)
+    if ( desk > int(m_bDrawBackgroundPerScreen.size()) )
+        return _defDrawBackgroundPerScreen;
+    return m_bDrawBackgroundPerScreen[desk];
+}
+
+
+void KGlobalBackgroundSettings::setDrawBackgroundPerScreen(int desk, bool perScreen)
+{
+    if ( desk >= int(m_bDrawBackgroundPerScreen.size()) )
+        return;
+    
+    if ( m_bDrawBackgroundPerScreen[desk] == perScreen )
+        return;
+    
+    dirty = true;
+    m_bDrawBackgroundPerScreen[desk] = perScreen;
+}
+
+
+void KGlobalBackgroundSettings::setCommonScreenBackground(bool common)
+{
+    if (common == m_bCommonScreen)
 	return;
     dirty = true;
-    m_bCommon = common;
+    m_bCommonScreen = common;
+}
+
+
+void KGlobalBackgroundSettings::setCommonDeskBackground(bool common)
+{
+    if (common == m_bCommonDesk)
+	return;
+    dirty = true;
+    m_bCommonDesk = common;
 }
 
 
@@ -1169,11 +1201,18 @@ void KGlobalBackgroundSettings::setTextWidth(int width)
 void KGlobalBackgroundSettings::readSettings()
 {
     m_pConfig->setGroup("Background Common");
-    m_bCommon = m_pConfig->readBoolEntry("CommonDesktop", _defCommon);
+    m_bCommonScreen = m_pConfig->readBoolEntry("CommonScreen", _defCommonScreen);
+    m_bCommonDesk = m_pConfig->readBoolEntry("CommonDesktop", _defCommonDesk);
     m_bDock = m_pConfig->readBoolEntry("Dock", _defDock);
     m_bExport = m_pConfig->readBoolEntry("Export", _defExport);
     m_bLimitCache = m_pConfig->readBoolEntry("LimitCache", _defLimitCache);
     m_CacheSize = m_pConfig->readNumEntry("CacheSize", _defCacheSize);
+    
+    m_Names.clear();
+    NETRootInfo info( qt_xdisplay(), NET::DesktopNames | NET::NumberOfDesktops );
+    m_bDrawBackgroundPerScreen.resize(info.numberOfDesktops());
+    for ( int i = 0 ; i < info.numberOfDesktops() ; ++i )
+        m_bDrawBackgroundPerScreen[i] = m_pConfig->readBoolEntry( QString("DrawBackgroundPerScreen_%1").arg(i), _defDrawBackgroundPerScreen );
 
     m_TextColor = KGlobalSettings::textColor();
     m_pConfig->setGroup("FMSettings");
@@ -1183,8 +1222,6 @@ void KGlobalBackgroundSettings::readSettings()
     m_textLines = m_pConfig->readNumEntry("TextHeight", DEFAULT_TEXTHEIGHT);
     m_textWidth = m_pConfig->readNumEntry("TextWidth", DEFAULT_TEXTWIDTH);
 
-    m_Names.clear();
-    NETRootInfo info( qt_xdisplay(), NET::DesktopNames | NET::NumberOfDesktops );
     for ( int i = 0 ; i < info.numberOfDesktops() ; ++i )
       m_Names.append( QString::fromUtf8(info.desktopName(i+1)) );
 
@@ -1197,18 +1234,23 @@ void KGlobalBackgroundSettings::writeSettings()
         return;
 
     m_pConfig->setGroup("Background Common");
-    m_pConfig->writeEntry("CommonDesktop", m_bCommon);
+    m_pConfig->writeEntry("CommonScreen", m_bCommonScreen);
+    m_pConfig->writeEntry("CommonDesktop", m_bCommonDesk);
     m_pConfig->writeEntry("Dock", m_bDock);
     m_pConfig->writeEntry("Export", m_bExport);
     m_pConfig->writeEntry("LimitCache", m_bLimitCache);
     m_pConfig->writeEntry("CacheSize", m_CacheSize);
 
+    for ( unsigned i = 0 ; i < m_bDrawBackgroundPerScreen.size() ; ++i )
+        m_pConfig->writeEntry(QString("DrawBackgroundPerScreen_%1").arg(i), m_bDrawBackgroundPerScreen[i] );
+
     m_pConfig->setGroup("FMSettings");
     m_pConfig->writeEntry("NormalTextColor", m_TextColor);
     m_pConfig->writeEntry("ItemTextBackground", m_TextBackgroundColor);
     m_pConfig->writeEntry("ShadowEnabled", m_shadowEnabled);
     m_pConfig->writeEntry("TextHeight", m_textLines);
     m_pConfig->writeEntry("TextWidth", m_textWidth);
+    
     m_pConfig->sync();
     dirty = false;
 
