@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "kxftconfig.h"
@@ -29,11 +29,14 @@
 #include <klocale.h>
 #include <klargefile.h>
 #include <qdir.h>
+#include <qsettings.h>
+#include <qfont.h>
 
 #ifdef HAVE_FONTCONFIG
 #include <stdarg.h>
 #include <stdio.h>
 #include <fontconfig/fontconfig.h>
+#include <kdebug.h>
 #endif
 
 using namespace std;
@@ -156,6 +159,63 @@ static time_t getTimeStamp(const QString &item)
 }
 
 #ifdef HAVE_FONTCONFIG
+
+inline QString fileSyntax(const QString &f) { return xDirSyntax(f); }
+//
+// Obtain location of config file to use.
+//
+// For system, prefer the following:
+//
+//     <...>/config.d/00kde   = preferred method from FontConfig >= 2.3
+//     <...>/local.conf
+//
+// Non-system, prefer:
+//
+//     $HOME/<...>/.fonts.conf
+//     $HOME/<...>/fonts.conf
+//
+QString getConfigFile(bool system)
+{
+#if (FC_VERSION>=20300)
+    static const char * constKdeRootFcFile="00kde";
+#endif
+
+    FcStrList   *list=FcConfigGetConfigFiles(FcConfigGetCurrent());
+    QStringList files;
+    FcChar8     *file;
+    QString     home(dirSyntax(QDir::homeDirPath()));
+
+    while((file=FcStrListNext(list)))
+    {
+        QString f((const char *)file);
+
+        if(fExists(f))
+        {
+            if(system || 0==fileSyntax(f).find(home)) // For nonsystem, only consider file within $HOME
+                files.append(f);
+        }
+#if (FC_VERSION>=20300)
+        else if(system && dExists(f) && -1!=f.find(QRegExp("/conf\\.d/?$")))
+            return dirSyntax(f)+constKdeRootFcFile;   // This ones good enough for me!
+#endif
+    }
+
+    //
+    // Go through list of files, looking for the preferred one...
+    if(files.count())
+    {
+        QStringList::Iterator it(files.begin()),
+                              end(files.end());
+
+        for(; it!=end; ++it)
+            if(-1!=(*it).find(QRegExp(system ? "/local\\.conf$" : "/\\.?fonts\\.conf$")))
+                return *it;
+        return files.front();  // Just return the 1st one...
+    }
+    else
+        return system ? "/etc/fonts/local.conf" : fileSyntax(home+"/.fonts.conf"); // Hmmm... no known files?
+}
+
 static QString getEntry(QDomElement element, const char *type, unsigned int numAttributes, ...)
 {
     if(numAttributes==element.attributes().length())
@@ -196,9 +256,7 @@ static QString getEntry(QDomElement element, const char *type, unsigned int numA
 
 static KXftConfig::SubPixel::Type strToType(const char *str)
 {
-    if(0==strcmp(str, "\0"))
-        return KXftConfig::SubPixel::Greyscale;
-    else if(0==strcmp(str, "rgb"))
+    if(0==strcmp(str, "rgb"))
         return KXftConfig::SubPixel::Rgb;
     else if(0==strcmp(str, "bgr")) 
         return KXftConfig::SubPixel::Bgr;
@@ -225,9 +283,7 @@ static KXftConfig::Hint::Style strToStyle(const char *str)
 #else
 static bool strToType(const char *str, KXftConfig::SubPixel::Type &type)
 {   
-    if(0==memcmp(str, "\0", 1))
-        type=KXftConfig::SubPixel::Greyscale;
-    else if(0==memcmp(str, "rgb", 3))
+    if(0==memcmp(str, "rgb", 3))
         type=KXftConfig::SubPixel::Rgb;
     else if(0==memcmp(str, "bgr", 3))
         type=KXftConfig::SubPixel::Bgr;
@@ -344,16 +400,11 @@ static KXftConfig::ListItem * getLastItem(QPtrList<KXftConfig::ListItem> &list)
     return NULL;
 }
 
-#ifdef HAVE_FONTCONFIG
-static const QString defaultPath("/etc/fonts/local.conf");
-static const QString defaultUserFile(".fonts.conf");
-#else
+#ifndef HAVE_FONTCONFIG
 static const QString defaultPath("/usr/X11R6/lib/X11/XftConfig");
 static const QString defaultUserFile(".xftconfig");
 static const char *  constSymEnc="\"glyphs-fontspecific\"";
-#endif
 
-#ifndef HAVE_FONTCONFIG
 static const QString constConfigFiles[]=
 {
     defaultPath,
@@ -373,11 +424,13 @@ KXftConfig::KXftConfig(int required, bool system)
 #endif
             m_system(system)
 {
+#ifdef HAVE_FONTCONFIG
+    m_file=getConfigFile(system);
+    kdDebug(1208) << "Using fontconfig file:" << m_file << endl;
+    m_antiAliasing = aliasingEnabled();
+#else
     if(system) 
     {
-#ifdef HAVE_FONTCONFIG
-        m_file=defaultPath;
-#else
         int f;
 
         for(f=0; !constConfigFiles[f].isNull(); ++f)
@@ -386,13 +439,10 @@ KXftConfig::KXftConfig(int required, bool system)
 
         if(m_file.isNull())
             m_file=defaultPath;
-#endif
     }
     else
-    {
         m_file= QString(QDir::homeDirPath()+"/"+defaultUserFile);
-    }
-
+#endif
 #ifndef HAVE_FONTCONFIG
     m_symbolFamilies.setAutoDelete(true);
 #endif
@@ -530,6 +580,7 @@ bool KXftConfig::apply()
 #ifdef HAVE_FONTCONFIG
             if(m_required&HintStyle)
                 newConfig.setHintStyle(m_hint.style);
+            newConfig.setAntiAliasing(m_antiAliasing.set);
 #else
             if(m_required&SymbolFamilies)
             {
@@ -575,6 +626,7 @@ bool KXftConfig::apply()
                             applySubPixelType();
                         if(m_required&HintStyle)
                             applyHintStyle();
+                        applyAntiAliasing();
                         if(m_required&ExcludeRange)
                         {
                             applyExcludeRange(false);
@@ -583,10 +635,10 @@ bool KXftConfig::apply()
     
                         //
                         // Check document syntax...
-                        static const char * qtXmlHeader   = "<?xml version = '1.0'?>";
-                        static const char * xmlHeader     = "<?xml version=\"1.0\"?>";
-                        static const char * qtDocTypeLine = "<!DOCTYPE fontconfig>";
-                        static const char * docTypeLine   = "<!DOCTYPE fontconfig SYSTEM \"fonts.dtd\">";
+                        const char qtXmlHeader[]   = "<?xml version = '1.0'?>";
+                        const char xmlHeader[]     = "<?xml version=\"1.0\"?>";
+                        const char qtDocTypeLine[] = "<!DOCTYPE fontconfig>";
+                        const char docTypeLine[]   = "<!DOCTYPE fontconfig SYSTEM \"fonts.dtd\">";
     
                         QString str(m_doc.toString());
                         int     idx;
@@ -723,21 +775,14 @@ bool KXftConfig::apply()
 
 bool KXftConfig::getSubPixelType(SubPixel::Type &type)
 {
-    if(SubPixel::None!=m_subPixel.type && !m_subPixel.toBeRemoved)
-    {
-        type=m_subPixel.type;
-        return true;
-    }
-    else
-        return false;
+    type=m_subPixel.type;
+    return SubPixel::None!=m_subPixel.type;
 }
 
 void KXftConfig::setSubPixelType(SubPixel::Type type)
 {
-    if((SubPixel::None==type && SubPixel::None!=m_subPixel.type && !m_subPixel.toBeRemoved) ||
-       (SubPixel::None!=type && (type!=m_subPixel.type || m_subPixel.toBeRemoved)) )
+    if(type!=m_subPixel.type)
     {
-        m_subPixel.toBeRemoved=(SubPixel::None==type);
         m_subPixel.type=type;
         m_madeChanges=true;
     }
@@ -827,8 +872,6 @@ QString KXftConfig::description(SubPixel::Type t)
         default:
         case SubPixel::None:
             return i18n("None");
-        case SubPixel::Greyscale:
-            return i18n("Grayscale");
         case SubPixel::Rgb:
             return i18n("RGB");
         case SubPixel::Bgr:
@@ -847,8 +890,6 @@ const char * KXftConfig::toStr(SubPixel::Type t)
         default:
         case SubPixel::None:
             return "none";
-        case SubPixel::Greyscale:
-            return "";
         case SubPixel::Rgb:
             return "rgb";
         case SubPixel::Bgr:
@@ -1015,6 +1056,11 @@ void KXftConfig::readContents()
                                     m_hinting.node=n;
                                     m_hinting.set=str.lower()!="false";
                                 }
+                                else if(!(str=getEntry(ene, "bool", 2, "name", "antialias", "mode", "assign")).isNull())
+                                {
+                                    m_antiAliasing.node=n;
+                                    m_antiAliasing.set=str.lower()!="false";
+                                }
                         }
                         break;
                     case 3:
@@ -1035,13 +1081,23 @@ void KXftConfig::readContents()
                                 if(!ene.isNull())
                                     if("test"==ene.tagName())
                                     {
+                                        // kcmfonts used to write incorrectly more or less instead of more_eq and less_eq, so read both,
+                                        // first the old (wrong) one then the right one
                                         if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "size", "compare", "more")).isNull())
                                             from=str.toDouble();
-                                        else if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "size", "compare", "less")).isNull())
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "size", "compare", "more_eq")).isNull())
+                                            from=str.toDouble();
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "size", "compare", "less")).isNull())
                                             to=str.toDouble();
-                                        else if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "more")).isNull())
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "size", "compare", "less_eq")).isNull())
+                                            to=str.toDouble();
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "more")).isNull())
                                             pixelFrom=str.toDouble();
-                                        else if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "less")).isNull())
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "more_eq")).isNull())
+                                            pixelFrom=str.toDouble();
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "less")).isNull())
+                                            pixelTo=str.toDouble();
+                                        if(!(str=getEntry(ene, "double", 3, "qual", "any", "name", "pixelsize", "compare", "less_eq")).isNull())
                                             pixelTo=str.toDouble();
                                     }
                                     else if("edit"==ene.tagName() && "false"==getEntry(ene, "bool", 2, "name", "antialias", "mode", "assign"))
@@ -1219,33 +1275,22 @@ void KXftConfig::applyDirs()
 
 void KXftConfig::applySubPixelType()
 {
-    if(SubPixel::None==m_subPixel.type || m_subPixel.toBeRemoved)
-    {
-        if(!m_subPixel.node.isNull())
-        {
-            m_doc.documentElement().removeChild(m_subPixel.node);
-            m_subPixel.node.clear();
-        }
-    }
-    else
-    {
-        QDomElement matchNode = m_doc.createElement("match"),
-                    typeNode  = m_doc.createElement("const"),
-                    editNode  = m_doc.createElement("edit");
-        QDomText    typeText  = m_doc.createTextNode(toStr(m_subPixel.type));
+    QDomElement matchNode = m_doc.createElement("match"),
+                typeNode  = m_doc.createElement("const"),
+                editNode  = m_doc.createElement("edit");
+    QDomText    typeText  = m_doc.createTextNode(toStr(m_subPixel.type));
 
-        matchNode.setAttribute("target", "font");
-        editNode.setAttribute("mode", "assign");
-        editNode.setAttribute("name", "rgba");
-        editNode.appendChild(typeNode);
-        typeNode.appendChild(typeText);
-        matchNode.appendChild(editNode);
-        if(m_subPixel.node.isNull())
-            m_doc.documentElement().appendChild(matchNode);
-        else
-            m_doc.documentElement().replaceChild(matchNode, m_subPixel.node);
-        m_subPixel.node=matchNode;
-    }
+    matchNode.setAttribute("target", "font");
+    editNode.setAttribute("mode", "assign");
+    editNode.setAttribute("name", "rgba");
+    editNode.appendChild(typeNode);
+    typeNode.appendChild(typeText);
+    matchNode.appendChild(editNode);
+    if(m_subPixel.node.isNull())
+        m_doc.documentElement().appendChild(matchNode);
+    else
+        m_doc.documentElement().replaceChild(matchNode, m_subPixel.node);
+    m_subPixel.node=matchNode;
 }
 
 void KXftConfig::applyHintStyle()
@@ -1335,12 +1380,12 @@ void KXftConfig::applyExcludeRange(bool pixel)
         matchNode.setAttribute("target", "font");   // CPD: Is target "font" or "pattern" ????
         fromTestNode.setAttribute("qual", "any");
         fromTestNode.setAttribute("name", pixel ? "pixelsize" : "size");
-        fromTestNode.setAttribute("compare", "more");
+        fromTestNode.setAttribute("compare", "more_eq");
         fromTestNode.appendChild(fromNode);
         fromNode.appendChild(fromText);
         toTestNode.setAttribute("qual", "any");
         toTestNode.setAttribute("name", pixel ? "pixelsize" : "size");
-        toTestNode.setAttribute("compare", "less");
+        toTestNode.setAttribute("compare", "less_eq");
         toTestNode.appendChild(toNode);
         toNode.appendChild(toText);
         editNode.setAttribute("mode", "assign");
@@ -1400,7 +1445,7 @@ void KXftConfig::outputNewSymbolFamilies(std::ofstream &f)
 
 void KXftConfig::outputSubPixelType(std::ofstream &f, bool ifNew)
 {
-    if(!m_subPixel.toBeRemoved && ((ifNew && NULL==m_subPixel.end) || (!ifNew && NULL!=m_subPixel.end)) && SubPixel::None!=m_subPixel.type)
+    if((ifNew && NULL==m_subPixel.end) || (!ifNew && NULL!=m_subPixel.end))
         f << "match edit rgba = " << toStr(m_subPixel.type) << ';' << endl;
 }
 
@@ -1424,4 +1469,75 @@ void KXftConfig::outputExcludeRange(std::ofstream &f, bool ifNew, bool pixel)
         f << range.to << " edit antialias = false;" << endl;
 }
 }
+
+#endif
+
+#ifdef HAVE_FONTCONFIG
+bool KXftConfig::getAntiAliasing() const
+{
+    return m_antiAliasing.set;
+}
+
+void KXftConfig::setAntiAliasing( bool set )
+{
+    if ((set && !m_antiAliasing.set) || (!set && m_antiAliasing.set))
+    { 
+        m_antiAliasing.set = set; 
+        m_madeChanges = true; 
+    } 
+}
+
+void KXftConfig::applyAntiAliasing()
+{
+    QDomElement matchNode = m_doc.createElement("match"),
+                typeNode  = m_doc.createElement("bool"),
+                editNode  = m_doc.createElement("edit");
+    QDomText    typeText  = m_doc.createTextNode(m_antiAliasing.set ? "true" : "false");
+
+    matchNode.setAttribute("target", "font");
+    editNode.setAttribute("mode", "assign");
+    editNode.setAttribute("name", "antialias");
+    editNode.appendChild(typeNode);
+    typeNode.appendChild(typeText);
+    matchNode.appendChild(editNode);
+    if(m_antiAliasing.node.isNull())
+        m_doc.documentElement().appendChild(matchNode);
+    else
+        m_doc.documentElement().replaceChild(matchNode, m_antiAliasing.node);
+    m_antiAliasing.node=matchNode;
+}
+
+// KXftConfig only parses one config file, user's .fonts.conf usually.
+// If that one doesn't exist, then KXftConfig doesn't know if antialiasing
+// is enabled or not. So try to find out the default value from the default font.
+// Maybe there's a better way *shrug*.
+bool KXftConfig::aliasingEnabled()
+{
+    FcPattern *pattern = FcPatternCreate();
+    FcConfigSubstitute(0, pattern, FcMatchPattern);
+    FcDefaultSubstitute(pattern);
+    FcResult result;
+    FcPattern *f = FcFontMatch( 0, pattern, &result );
+    FcBool antialiased = FcTrue;
+    FcPatternGetBool( f, FC_ANTIALIAS, 0, &antialiased );
+    FcPatternDestroy( f );
+    FcPatternDestroy( pattern );
+    return antialiased == FcTrue;
+}
+
+#else
+
+void KXftConfig::setAntiAliasing( bool set )
+{
+  QSettings().writeEntry("/qt/useXft", set);
+  if (set)
+    QSettings().writeEntry("/qt/enableXft", set);
+}
+
+bool KXftConfig::getAntiAliasing() const
+{
+  return QSettings().readBoolEntry("/qt/useXft");
+}
+
+
 #endif
