@@ -1,6 +1,4 @@
 /*
- * $Id: info_fbsd.cpp,v 1.27 2003/01/25 14:31:11 augustin Exp $
- *
  * info_fbsd.cpp is part of the KDE program kcminfo.  This displays
  * various information about the system (hopefully a FreeBSD system)
  * it's running on.
@@ -228,18 +226,17 @@ bool GetInfo_PCI (QListView *lbox)
 {
 	FILE *pipe;
 	QFile *pcicontrol;
-	QTextStream *t;
 	QString s, cmd;
 	QListViewItem *olditem = 0;
 
-	pcicontrol = new QFile("/usr/X11R6/bin/scanpci");
+	pcicontrol = new QFile("/usr/sbin/pciconf");
 
 	if (!pcicontrol->exists()) {
 		delete pcicontrol;
-		pcicontrol = new QFile("/usr/X11R6/bin/pcitweak");
+		pcicontrol = new QFile("/usr/X11R6/bin/scanpci");
 		if (!pcicontrol->exists()) {
 			delete pcicontrol;
-			pcicontrol = new QFile("/usr/sbin/pciconf");
+			pcicontrol = new QFile("/usr/X11R6/bin/pcitweak");
 			if (!pcicontrol->exists()) {
 				QString s;
 				s = i18n("Could not find any programs with which to query your system's PCI information");
@@ -247,13 +244,13 @@ bool GetInfo_PCI (QListView *lbox)
 				delete pcicontrol;
 				return true;
 			} else {
-				cmd = "/usr/sbin/pciconf -l";
+				cmd = "/usr/X11R6/bin/pcitweak -l 2>&1";
 			}
 		} else {
-			cmd = "/usr/X11R6/bin/pcitweak -l 2>&1";
+			cmd = "/usr/X11R6/bin/scanpci";
 		}
 	} else {
-		cmd = "/usr/X11R6/bin/scanpci";
+		cmd = "/usr/sbin/pciconf -l -v 2>&1";
 	}
 	delete pcicontrol;
 
@@ -265,17 +262,8 @@ bool GetInfo_PCI (QListView *lbox)
 		/* This prints out a list of all the pci devies, perhaps eventually we could
 		   parse it as opposed to schlepping it into a listbox */
 
-		t = new QTextStream(pipe, IO_ReadOnly);
-
-		while (true) {
-			s = t->readLine();
-			if ( s.isEmpty() )
-				break;
-			olditem = new QListViewItem(lbox, olditem, s);
-		}
-
-		delete t;
 		pclose(pipe);
+		GetInfo_ReadfromPipe(lbox, cmd.latin1(), true);
 	}
 
 	if (!lbox->childCount()) {
