@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef KPROXY_DIALOG_BASE_H
@@ -29,7 +29,6 @@
 #include <kdialogbase.h>
 #include <kprotocolmanager.h>
 
-
 class KProxyData
 {
 public:
@@ -63,6 +62,7 @@ public:
   virtual void setProxyData (const KProxyData&)=0;
 
 protected:
+  void setHighLight (QWidget* widget = 0, bool highlight = false);
   bool m_bHasValidData;
 };
 #endif
