@@ -1,19 +1,19 @@
 // -*- Mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4; -*-
 /*  Copyright (C) 2003 Lukas Tinkl <lukas@kde.org>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "ktheme.h"
@@ -41,9 +41,10 @@
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
 #include <ktar.h>
+#include <kstyle.h>
 
-                             KTheme::KTheme( QWidget *parent, const QString & xmlFile )
-                                 : m_parent(parent)
+KTheme::KTheme( QWidget *parent, const QString & xmlFile )
+	: m_parent(parent)
 {
     QFile file( xmlFile );
     file.open( IO_ReadOnly );
@@ -186,7 +187,7 @@ QString KTheme::createYourself( bool pack )
     // 3. Icons
     globalConf->setGroup( "Icons" );
     QDomElement iconElem = m_dom.createElement( "icons" );
-    iconElem.setAttribute( "name", globalConf->readEntry( "Theme" ) );
+    iconElem.setAttribute( "name", globalConf->readEntry( "Theme",KIconTheme::current() ) );
     createIconElems( "DesktopIcons", "desktop", iconElem, globalConf );
     createIconElems( "MainToolbarIcons", "mainToolbar", iconElem, globalConf );
     createIconElems( "PanelIcons", "panel", iconElem, globalConf );
@@ -246,10 +247,12 @@ QString KTheme::createYourself( bool pack )
     m_root.appendChild( colorsElem );
 
     // 6. Cursors
-    globalConf->setGroup( "KDE" );
+    KConfig* mouseConf = new KConfig( "kcminputrc", true );
+    mouseConf->setGroup( "Mouse" );
     QDomElement cursorsElem = m_dom.createElement( "cursors" );
-    cursorsElem.setAttribute( "name", globalConf->readEntry( "cursorTheme" ) );
+    cursorsElem.setAttribute( "name", mouseConf->readEntry( "cursorTheme" ) );
     m_root.appendChild( cursorsElem );
+    delete mouseConf;
     // TODO copy the cursor theme?
 
     // 7. KWin
@@ -305,7 +308,7 @@ QString KTheme::createYourself( bool pack )
     // 10. Widget style
     globalConf->setGroup( "General" );
     QDomElement widgetsElem = m_dom.createElement( "widgets" );
-    widgetsElem.setAttribute( "name", globalConf->readEntry( "widgetStyle" ) );
+    widgetsElem.setAttribute( "name", globalConf->readEntry( "widgetStyle",KStyle::defaultStyle()  ) );
     m_root.appendChild( widgetsElem );
 
     // 12.
@@ -543,10 +546,9 @@ void KTheme::apply()
 
     if ( !cursorsElem.isNull() )
     {
-        KConfig * cursorConf = KGlobal::config();
-        cursorConf->setGroup( "KDE" );
-        cursorConf->writeEntry( "cursorTheme", cursorsElem.attribute( "name" ), true, true );
-        cursorConf->sync();
+        KConfig mouseConf( "kcminputrc" );
+        mouseConf.setGroup( "Mouse" );
+        mouseConf.writeEntry( "cursorTheme", cursorsElem.attribute( "name" ));
         // FIXME is there a way to notify KDE of cursor changes?
     }
 
