@@ -20,36 +20,37 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __KCMSTYLE_H
 #define __KCMSTYLE_H
 
-#include <qtimer.h>
 #include <qstring.h>
+#include <qtimer.h>
+
 #include <kcmodule.h>
 
 #include "stylepreview.h"
 #include "menupreview.h"
 
+class KComboBox;
 class QCheckBox;
 class QComboBox;
 class QFrame;
 class QGroupBox;
+class QLabel;
 class QListBox;
 class QListViewItem;
+class QSettings;
+class QSlider;
+class QSpacerItem;
 class QStyle;
 class QTabWidget;
 class QVBoxLayout;
-class WidgetPreview;
-class QSettings;
-class QSpacerItem;
-class QLabel;
-class QSlider;
-class KComboBox;
 class StyleConfigDialog;
+class WidgetPreview;
 
 struct StyleEntry {
 	QString name;
@@ -69,11 +70,10 @@ public:
 	virtual void load();
 	virtual void save();
 	virtual void defaults();
-	virtual QString quickHelp() const;
 
 protected:
 	bool findStyle( const QString& str, int& combobox_item );
-	void switchStyle(const QString& styleName);
+	void switchStyle(const QString& styleName, bool force = false);
 	void setStyleRecursive(QWidget* w, QStyle* s);
 
 	void loadStyle( KConfig& config );
@@ -120,7 +120,6 @@ private:
 	QPalette palette;
 
 	// Page2 widgets
-	QGroupBox* gbEffects;
 	QCheckBox* cbEnableEffects;
 
 	QFrame* containerFrame;
@@ -147,7 +146,6 @@ private:
 	QCheckBox* cbMenuShadow;
 
 	// Page3 widgets
-	QGroupBox* gbToolbarSettings;
 	QGroupBox* gbVisualAppearance;
 
 	QCheckBox* cbHoverButtons;
