@@ -66,8 +66,7 @@ bool TaskbarAppearance::matchesSettings() const
            TaskBarSettings::haloText() == m_haloText &&
            TaskBarSettings::showButtonOnHover() == m_showButtonOnHover;
 }
-#include <iostream>
-using namespace std;
+
 void TaskbarAppearance::alterSettings() const
 {
     TaskBarSettings::self()->setDrawButtons(m_drawButtons);
@@ -165,7 +164,7 @@ TaskbarConfig::TaskbarConfig(QWidget *parent, const char* name, const QStringLis
     if (!QApplication::desktop()->isVirtualDesktop() ||
         QApplication::desktop()->numScreens() == 1) // No Ximerama
     {
-        m_widget->kcfg_ShowCurrentScreenOnly->hide();
+        m_widget->showAllScreens->hide();
     }
 
     KAboutData *about = new KAboutData(I18N_NOOP("kcmtaskbar"),
@@ -242,10 +241,12 @@ void TaskbarConfig::load()
     KCModule::load();
     slotUpdateComboBox();
     updateAppearanceCombo();
+    m_widget->showAllScreens->setChecked(!TaskBarSettings::self()->showCurrentScreenOnly());
 }
 
 void TaskbarConfig::save()
 {
+    TaskBarSettings::self()->setShowCurrentScreenOnly(!m_widget->showAllScreens->isChecked());
     int selectedAppearance = m_widget->appearance->currentItem();
     if (selectedAppearance < m_appearances.count())
     {
