@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 
@@ -31,9 +31,11 @@
 #include <string.h>
 #include <unistd.h>
 #include <kcolordrag.h>
+#include <kurldrag.h>
 
 #include <qdir.h>
 #include <qevent.h>
+#include <qtooltip.h>
 
 #include <netwm.h>
 #include <dcopclient.h>
@@ -70,6 +72,7 @@
 KRootWidget::KRootWidget() : QObject()
 {
      kapp->desktop()->installEventFilter(this);
+     kapp->desktop()->setAcceptDrops( true );
 }
 
 bool KRootWidget::eventFilter ( QObject *, QEvent * e )
@@ -86,6 +89,43 @@ bool KRootWidget::eventFilter ( QObject *, QEvent * e )
        emit wheelRolled(we->delta());
        return true;
      }
+     else if ( e->type() == QEvent::DragEnter )
+     {
+       QDragEnterEvent* de = static_cast<QDragEnterEvent *>( e );
+       bool b = !KGlobal::config()->isImmutable();
+       b &= !KGlobal::dirs()->isRestrictedResource( "wallpaper" );
+
+       bool imageURL = false;
+       if ( KURLDrag::canDecode( de ) )
+       {
+         KURL::List list;
+         KURLDrag::decode( de, list );
+         KURL url = list.first();
+         KMimeType::Ptr mime = KMimeType::findByURL( url );
+         if ( !KImageIO::type( url.path() ).isEmpty() ||
+              KImageIO::isSupported( mime->name(), KImageIO::Reading ) || mime->is( "image/svg+xml" ) )
+           imageURL = true;
+       }
+
+       b &= KColorDrag::canDecode( de ) || QImageDrag::canDecode( de ) || imageURL;
+       de->accept( b );
+       return true;
+     }
+     else if ( e->type() == QEvent::Drop )
+     {
+       QDropEvent* de = static_cast<QDropEvent*>( e );
+       if ( KColorDrag::canDecode( de ) ) 
+         emit colorDropEvent( de );
+       else if ( QImageDrag::canDecode( de ) )
+         emit imageDropEvent( de );
+       else if ( KURLDrag::canDecode( de ) ) {
+	 KURL::List list;
+         KURLDrag::decode( de, list );
+         KURL url = list.first();
+         emit newWallpaper( url );
+       }
+       return true;
+     }
      return false; // Don't filter.
 }
 
@@ -102,7 +142,7 @@ KDesktop::KDesktop( bool x_root_hack, bool auto_start, bool wait_for_kded ) :
     DCOPObject( "KDesktopIface" ),
     QWidget( 0L, "desktop", WResizeNoErase | ( x_root_hack ? (WStyle_Customize | WStyle_NoBorder) : 0) ),
     // those two WStyle_ break kdesktop when the root-hack isn't used (no Dnd)
-   startup_id( NULL )
+   startup_id( NULL ), m_waitForKicker(0)
 {
   m_bAutoStart = auto_start;
   m_bWaitForKded = wait_for_kded;
@@ -116,11 +156,10 @@ KDesktop::KDesktop( bool x_root_hack, bool auto_start, bool wait_for_kded ) :
 
   setAcceptDrops(true); // WStyle_Customize seems to disable that
   m_pKwinmodule = new KWinModule( this );
-  updateWorkAreaTimer = new QTimer( this );
-  connect( updateWorkAreaTimer, SIGNAL( timeout() ),
-           this, SLOT( updateWorkArea() ) );
-  connect( m_pKwinmodule, SIGNAL( workAreaChanged() ),
-           this, SLOT( workAreaChanged() ) );
+
+  kapp->dcopClient()->setNotifications(true);
+  kapp->dcopClient()->connectDCOPSignal("kicker", "kicker", "desktopIconsAreaChanged(QRect, int)",
+                                        "KDesktopIface", "desktopIconsAreaChanged(QRect, int)", false);
 
   // Dont repaint on configuration changes during construction
   m_bInit = true;
@@ -188,12 +227,21 @@ KDesktop::initRoot()
      delete m_pIconView;
      m_pIconView = 0;
 
+     { // trigger creation of QToolTipManager, it does XSelectInput() on the root window
+     QWidget w;
+     QToolTip::add( &w, "foo" );
+     }
+     // NOTE: If mouse clicks stop working again, it's most probably something doing XSelectInput()
+     // on the root window after this, and setting it to some fixed value instead of adding its mask.
      XWindowAttributes attrs;
      XGetWindowAttributes(dpy, root, &attrs);
      XSelectInput(dpy, root, attrs.your_event_mask | ButtonPressMask);
 
      m_pRootWidget = new KRootWidget;
      connect(m_pRootWidget, SIGNAL(wheelRolled(int)), this, SLOT(slotSwitchDesktops(int)));
+     connect(m_pRootWidget, SIGNAL(colorDropEvent(QDropEvent*)), this, SLOT(handleColorDropEvent(QDropEvent*)) );
+     connect(m_pRootWidget, SIGNAL(imageDropEvent(QDropEvent*)), this, SLOT(handleImageDropEvent(QDropEvent*)) );
+     connect(m_pRootWidget, SIGNAL(newWallpaper(const KURL&)), this, SLOT(slotNewWallpaper(const KURL&)) );
 
      // Geert Jansen: backgroundmanager belongs here
      // TODO tell KBackgroundManager if we change widget()
@@ -242,17 +290,52 @@ KDesktop::initRoot()
      bgMgr->setExport(1);
      connect( bgMgr, SIGNAL( initDone()), SLOT( backgroundInitDone()));
 
-     //kdDebug(1204) << "KDesktop constructor -> workAreaChanged" << endl;
-     workAreaChanged();
+     // make sure it is initialized before we first call updateWorkArea()
+     m_pIconView->initConfig( m_bInit );
+
+     // set the size of the area for desktop icons placement
+     {
+        QByteArray data, result;
+        QDataStream arg(data, IO_WriteOnly);
+        arg << kdesktop_screen_number;
+        QCString replyType;
+        QRect area;
+
+        if ( kapp->dcopClient()->call("kicker", "kicker", "desktopIconsArea(int)",
+                                       data, replyType, result, false, 2000) )
+        {
+          QDataStream res(result, IO_ReadOnly);
+          res >> area;
+
+          m_pIconView->updateWorkArea(area);
+        }
+        else
+          if ( m_bInit )
+          {
+            // if we failed to get the information from kicker wait a little - probably
+            // this is the KDE startup and kicker is simply not running yet
+            m_waitForKicker = new QTimer(this);
+            connect(m_waitForKicker, SIGNAL(timeout()), this, SLOT(slotNoKicker()));
+            m_waitForKicker->start(15000, true);
+          }
+          else  // we are not called from the ctor, so kicker should already run
+          {
+            area = kwinModule()->workArea(kwinModule()->currentDesktop());
+            m_pIconView->updateWorkArea(area);
+          }
+     }
+
      if (!m_bInit)
      {
-        m_pIconView->initConfig( m_bInit );
         m_pIconView->start();
         delete KRootWm::self();
         KRootWm* krootwm = new KRootWm( this ); // handler for root menu (used by kdesktop on RMB click)
         keys->setSlot("Lock Session", krootwm, SLOT(slotLock()));
         keys->updateConnections();
      }
+   } else {
+     DCOPRef r( "ksmserver", "ksmserver" );
+     r.send( "resumeStartup" );
    }
 
    KWin::setType( winId(), NET::Desktop );
@@ -260,10 +343,20 @@ KDesktop::initRoot()
    KWin::setOnAllDesktops( winId(), true );
 }
 
+void KDesktop::slotNoKicker()
+{
+    kdDebug(1204) << "KDesktop::slotNoKicker ... kicker did not respond" << endl;
+    // up till now, we got no desktopIconsArea from kicker - probably
+    // it's not running, so use the area from KWinModule
+    m_pIconView->updateWorkArea(kwinModule()->workArea(kwinModule()->currentDesktop()));
+}
+
 void
 KDesktop::backgroundInitDone()
 {
     //kdDebug(1204) << "KDesktop::backgroundInitDone" << endl;
+    DCOPRef r( "ksmserver", "ksmserver" );
+    r.send( "resumeStartup" );
 
     // avoid flicker
     if (m_bDesktopEnabled)
@@ -354,7 +447,9 @@ KDesktop::slotStart()
 KDesktop::~KDesktop()
 {
   delete m_miniCli;
+  m_miniCli = 0; // see #120382
   delete bgMgr;
+  bgMgr = 0;
   delete startup_id;
 }
 
@@ -654,20 +749,13 @@ void KDesktop::closeEvent(QCloseEvent *e)
     e->ignore();
 }
 
-void KDesktop::workAreaChanged()
+void KDesktop::desktopIconsAreaChanged(const QRect &area, int screen)
 {
-    //kdDebug(1204) << "KDesktop::workAreaChanged() -> starting timer" << endl;
-    updateWorkAreaTimer->stop();
-    updateWorkAreaTimer->start( 100, TRUE );
-}
+    // hurra! kicker is alive
+    if ( m_waitForKicker ) m_waitForKicker->stop();
 
-void KDesktop::updateWorkArea()
-{
-    if (m_pIconView)
-    {
-        QRect wr( kwinModule()->workArea( kwinModule()->currentDesktop() ) );
-        m_pIconView->updateWorkArea( wr );
-    }
+    if ( (screen == kdesktop_screen_number) && m_pIconView )
+        m_pIconView->updateWorkArea(area);
 }
 
 void KDesktop::slotSwitchDesktops(int delta)
@@ -705,8 +793,9 @@ void KDesktop::handleColorDropEvent(QDropEvent * e)
 void KDesktop::handleImageDropEvent(QDropEvent * e)
 {
     KPopupMenu popup;
+    if ( m_pIconView )
     popup.insertItem(SmallIconSet("filesave"),i18n("&Save to Desktop..."), 1);
-    if (m_pIconView && m_pIconView->maySetWallpaper())
+    if ( ( m_pIconView && m_pIconView->maySetWallpaper() ) || m_pRootWidget )
        popup.insertItem(SmallIconSet("background"),i18n("Set as &Wallpaper"), 2);
     popup.insertSeparator();
     popup.insertItem(SmallIconSet("cancel"), i18n("&Cancel"));
@@ -758,10 +847,21 @@ void KDesktop::slotNewWallpaper(const KURL &url)
 {
     // This is called when a file containing an image is dropped
     // (called by KonqOperations)
-    QString tmpFile;
-    // We pass 0 as parent window because passing the desktop is not a good idea
-    KIO::NetAccess::download( url, tmpFile, 0 );
-    bgMgr->setWallpaper(tmpFile);
+    if ( url.isLocalFile() )
+        bgMgr->setWallpaper( url.path() );
+    else
+    {
+        // Figure out extension
+        QString fileName = url.fileName();
+        QFileInfo fileInfo( fileName );
+        QString ext = fileInfo.extension();
+        // Store tempfile in a place where it will still be available after a reboot
+        KTempFile tmpFile( KGlobal::dirs()->saveLocation("wallpaper"), "." + ext );
+        KURL localURL; localURL.setPath( tmpFile.name() );
+        // We pass 0 as parent window because passing the desktop is not a good idea
+        KIO::NetAccess::file_copy( url, localURL, -1, true /*overwrite*/ );
+        bgMgr->setWallpaper( localURL.path() );
+    }
 }
 
 // for dcop interface backward compatibility
@@ -842,7 +942,35 @@ void KDesktop::setIconsEnabled( bool enable )
 
 void KDesktop::desktopResized()
 {
-    resize( kapp->desktop()->size());
+    resize(kapp->desktop()->size());
+
+    if ( m_pIconView )
+    {
+        // the sequence of actions is important:
+        // remove all icons, resize desktop, tell kdiconview new iconsArea size
+        // tell kdiconview to reget all icons
+        m_pIconView->slotClear();
+        m_pIconView->resize(kapp->desktop()->size());
+
+        // get new desktopIconsArea from kicker
+        QByteArray data, result;
+        QDataStream arg(data, IO_WriteOnly);
+        arg << kdesktop_screen_number;
+        QCString replyType;
+        QRect area;
+
+        if ( kapp->dcopClient()->call("kicker", "kicker", "desktopIconsArea(int)",
+                                       data, replyType, result, false, 2000) )
+        {
+            QDataStream res(result, IO_ReadOnly);
+            res >> area;
+        }
+        else
+            area = kwinModule()->workArea(kwinModule()->currentDesktop());
+
+        m_pIconView->updateWorkArea(area);
+        m_pIconView->startDirLister();
+    }
 }
 
 void KDesktop::switchDesktops( int delta )
