@@ -24,8 +24,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <pwd.h>
@@ -44,6 +44,7 @@
 #include <qwhatsthis.h>
 #include <qstylesheet.h>
 
+#include <dcopclient.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
@@ -120,8 +121,6 @@ Minicli::Minicli( QWidget *parent, const char *name)
 
   m_dlg->leUsername->setText("root");
 
-  setMaxCommandBoxWidth();
-
   // Main widget buttons...
   connect( m_dlg->pbRun, SIGNAL(clicked()), this, SLOT(accept()) );
   connect( m_dlg->pbCancel, SIGNAL(clicked()), this, SLOT(reject()) );
@@ -162,9 +161,9 @@ void Minicli::setCommand(const QString& command)
   }
 }
 
-void Minicli::setMaxCommandBoxWidth()
+QSize Minicli::sizeHint() const
 {
-  int maxWidth = qApp->desktop()->screenGeometry(this).width();
+  int maxWidth = qApp->desktop()->screenGeometry((QWidget*)this).width();
   if (maxWidth < 603)
   {
     // a sensible max for smaller screens
@@ -175,19 +174,13 @@ void Minicli::setMaxCommandBoxWidth()
     maxWidth = maxWidth * 2 / 5;
   }
 
-  m_dlg->cbCommand->setMaximumWidth(maxWidth);
+  return QSize(maxWidth, -1);
 }
 
 void Minicli::show()
 {
-  setMaxCommandBoxWidth();
   KWin::setState( winId(), NET::StaysOnTop );
-  setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
-  adjustSize();
-  setFixedSize(size());
   KDialog::show();
-  adjustSize();
-  setFixedSize(size());
 }
 
 void Minicli::loadConfig()
@@ -481,6 +474,7 @@ int Minicli::runCommand()
     proc.setTerminal(true);
     proc.setErase(true);
     _exit(proc.exec(m_dlg->lePassword->password()));
+    return 0;
   }
   else
   {
@@ -521,6 +515,7 @@ int Minicli::runCommand()
             KService::Ptr service = KService::serviceByDesktopName(cmd);
             if (service && service->isValid() && service->type() == "Application")
             {
+              notifyServiceStarted(service);
               KRun::run(*service, KURL::List());
               return 0;
             }
@@ -554,6 +549,7 @@ int Minicli::runCommand()
           KService::Ptr service = KService::serviceByDesktopName(cmd);
           if (service && service->isValid() && service->type() == "Application")
           {
+            notifyServiceStarted(service);
             KRun::run(*service, KURL::List());
             return 0;
           }
@@ -561,6 +557,7 @@ int Minicli::runCommand()
           service = KService::serviceByName(cmd);
           if (service && service->isValid() && service->type() == "Application")
           {
+            notifyServiceStarted(service);
             KRun::run(*service, KURL::List());
             return 0;
           }
@@ -584,6 +581,18 @@ int Minicli::runCommand()
   }
 }
 
+void Minicli::notifyServiceStarted(KService::Ptr service)
+{
+    // Inform other applications (like the quickstarter applet)
+    // that an application was started
+    QByteArray params;
+    QDataStream stream(params, IO_WriteOnly);
+    stream << "minicli" << service->storageId();
+    kdDebug() << "minicli appLauncher dcop signal: " << service->storageId() << endl;
+    KApplication::kApplication()->dcopClient()->emitDCOPSignal("appLauncher",
+        "serviceStartedByStorageId(QString,QString)", params);
+}
+
 void Minicli::slotCmdChanged(const QString& text)
 {
   bool isEmpty = text.isEmpty();
@@ -863,9 +872,7 @@ QString Minicli::calculate(const QString &exp)
 
 void Minicli::fontChange( const QFont & )
 {
-   setMaxCommandBoxWidth();
    adjustSize();
-   setFixedSize(size());
 }
 
 // vim: set et ts=2 sts=2 sw=2:
