@@ -117,7 +117,7 @@ int main(int argc, char *argv[])
     {
         KStartupInfoId id;
         id.initId( kapp->startupId());
-        id.setupStartupEnv(); // make KDE_STARTUP_ENV env. var. available again
+        id.setupStartupEnv(); // make DESKTOP_STARTUP_ID env. var. available again
     }
 
     int result = startApp();
@@ -275,7 +275,7 @@ static int startApp()
 
     QCStringList env;
     QCString options;
-    env << ( "KDE_STARTUP_ENV=" + kapp->startupId());
+    env << ( "DESKTOP_STARTUP_ID=" + kapp->startupId());
     
     if (pw->pw_uid)
     {
@@ -297,7 +297,7 @@ static int startApp()
     
     if (!new_dcop)
     {
-        QCString ksycoca = "KDESYCOCA="+QFile::encodeName(locateLocal("tmp", "ksycoca"));
+        QCString ksycoca = "KDESYCOCA="+QFile::encodeName(locateLocal("cache", "ksycoca"));
         env << ksycoca;
 
         options += "xf"; // X-only, dcop forwarding enabled.
