@@ -18,7 +18,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -45,6 +45,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <unistd.h> // alarm()
 
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include <X11/cursorfont.h>
 
 extern "C" {
@@ -83,13 +85,21 @@ GreeterApp::timerEvent( QTimerEvent * )
 bool
 GreeterApp::x11EventFilter( XEvent * ev )
 {
+	KeySym sym;
+
 	switch (ev->type) {
 	case FocusIn:
 	case FocusOut:
 		// Hack to tell dialogs to take focus when the keyboard is grabbed
 		ev->xfocus.mode = NotifyNormal;
 		break;
+	case KeyPress:
+		sym = XLookupKeysym( &ev->xkey, 0 );
+		if (sym != XK_Return && !IsModifierKey( sym ))
+			emit activity();
+		break;
 	case ButtonPress:
+		emit activity();
 	case ButtonRelease:
 		// Hack to let the RMB work as LMB
 		if (ev->xbutton.button == 3)
@@ -167,12 +177,14 @@ kg_main( const char *argv0 )
 			if (os)
 				free( os );
 			GSendInt( G_Ready );
+			_autoLoginDelay = 0;
 			continue;
 		}
 
 		if (cmd == G_ErrorGreet) {
 			if (KGVerify::handleFailVerify( qApp->desktop()->screen( _greeterScreen ) ))
 				break;
+			_autoLoginDelay = 0;
 			cmd = G_Greet;
 		}
 
@@ -187,6 +199,9 @@ kg_main( const char *argv0 )
 		} else
 #endif
 		{
+			if ((cmd != G_GreetTimed && !_autoLoginAgain) ||
+			    _autoLoginUser.isEmpty())
+				_autoLoginDelay = 0;
 			if (_useTheme && !_theme.isEmpty()) {
 				KThemedGreeter *tgrt;
 				dialog = tgrt = new KThemedGreeter;
@@ -233,3 +248,5 @@ kg_main( const char *argv0 )
 }
 
 } // extern "C"
+
+#include "kgapp.moc"
