@@ -14,8 +14,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.	 If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.
 
 */
 
@@ -60,10 +60,15 @@ MyApplication::renderDone()
 {
 	desktop()->setBackgroundPixmap( *renderer.pixmap() );
 	desktop()->repaint( true );
-	if (renderer.backgroundMode() != KBackgroundSettings::Program &&
-	    (renderer.multiWallpaperMode() == KBackgroundSettings::NoMulti ||
-	     renderer.multiWallpaperMode() == KBackgroundSettings::NoMultiRandom))
-		quit();
+	for (unsigned i=0; i<renderer.numRenderers(); ++i)
+	{
+		KBackgroundRenderer * r = renderer.renderer(i);
+		if (r->backgroundMode() == KBackgroundSettings::Program ||
+		    (r->multiWallpaperMode() != KBackgroundSettings::NoMulti &&
+		     r->multiWallpaperMode() != KBackgroundSettings::NoMultiRandom))
+			return;
+	}
+	quit();
 }
 
 void
@@ -71,10 +76,8 @@ MyApplication::slotTimeout()
 {
 	bool change = false;
 
-	if ((renderer.backgroundMode() == KBackgroundSettings::Program) &&
-	    (renderer.KBackgroundProgram::needUpdate()))
-	{
-		renderer.KBackgroundProgram::update();
+	if (renderer.needProgramUpdate()) {
+		renderer.programUpdate();
 		change = true;
 	}
 
