@@ -23,8 +23,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "kgreet_winbind.h"
+#include "themer/kdmthemer.h"
+#include "themer/kdmitem.h"
 
 #include <klocale.h>
+#include <kdebug.h>
 #include <kcombobox.h>
 #include <klineedit.h>
 #include <kpassdlg.h>
@@ -34,6 +37,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <qlayout.h>
 #include <qlabel.h>
 
+#include <stdlib.h>
+
 class KDMPasswordEdit : public KPasswordEdit {
 public:
 	KDMPasswordEdit( QWidget *parent ) : KPasswordEdit( parent, 0 ) {}
@@ -58,6 +63,7 @@ splitEntity( const QString &ent, QString &dom, QString &usr )
 }
 
 KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
+                                  KdmThemer *themer,
                                   QWidget *parent, QWidget *pred,
                                   const QString &_fixedEntity,
                                   Function _func, Context _ctx ) :
@@ -69,9 +75,20 @@ KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
 	pExp( -1 ),
 	running( false )
 {
-	QGridLayout *grid = new QGridLayout( 0, 0, 10 );
-	layoutItem = grid;
+	KdmItem *user_entry = 0, *pw_entry = 0, *domain_entry = 0;
+	QGridLayout *grid = 0;
+
 	int line = 0;
+	layoutItem = 0;
+
+	if (themer &&
+	    (!(user_entry = themer->findNode( "user-entry" )) ||
+	     !(pw_entry = themer->findNode( "pw-entry" )) ||
+	     !(domain_entry = themer->findNode( "domain-entry" ))))
+		themer = 0;
+
+	if (!themer)
+		layoutItem = grid = new QGridLayout( 0, 0, 10 );
 
 	domainLabel = loginLabel = passwdLabel = passwd1Label = passwd2Label = 0;
 	domainCombo = 0;
@@ -84,7 +101,6 @@ KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
 	if (func != ChAuthTok) {
 		if (fixedUser.isEmpty()) {
 			domainCombo = new KComboBox( parent );
-			domainLabel = new QLabel( domainCombo, i18n("&Domain:"), parent );
 			connect( domainCombo, SIGNAL(activated( const QString & )),
 			         SLOT(slotChangedDomain( const QString & )) );
 			connect( domainCombo, SIGNAL(activated( const QString & )),
@@ -94,20 +110,29 @@ KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
 			// should handle loss of focus
 			loginEdit = new KLineEdit( parent );
 			loginEdit->setContextMenuEnabled( false );
-			loginLabel = new QLabel( loginEdit, i18n("&Username:"), parent );
-			connect( loginEdit, SIGNAL(lostFocus()), SLOT(slotLoginLostFocus()) );
-			connect( loginEdit, SIGNAL(lostFocus()), SLOT(slotActivity()) );
-			connect( loginEdit, SIGNAL(textChanged( const QString & )), SLOT(slotActivity()) );
-			connect( loginEdit, SIGNAL(selectionChanged()), SLOT(slotActivity()) );
+
 			if (pred) {
 				parent->setTabOrder( pred, domainCombo );
 				parent->setTabOrder( domainCombo, loginEdit );
 				pred = loginEdit;
 			}
-			grid->addWidget( domainLabel, line, 0 );
-			grid->addWidget( domainCombo, line++, 1 );
-			grid->addWidget( loginLabel, line, 0 );
-			grid->addWidget( loginEdit, line++, 1 );
+			if (!grid) {
+				loginEdit->adjustSize();
+				domainCombo->adjustSize();
+				user_entry->setWidget( loginEdit );
+				domain_entry->setWidget( domainCombo );
+			} else {
+				domainLabel = new QLabel( domainCombo, i18n("&Domain:"), parent );
+				loginLabel = new QLabel( loginEdit, i18n("&Username:"), parent );
+				grid->addWidget( domainLabel, line, 0 );
+				grid->addWidget( domainCombo, line++, 1 );
+				grid->addWidget( loginLabel, line, 0 );
+				grid->addWidget( loginEdit, line++, 1 );
+			}
+			connect( loginEdit, SIGNAL(lostFocus()), SLOT(slotLoginLostFocus()) );
+			connect( loginEdit, SIGNAL(lostFocus()), SLOT(slotActivity()) );
+			connect( loginEdit, SIGNAL(textChanged( const QString & )), SLOT(slotActivity()) );
+			connect( loginEdit, SIGNAL(selectionChanged()), SLOT(slotActivity()) );
 			domainCombo->insertStringList( domains );
 			domainCombo->setCurrentItem( defaultDomain );
 		} else if (ctx != Login && ctx != Shutdown) {
@@ -126,19 +151,26 @@ KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
 		connect( passwdEdit, SIGNAL(textChanged( const QString & )),
 		         SLOT(slotActivity()) );
 		connect( passwdEdit, SIGNAL(lostFocus()), SLOT(slotActivity()) );
-		passwdLabel = new QLabel( passwdEdit,
-		                          func == Authenticate ?
-		                            i18n("&Password:") :
-		                            i18n("Current &password:"),
-		                          parent );
-		if (pred) {
-			parent->setTabOrder( pred, passwdEdit );
-			pred = passwdEdit;
+
+		if (!grid) {
+			passwdEdit->adjustSize();
+			pw_entry->setWidget( passwdEdit );
+		} else {
+			passwdLabel = new QLabel( passwdEdit,
+			                          func == Authenticate ?
+			                          i18n("&Password:") :
+			                          i18n("Current &password:"),
+			                          parent );
+			if (pred) {
+				parent->setTabOrder( pred, passwdEdit );
+				pred = passwdEdit;
+			}
+			grid->addWidget( passwdLabel, line, 0 );
+			grid->addWidget( passwdEdit, line++, 1 );
 		}
-		grid->addWidget( passwdLabel, line, 0 );
-		grid->addWidget( passwdEdit, line++, 1 );
-		if (domainCombo)
-			domainCombo->setFocus();
+
+		if (loginEdit)
+			loginEdit->setFocus();
 		else
 			passwdEdit->setFocus();
 	}
@@ -169,9 +201,15 @@ KWinbindGreeter::KWinbindGreeter( KGreeterPluginHandler *_handler,
 KWinbindGreeter::~KWinbindGreeter()
 {
 	abort();
+	if (!layoutItem) {
+		delete loginEdit;
+		delete passwdEdit;
+		delete domainCombo;
+		return;
+	}
 	QLayoutIterator it = static_cast<QLayout *>(layoutItem)->iterator();
 	for (QLayoutItem *itm = it.current(); itm; itm = ++it)
-		 delete itm->widget();
+		delete itm->widget();
 	delete layoutItem;
 }
 
@@ -518,9 +556,33 @@ static bool init( const QString &,
                   void *ctx )
 {
 	echoMode = getConf( ctx, "EchoMode", QVariant( -1 ) ).toInt();
-	domains = QStringList::split( ':', getConf( ctx, "winbind.Domains", QVariant( "<local>" ) ).toString() );
+	domains = QStringList::split( ':', getConf( ctx, "winbind.Domains", QVariant( "" ) ).toString() );
+	if (!domains.size()) {
+		FILE *domfile = popen( "wbinfo -m 2>/dev/null", "r" );
+		if (domfile) {
+			QString tmp;
+			QTextIStream is( domfile );
+			while (!is.atEnd()) {
+				is >> tmp;
+				domains << tmp;
+			}
+			if (pclose( domfile )) // error
+				domains = QStringList();
+		}
+		domains << "<local>";
+	}
 	defaultDomain = getConf( ctx, "winbind.DefaultDomain", QVariant( domains.first() ) ).toString();
-	separator = getConf( ctx, "winbind.Separator", QVariant( '\\' ) ).toString()[0];
+	QString sepstr = getConf( ctx, "winbind.Separator", QVariant( QString::null ) ).toString();
+	if (sepstr.isNull()) {
+		FILE *sepfile = popen( "wbinfo --separator 2>/dev/null", "r" );
+		if (sepfile) {
+			QTextIStream( sepfile ) >> sepstr;
+			if (pclose( sepfile ))
+				sepstr = "\\";
+		} else
+			sepstr = "\\";
+	}
+	separator = sepstr[0].latin1();
 	KGlobal::locale()->insertCatalogue( "kgreet_winbind" );
 	return true;
 }
@@ -534,13 +596,13 @@ static void done( void )
 }
 
 static KGreeterPlugin *
-create( KGreeterPluginHandler *handler, KdmThemer * /*themer*/,
+create( KGreeterPluginHandler *handler, KdmThemer *themer,
         QWidget *parent, QWidget *predecessor,
         const QString &fixedEntity,
         KGreeterPlugin::Function func,
         KGreeterPlugin::Context ctx )
 {
-	return new KWinbindGreeter( handler, parent, predecessor, fixedEntity, func, ctx );
+	return new KWinbindGreeter( handler, themer, parent, predecessor, fixedEntity, func, ctx );
 }
 
 KDE_EXPORT kgreeterplugin_info kgreeterplugin_info = {
