@@ -8,8 +8,6 @@
 #define KFTABDLG_H
 
 #include <qtabwidget.h>
-#include <qcombobox.h>
-#include <qspinbox.h>
 #include <qvalidator.h> // for KDigitValidator
 
 #include <kurl.h>
@@ -26,6 +24,8 @@ class QString;
 class QDate;
 class QRegExp;
 class QDialog;
+class QComboBox;
+class QSpinBox;
 
 class KfDirDialog;
 
@@ -37,6 +37,7 @@ public:
   KfindTabWidget(QWidget * parent = 0, const char *name=0);
   virtual ~KfindTabWidget();
   void initMimeTypes();
+  void initSpecialMimeTypes();
   void setQuery(class KQuery * query);
   void setDefaults();
 
@@ -44,6 +45,7 @@ public:
   void endSearch();
   void loadHistory();
   void saveHistory();
+  bool isSearchRecursive();
 
   void setURL( const KURL & url );
 
@@ -65,6 +67,7 @@ public:
   QComboBox   *dirBox;
   // for first page
   QCheckBox   *subdirsCb;
+  QCheckBox *useLocateCb;
   // for third page
   QComboBox *typeBox;
   QLineEdit * textEdit;
@@ -102,12 +105,16 @@ private:
   QComboBox *sizeUnitBox;
   QSpinBox *sizeEdit;
   QCheckBox *caseContextCb;
+  QCheckBox *binaryContextCb;
   QCheckBox *regexpContentCb;
   QDialog *regExpDialog;
 
   KURL m_url;
 
   KMimeType::List m_types;
+  QStringList m_ImageTypes;
+  QStringList m_VideoTypes;
+  QStringList m_AudioTypes;
 };
 
 class KDigitValidator : public QValidator
