@@ -7,6 +7,7 @@
 #include <qregexp.h>
 #include <qptrqueue.h>
 #include <qdir.h>
+#include <qstringlist.h>
 
 #include <kio/job.h>
 #include <kurl.h>
@@ -86,12 +87,15 @@ class KQuery : public QObject
   QStringList locateList;
   KProcess *processLocate;
   QPtrList<QRegExp> m_regexps;// regexps for file name
-  QValueList<bool> m_regexpsContainsGlobs;
+//  QValueList<bool> m_regexpsContainsGlobs;  // what should this be good for ? Alex
   KIO::ListJob *job;
   bool m_insideCheckEntries;
   QPtrQueue<KFileItem> m_fileItems;
   QRegExp* metaKeyRx;
   int m_result;
+  QStringList ignore_mimetypes;
+  QStringList ooo_mimetypes;     // OpenOffice.org mimetypes
+  QStringList koffice_mimetypes;
 };
 
 #endif
