@@ -52,6 +52,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kglobalsettings.h>
 #include <kconfigdialogmanager.h>
 #include <kcalendarsystem.h>
+#include <kicontheme.h>
+#include <kiconloader.h>
+
+#include <global.h> // libkickermain
 
 #include "clock.h"
 #include "datepicker.h"
@@ -120,6 +124,8 @@ KConfigDialogSingle::KConfigDialogSingle(Zone *zone, QWidget *parent,
             SLOT(dateToggled()));
     connect(digitalPage->kcfg_DigitalShowDate, SIGNAL(toggled(bool)),
             SLOT(dateToggled()));
+    connect(analogPage->kcfg_AnalogShowDate, SIGNAL(toggled(bool)),
+            SLOT(dateToggled()));
     connect(analogPage->kcfg_AnalogShowDayOfWeek, SIGNAL(toggled(bool)),
             SLOT(dateToggled()));
     connect(fuzzyPage->kcfg_FuzzyShowDate, SIGNAL(toggled(bool)),
@@ -905,6 +911,8 @@ ClockApplet::ClockApplet(const QString& configFile, Type t, int actions,
         connect(menu, SIGNAL(activated(int)), SLOT(contextMenuActivated(int)));
         setCustomMenu(menu);
     }
+
+    installEventFilter(KickerTip::the());
 }
 
 
@@ -921,6 +929,8 @@ ClockApplet::~ClockApplet()
         _calendar->close();
     }
 
+    zone->writeSettings();
+
     delete _prefs; _prefs = 0;
     delete zone; zone = 0;
     delete menu; menu = 0;
@@ -1020,6 +1030,7 @@ int ClockApplet::widthForHeight(int h) const
 
             _clock->widget()->setFixedSize(w, h - shareDateHeight - shareDayOfWeekHeight);
             _clock->widget()->setMinimumSize(w, h - shareDateHeight - shareDayOfWeekHeight);
+            _clock->widget()->move(0, 0);
             if (showDayOfWeek)
             {
                 _dayOfWeek->setFixedSize(w, _dayOfWeek->sizeHint().height());
@@ -1315,6 +1326,7 @@ void ClockApplet::slotUpdate()
     }
 
     _clock->updateClock();
+    KickerTip::Client::updateKickerTip();
 }
 
 void ClockApplet::slotCalendarDeleted()
@@ -1323,6 +1335,9 @@ void ClockApplet::slotCalendarDeleted()
     // don't reopen the calendar immediately ...
     _disableCalendar = true;
     QTimer::singleShot(100, this, SLOT(slotEnableCalendar()));
+
+    // we are free to show a tip know :)
+    installEventFilter(KickerTip::the());
 }
 
 
@@ -1333,65 +1348,45 @@ void ClockApplet::slotEnableCalendar()
 
 void ClockApplet::toggleCalendar()
 {
-    if (_calendar && !_disableCalendar) {
+    if (_calendar && !_disableCalendar)
+    {
         // calls slotCalendarDeleted which does the cleanup for us
         _calendar->close();
         return;
     }
+
+    KickerTip::enableTipping(false);
+    // don't show KickerTip above calender.
+    removeEventFilter(KickerTip::the());
+
     if (_calendar || _disableCalendar)
+    {
         return;
+    }
+
     _calendar = new DatePicker(this, _lastDate, _prefs);
-    connect( _calendar, SIGNAL( destroyed() ), SLOT( slotCalendarDeleted() ));
+    connect(_calendar, SIGNAL(destroyed()), SLOT(slotCalendarDeleted()));
 
     QSize size = _prefs->calendarSize();
-    int w;
-    int h;
 
     if (size != QSize())
     {
         _calendar->resize(size);
-        // Add 28 px. to width poperly as said in API
-        w = size.width() + 28;
-        h = size.height();
     }
     else
     {
-        // Add 28 px. to width poperly as said in API
-        w = _calendar->sizeHint().width() + 28;
-        h = _calendar->sizeHint().height();
-    }
-
-    // BUG: this does not work when the panel is hidden
-    //      since QPoint(0,0) may be reported as being off the screen!
-    QPoint c = mapToGlobal(QPoint(0,0));
-
-    // some extra spacing is included if aligned on a desktop edge
-    switch (position()) {
-        case KPanelApplet::pLeft:
-            c.setX(c.x()+width()+2);
-            break;
-        case KPanelApplet::pRight:
-            c.setX(c.x()-w-2);
-            break;
-        case KPanelApplet::pTop:
-            c.setY(c.y()+height()+2);
-            break;
-        case KPanelApplet::pBottom:
-            c.setY(c.y()-h-2);
-            break;
+        _calendar->adjustSize();
     }
 
     // make calendar fully visible
-    QRect deskR = KGlobalSettings::desktopGeometry(c);
-
-    if (c.y()+h > deskR.bottom())
-        c.setY(deskR.bottom()-h-1);
-    if (c.x()+w > deskR.right())
-        c.setX(deskR.right()-w+28-1);
-
-    _calendar->move(c);
+    QPoint popupAt = KickerLib::popupPosition(popupDirection(),
+                                              _calendar,
+                                              this);
+    _calendar->move(popupAt);
     _calendar->show();
     _calendar->setFocus();
+
+    KickerTip::enableTipping(true);
 }
 
 
@@ -1416,6 +1411,7 @@ void ClockApplet::contextMenuActivated(int result)
     if ((result >= 500) && (result < 600))
     {
         showZone(result-500);
+        zone->writeSettings();
         return;
     };
 
@@ -1532,7 +1528,6 @@ QDate ClockApplet::clockGetDate()
 void ClockApplet::showZone(int z)
 {
     zone->setZone(z);
-    zone->writeSettings();
     TZoffset = zone->calc_TZ_offset( zone->zone() );
     _clock->updateClock();
     _clock->forceUpdate(); /* force repaint */
@@ -1585,6 +1580,7 @@ void ClockApplet::wheelEvent(QWheelEvent* e)
     }
 
     QToolTip::remove(_clock->widget());
+    KickerTip::Client::updateKickerTip();
 }
 
 // catch the mouse clicks of our child widgets
@@ -1636,6 +1632,69 @@ void ClockApplet::updateDateLabel(bool reLayout)
     }
 }
 
+void ClockApplet::updateKickerTip(KickerTip::Data& data)
+{
+    int zoneCount = zone->remoteZoneCount();
+
+    QString activeZone = zone->zone();
+    if (zoneCount == 0)
+    {
+        QString _time = KGlobal::locale()->formatTime(clockGetTime(),
+                                                    _prefs->plainShowSeconds());
+        QString _date = KGlobal::locale()->formatDate(clockGetDate(), false);
+        data.message = _time;
+        data.subtext = _date;
+
+        if (!activeZone.isEmpty())
+        {
+            activeZone = i18n(activeZone.utf8());
+            data.subtext.append("<br>").append(activeZone.mid(activeZone.find('/') + 1).replace("_", " "));
+        }
+    }
+    else
+    {
+        int activeIndex = zone->zoneIndex();
+
+        for (int i = 0; i <= zone->remoteZoneCount(); i++)
+        {
+            QString m_zone = zone->zone(i);
+            TZoffset = zone->calc_TZ_offset(m_zone);
+
+            if (!m_zone.isEmpty())
+            {
+                m_zone = i18n(m_zone.utf8()); // ensure it gets translated
+            }
+
+            QString _time = KGlobal::locale()->formatTime(clockGetTime(),
+                                                          _prefs->plainShowSeconds());
+            QString _date = KGlobal::locale()->formatDate(clockGetDate(), false);
+
+            if (activeIndex == i)
+            {
+                data.message = m_zone.mid(m_zone.find('/') + 1).replace("_", " ");
+                data.message += "  " + _time + "<br>" + _date;
+            }
+            else
+            {
+                if (i == 0)
+                {
+                    data.subtext += "<b>" + i18n("Local Timezone") + "</b>";
+                }
+                else
+                {
+                    data.subtext += "<b>" + m_zone.mid(m_zone.find('/') + 1).replace("_", " ") + "</b>";
+                }
+                data.subtext += " " + _time + ", " + _date + "<br>";
+            }
+        }
+
+        TZoffset = zone->calc_TZ_offset(activeZone);
+    }
+
+    data.icon = DesktopIcon("date", KIcon::SizeMedium);
+    data.direction = popupDirection();
+}
+
 void ClockApplet::fixupLayout()
 {
     m_layoutDelay = 0;
@@ -1700,7 +1759,7 @@ void ClockAppletToolTip::maybeTip( const QPoint & /*point*/ )
 
     if (m_clock->timezones() && m_clock->timezones()->zoneIndex() > 0)
     {
-        tipText += "\n" + i18n("Showing time for %1").arg(m_clock->timezones()->zone(), false);
+        tipText += "\n" + i18n("Showing time for %1").arg(i18n(m_clock->timezones()->zone().utf8()), false);
     }
 
     tip(m_clock->geometry(), tipText);
