@@ -29,6 +29,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <qpopupmenu.h>
 #include <qtooltip.h>
 
+#include <kactionclasses.h>
+#include <kickertip.h>
 #include <klocale.h>
 #include <kdesktopfile.h>
 #include <krun.h>
@@ -41,6 +43,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kurldrag.h>
 #include <kstandarddirs.h>
 
+#include <math.h>
 #include <algorithm>
 
 #ifdef DEBUG
@@ -88,17 +91,17 @@ QuickURL::QuickURL(const QString &u)
          _service = 0;
       } else {
          DEBUGSTR << "QuickURL: _service='"<<_service<<"' _service->desktopEntryPath()="<<_service->desktopEntryPath()<<endl<<flush;
-         _kurl.setPath(locate("apps", _service->desktopEntryPath()));
+         if (_kurl.path().length() == 0)
+         {
+            _kurl.setPath(locate("apps", _service->desktopEntryPath()));
+         }
          if (!_service->menuId().isEmpty())
             _menuId = _service->menuId();
 
          _comment = _service->comment();
-         if (_comment.isEmpty())
-            _comment = _service->genericName();
          if (_comment.isEmpty())
             _comment = _service->name();
-         else
-            _comment = _service->name() + " - " + _comment;
+         m_genericName = _service->genericName();
       }
    } else {
       _comment = _kurl.prettyURL();
@@ -125,44 +128,63 @@ QPixmap QuickURL::pixmap( mode_t _mode, KIcon::Group _group,
 }
 
 
-QuickButton::QuickButton(const QString &u, QWidget *parent, const char *name)
-   : QButton(parent, name)
+QuickButton::QuickButton(const QString &u, KAction* configAction, 
+                         QWidget *parent, const char *name) : 
+     QButton(parent, name),
+     m_flashCounter(0),
+     m_sticky(false)
 {
-   if (parent && ! parent->parent()) {
-      setBackgroundMode(X11ParentRelative);
-   }
-   setBackgroundOrigin( AncestorOrigin );
-   setMouseTracking(true);
-   _highlight = false;
-   _oldCursor = cursor();
-   _qurl=new QuickURL(u);
+    installEventFilter(KickerTip::the());
+    if (parent && ! parent->parent())
+    {
+        setBackgroundMode(X11ParentRelative);
+    }
+    setBackgroundOrigin( AncestorOrigin );
+    setMouseTracking(true);
+    _highlight = false;
+    _oldCursor = cursor();
+    _qurl=new QuickURL(u);
+    
+    QToolTip::add(this, _qurl->comment());
+    resize(int(DEFAULT_ICON_DIM),int(DEFAULT_ICON_DIM));
+    QBrush bgbrush(colorGroup().brush(QColorGroup::Background));
+    
+    QuickAddAppsMenu *addAppsMenu = new QuickAddAppsMenu(
+        parent, this, _qurl->url());
+    _popup = new QPopupMenu(this);
+    _popup->insertItem(i18n("Add Application"), addAppsMenu);
+    configAction->plug(_popup);
+        _popup->insertSeparator();
+    _popup->insertItem(SmallIcon("remove"), i18n("Remove"), 
+            this, SLOT(removeApp()));
 
-   QToolTip::add(this, _qurl->comment());
-   resize(int(DEFAULT_ICON_DIM),int(DEFAULT_ICON_DIM));
-   QBrush bgbrush( colorGroup().brush( QColorGroup::Background ) );
+    m_stickyAction = new KToggleAction(i18n("Never Remove Automatically"),
+        KShortcut(), this);
+    connect(m_stickyAction, SIGNAL(toggled(bool)), 
+        this, SLOT(slotStickyToggled(bool)));
+    m_stickyAction->plug(_popup, 2);
+    m_stickyId = _popup->idAt(2);
 
-   QuickAddAppsMenu *addAppsMenu = new QuickAddAppsMenu(parent, this,_qurl->url());
-   _popup = new QPopupMenu(this);
-   _popup->insertItem(i18n("Add Application"), addAppsMenu);
-   _popup->insertSeparator();
-   _popup->insertItem(SmallIcon("remove"), i18n("Remove"), this, SLOT(removeApp()));
-
-   settingsChanged(KApplication::SETTINGS_MOUSE);
-   connect(kapp, SIGNAL(settingsChanged(int)), SLOT(settingsChanged(int)));
-   connect(kapp, SIGNAL(iconChanged(int)), SLOT(iconChanged(int)));
-   connect(this, SIGNAL(clicked()), SLOT(launch()));
-   connect(this, SIGNAL(removeApp(QuickButton *)), parent, SLOT(removeApp(QuickButton *)));
-   kapp->addKipcEventMask(KIPC::SettingsChanged);
-   kapp->addKipcEventMask(KIPC::IconChanged);
+    settingsChanged(KApplication::SETTINGS_MOUSE);
+    connect(kapp, SIGNAL(settingsChanged(int)), SLOT(settingsChanged(int)));
+    connect(kapp, SIGNAL(iconChanged(int)), SLOT(iconChanged(int)));
+    connect(this, SIGNAL(clicked()), SLOT(launch()));
+    connect(this, SIGNAL(removeApp(QuickButton *)), parent,
+        SLOT(removeAppManually(QuickButton *)));
+    kapp->addKipcEventMask(KIPC::SettingsChanged);
+    kapp->addKipcEventMask(KIPC::IconChanged);
 }
 
 QuickButton::~QuickButton()
-{  delete _qurl;
+{
+    delete _qurl;
 }
 
 
 QString QuickButton::url() const
-{  return _qurl->url();}
+{
+    return _qurl->url();
+}
 
 
 QString QuickButton::menuId() const
@@ -205,7 +227,7 @@ void QuickButton::drawButtonLabel(QPainter *p)
    int d = ICON_MARGIN;
    if (isDown() || isOn()) d += 1;
 
-   p->drawPixmap(offset.x()+ d, offset.y() + d, *pix);
+   if (m_flashCounter%500<250) p->drawPixmap(offset.x()+ d, offset.y() + d, *pix);
 }
 
 void QuickButton::enterEvent(QEvent*)
@@ -278,6 +300,7 @@ void QuickButton::launch()
    repaint();
    KIconEffect::visualActivate(this, rect());
    _qurl->run();
+   emit executed(_qurl->menuId());
 }
 
 void QuickButton::setDragging(bool enable)
@@ -296,4 +319,63 @@ void QuickButton::removeApp()
 {
    emit removeApp(this);
 }
+
+void QuickButton::flash()
+{
+   m_flashCounter = 2000;
+   QTimer::singleShot(0, this, SLOT(slotFlash()));
+}
+
+void QuickButton::slotFlash()
+{
+    static const int timeout = 500/4;
+    if (m_flashCounter > 0)
+    {
+        m_flashCounter -= timeout;
+        if (m_flashCounter < 0) m_flashCounter = 0;
+        repaint();
+        QTimer::singleShot(timeout, this, SLOT(slotFlash()));
+    }
+}
+
+void QuickButton::slotStickyToggled(bool isSticky)
+{
+    m_sticky = isSticky;
+    emit stickyToggled(isSticky);
+}
+
+void QuickButton::setSticky(bool sticky)
+{
+    m_stickyAction->setChecked(sticky);
+    slotStickyToggled(sticky);
+}
+
+void QuickButton::updateKickerTip(KickerTip::Data &data)
+{
+    if (!_qurl)
+    {
+        return;
+    }
+    data.message = _qurl->comment();
+    data.direction = m_popupDirection;
+    data.subtext = _qurl->genericName();
+    if (data.subtext == QString())
+    {
+        data.subtext = data.message;
+    }
+    data.icon = KMimeType::pixmapForURL(_qurl->kurl(), 0, 
+        KIcon::Panel, KIcon::SizeHuge, KIcon::DefaultState);
+}
+
+void QuickButton::setPopupDirection(KPanelApplet::Direction d)
+{
+    m_popupDirection = d;
+}
+
+void QuickButton::setDynamicModeEnabled(bool enabled)
+{
+    _popup->setItemVisible(m_stickyId, enabled);
+}
+
+
 #include "quickbutton.moc"
