@@ -849,13 +849,15 @@ void PanelPopupButton::setPopup(QPopupMenu *popup)
     if (m_popup)
     {
         m_popup->removeEventFilter(this);
+        disconnect(m_popup, SIGNAL(aboutToHide()), this, SLOT(menuAboutToHide()));
     }
 
     m_popup = popup;
 
-    if (popup)
+    if (m_popup)
     {
         m_popup->installEventFilter(this);
+        connect(m_popup, SIGNAL(aboutToHide()), this, SLOT(menuAboutToHide()));
     }
 }
 
@@ -906,7 +908,11 @@ void PanelPopupButton::showMenu()
 {
     if (isDown())
     {
-        m_popup->hide();
+        if (m_popup)
+        {
+            m_popup->hide();
+        }
+
         setDown(false);
         return;
     }
@@ -935,6 +941,15 @@ void PanelPopupButton::slotExecMenu()
 
     m_popup->adjustSize();
     m_popup->exec(KickerLib::popupPosition(popupDirection(), m_popup, this));
+}
+
+void PanelPopupButton::menuAboutToHide()
+{
+    if (!m_popup)
+    {
+        return;
+    }
+
     setDown(false);
     KickerTip::enableTipping(true);
 }
