@@ -77,24 +77,25 @@ void PrefMenu::insertMenuItem(KService::Ptr& s,
 {
     QString serviceName = s->name();
     // add comment
-    if (KickerSettings::detailedMenuEntries())
+    QString comment = s->genericName();
+    if (!comment.isEmpty())
     {
-        QString comment = s->genericName();
-        if ( !comment.isEmpty() )
+        if (KickerSettings::menuEntryFormat() == KickerSettings::NameAndDescription)
         {
-            if (KickerSettings::detailedEntriesNamesFirst())
+            if (!suppressGenericNames ||
+                !suppressGenericNames->contains(s->untranslatedGenericName()))
             {
-                if (!suppressGenericNames ||
-                     !suppressGenericNames->contains(s->untranslatedGenericName()))
-                {
-                    serviceName = QString("%1 (%2)").arg(serviceName).arg(comment);
-                }
-            }
-            else
-            {
-                serviceName = QString("%1 (%2)").arg(comment).arg(serviceName);
+                serviceName = QString("%1 (%2)").arg(serviceName).arg(comment);
             }
         }
+        else if (KickerSettings::menuEntryFormat() == KickerSettings::DescriptionAndName)
+        {
+            serviceName = QString("%1 (%2)").arg(comment).arg(serviceName);
+        }
+        else if (KickerSettings::menuEntryFormat() == KickerSettings::DescriptionOnly)
+        {
+            serviceName = comment;
+        }
     }
 
     // restrict menu entries to a sane length
@@ -124,14 +125,14 @@ void PrefMenu::insertMenuItem(KService::Ptr& s,
 
     if (KickerSettings::menuEntryHeight() > 0 )
     {
-        iconset = KGlobal::instance()->iconLoader()->loadIconSet(
+        iconset = KGlobal::iconLoader()->loadIconSet(
                                             s->icon(),
                                             KIcon::NoGroup,
                                             KickerSettings::menuEntryHeight());
     }
     else
     {
-        QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(
+        QPixmap normal = KGlobal::iconLoader()->loadIcon(
                                                             s->icon(),
                                                             KIcon::Small,
                                                             0,
@@ -139,7 +140,7 @@ void PrefMenu::insertMenuItem(KService::Ptr& s,
                                                             0L,
                                                             true);
 
-        QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(
+        QPixmap active = KGlobal::iconLoader()->loadIcon(
                                                             s->icon(),
                                                             KIcon::Small,
                                                             0,
@@ -285,8 +286,18 @@ void PrefMenu::initialize()
 
     if (m_root.isEmpty())
     {
-        insertItem(SmallIconSet("kcontrol"), i18n("Control Center"),
-                   this, SLOT(launchControlCenter()));
+        if (KickerSettings::menuEntryHeight() > -1)
+        {
+            insertItem(SmallIconSet("kcontrol",
+                       KickerSettings::menuEntryHeight()),
+                       i18n("Control Center"),
+                       this, SLOT(launchControlCenter()));
+        }
+        else
+        {
+            insertItem(i18n("Control Center"), this,
+                       SLOT(launchControlCenter()));
+        }
         insertSeparator();
     }
 
@@ -299,8 +310,7 @@ void PrefMenu::initialize()
     }
 
     KServiceGroup::List list = root->entries(true, true, true,
-                                             KickerSettings::detailedMenuEntries() &&
-                                             !KickerSettings::detailedEntriesNamesFirst());
+                                             KickerSettings::menuEntryFormat() == KickerSettings:: NameAndDescription);
 
     if (list.isEmpty())
     {
@@ -347,14 +357,14 @@ void PrefMenu::initialize()
 
             if (KickerSettings::menuEntryHeight() > 0)
             {
-                iconset = KGlobal::instance()->iconLoader()->loadIconSet(
+                iconset = KGlobal::iconLoader()->loadIconSet(
                                                             g->icon(),
                                                             KIcon::NoGroup,
                                             KickerSettings::menuEntryHeight());
             }
             else
             {
-                QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(
+                QPixmap normal = KGlobal::iconLoader()->loadIcon(
                                                             g->icon(),
                                                             KIcon::Small,
                                                             0,
@@ -362,7 +372,7 @@ void PrefMenu::initialize()
                                                             0L,
                                                             true);
 
-                QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(
+                QPixmap active = KGlobal::iconLoader()->loadIcon(
                                                             g->icon(),
                                                             KIcon::Small,
                                                             0,
