@@ -116,7 +116,7 @@ void RecentDocsMenu::mouseMoveEvent(QMouseEvent* e) {
 
 	KDesktopFile f(_fileList[id], true /* read only */);
 
-	KURL url = f.readURL();
+	KURL url ( f.readURL() );
 
 	if (url.isEmpty()) // What are we to do ?
 		return;
