@@ -122,6 +122,12 @@ void TaskContainer::init()
 
 TaskContainer::~TaskContainer()
 {
+    if (m_menu)
+    {
+        delete m_menu;
+        m_menu = 0;
+    }
+
     stopTimers();
 }
 
@@ -338,6 +344,23 @@ void TaskContainer::remove(Task::Ptr task)
     update();
 }
 
+void TaskContainer::remove(Startup::Ptr startup)
+{
+    if (!startup || startup != m_startup)
+    {
+        return;
+    }
+
+    m_startup = 0;
+    animationTimer.stop();
+    frames.clear();
+
+    if (!tasks.isEmpty())
+    {
+        setEnabled(true);
+    }
+}
+
 bool TaskContainer::contains(Task::Ptr task)
 {
     if (!task)
@@ -453,7 +476,8 @@ void TaskContainer::drawButton(QPainter *p)
     Task::Ptr task = 0;
     bool iconified = !TaskBarSettings::showOnlyIconified();
     bool halo = TaskBarSettings::haloText();
-    bool drawButton = TaskBarSettings::drawButtons() ||
+    bool alwaysDrawButtons = TaskBarSettings::drawButtons();
+    bool drawButton = alwaysDrawButtons ||
                       (m_mouseOver && isEnabled() &&
                        TaskBarSettings::showButtonOnHover());
     QFont font(KGlobalSettings::taskbarFont());
@@ -522,7 +546,7 @@ void TaskContainer::drawButton(QPainter *p)
         pixmap = task->pixmap();
     }
 
-    bool sunken = isDown() || active || aboutToActivate;
+    bool sunken = isDown() || (alwaysDrawButtons && (active || aboutToActivate));
     bool reverse = QApplication::reverseLayout();
     QRect br(style().subRect(QStyle::SR_PushButtonContents, this));
     QPoint shift = QPoint(style().pixelMetric(QStyle::PM_ButtonShiftHorizontal),
@@ -891,6 +915,11 @@ void TaskContainer::mouseReleaseEvent(QMouseEvent *e)
 {
     m_dragStartPos = QPoint();
 
+    if (!TaskBarSettings::drawButtons())
+    {
+        setDown(false);
+    }
+
     // This is to avoid the flicker caused by redrawing the
     // button as unpressed just before it's activated.
     if (!rect().contains(e->pos()))
