@@ -13,10 +13,14 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
+// Get those asserts to work
+#undef NDEBUG
+#undef NO_DEBUG
+
 #include "kio_trash.h"
 #include "testtrash.h"
 
@@ -32,6 +36,7 @@
 
 #include <qdir.h>
 #include <qfileinfo.h>
+#include <qvaluevector.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -109,6 +114,20 @@ QString TestTrash::umlautFileName() const
     return QString( "umlaut" ) + QChar( 0xEB );
 }
 
+static void removeFile( const QString& trashDir, const QString& fileName )
+{
+    QDir dir;
+    dir.remove( trashDir + fileName );
+    assert( !QDir( trashDir + fileName ).exists() );
+}
+
+static void removeDir( const QString& trashDir, const QString& dirName )
+{
+    QDir dir;
+    dir.rmdir( trashDir + dirName );
+    assert( !QDir( trashDir + dirName ).exists() );
+}
+
 void TestTrash::setup()
 {
     m_trashDir = KGlobal::dirs()->localxdgdatadir() + "Trash";
@@ -120,10 +139,12 @@ void TestTrash::setup()
 
     TrashImpl::TrashDirMap trashDirs = impl.trashDirectories();
     TrashImpl::TrashDirMap topDirs = impl.topDirectories();
-    TrashImpl::TrashDirMap::ConstIterator it = trashDirs.begin();
     bool foundTrashDir = false;
     m_otherPartitionId = 0;
-    for ( ; it != trashDirs.end() ; ++it ) {
+    m_tmpIsWritablePartition = false;
+    m_tmpTrashId = -1;
+    QValueVector<int> writableTopDirs;
+    for ( TrashImpl::TrashDirMap::ConstIterator it = trashDirs.begin(); it != trashDirs.end() ; ++it ) {
         if ( it.key() == 0 ) {
             assert( it.data() == m_trashDir );
             assert( topDirs.find( 0 ) == topDirs.end() );
@@ -132,15 +153,38 @@ void TestTrash::setup()
             assert( topDirs.find( it.key() ) != topDirs.end() );
             const QString topdir = topDirs[it.key()];
             if ( QFileInfo( topdir ).isWritable() ) {
-                m_otherPartitionTopDir = topdir;
-                m_otherPartitionTrashDir = it.data();
-                m_otherPartitionId = it.key();
-                kdDebug() << "OK, found another writable partition: topDir=" << m_otherPartitionTopDir
-                          << " trashDir=" << it.data() << " id=" << m_otherPartitionId << endl;
-                break;
+                writableTopDirs.append( it.key() );
+                if ( topdir == "/tmp/" ) {
+                    m_tmpIsWritablePartition = true;
+                    m_tmpTrashId = it.key();
+                    kdDebug() << "/tmp is on its own partition (trashid=" << m_tmpTrashId << "), some tests will be skipped" << endl;
+                    removeFile( it.data(), "/info/fileFromOther.trashinfo" );
+                    removeFile( it.data(), "/files/fileFromOther" );
+                    removeFile( it.data(), "/info/symlinkFromOther.trashinfo" );
+                    removeFile( it.data(), "/files/symlinkFromOther" );
+                    removeFile( it.data(), "/info/trashDirFromOther.trashinfo" );
+                    removeFile( it.data(), "/files/trashDirFromOther/testfile" );
+                    removeDir( it.data(), "/files/trashDirFromOther" );
+                }
             }
         }
     }
+    for ( QValueVector<int>::const_iterator it = writableTopDirs.begin(); it != writableTopDirs.end(); ++it ) {
+        const QString topdir = topDirs[ *it ];
+        const QString trashdir = trashDirs[ *it ];
+        assert( !topdir.isEmpty() );
+        assert( !trashDirs.isEmpty() );
+        if ( topdir != "/tmp/" ||         // we'd prefer not to use /tmp here, to separate the tests
+               ( writableTopDirs.count() > 1 ) ) // but well, if we have no choice, take it
+        {
+            m_otherPartitionTopDir = topdir;
+            m_otherPartitionTrashDir = trashdir;
+            m_otherPartitionId = *it;
+            kdDebug() << "OK, found another writable partition: topDir=" << m_otherPartitionTopDir
+                      << " trashDir=" << m_otherPartitionTrashDir << " id=" << m_otherPartitionId << endl;
+            break;
+        }
+    }
     // Check that m_trashDir got listed
     assert( foundTrashDir );
     if ( m_otherPartitionTrashDir.isEmpty() )
@@ -159,19 +203,6 @@ void TestTrash::setup()
     cleanTrash();
 }
 
-static void removeFile( const QString& trashDir, const QString& fileName )
-{
-    QDir dir;
-    dir.remove( trashDir + fileName );
-    assert( !QDir( trashDir + fileName ).exists() );
-}
-
-static void removeDir( const QString& trashDir, const QString& dirName )
-{
-    QDir dir;
-    dir.rmdir( trashDir + dirName );
-    assert( !QDir( trashDir + dirName ).exists() );
-}
 
 void TestTrash::cleanTrash()
 {
@@ -324,8 +355,8 @@ static void checkInfoFile( const QString& infoPath, const QString& origFilePath
 {
     kdDebug() << k_funcinfo << infoPath << endl;
     QFileInfo info( infoPath );
-    assert( info.isFile() );
     assert( info.exists() );
+    assert( info.isFile() );
     KSimpleConfig infoFile( info.absFilePath(), true );
     if ( !infoFile.hasGroup( "Trash Info" ) )
         kdFatal() << "no Trash Info group in " << info.absFilePath() << endl;
@@ -361,12 +392,18 @@ void TestTrash::trashFile( const QString& origFilePath, const QString& fileId )
     QMap<QString, QString> metaData;
     //bool ok = KIO::NetAccess::move( u, "trash:/" );
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
+    if ( !ok )
+        kdError() << "moving " << u << " to trash failed with error " << KIO::NetAccess::lastError() << " " << KIO::NetAccess::lastErrorString() << endl;
     assert( ok );
-    checkInfoFile( m_trashDir + "/info/" + fileId + ".trashinfo", origFilePath );
+    if ( origFilePath.startsWith( "/tmp" ) && m_tmpIsWritablePartition ) {
+        kdDebug() << " TESTS SKIPPED" << endl;
+    } else {
+        checkInfoFile( m_trashDir + "/info/" + fileId + ".trashinfo", origFilePath );
 
-    QFileInfo files( m_trashDir + "/files/" + fileId );
-    assert( files.isFile() );
-    assert( files.size() == 12 );
+        QFileInfo files( m_trashDir + "/files/" + fileId );
+        assert( files.isFile() );
+        assert( files.size() == 12 );
+    }
 
     // coolo suggests testing that the original file is actually gone, too :)
     assert( !QFile::exists( origFilePath ) );
@@ -381,7 +418,10 @@ void TestTrash::trashFile( const QString& origFilePath, const QString& fileId )
             kdDebug() << trashURL << endl;
             assert( !trashURL.isEmpty() );
             assert( trashURL.protocol() == "trash" );
-            assert( trashURL.path() == "/0-" + fileId );
+            int trashId = 0;
+            if ( origFilePath.startsWith( "/tmp" ) && m_tmpIsWritablePartition )
+                trashId = m_tmpTrashId;
+            assert( trashURL.path() == "/" + QString::number( trashId ) + "-" + fileId );
             found = true;
         }
     }
@@ -522,6 +562,10 @@ void TestTrash::trashSymlink( const QString& origFilePath, const QString& fileId
     // test
     ok = KIO::NetAccess::move( u, "trash:/" );
     assert( ok );
+    if ( origFilePath.startsWith( "/tmp" ) && m_tmpIsWritablePartition ) {
+        kdDebug() << " TESTS SKIPPED" << endl;
+        return;
+    }
     checkInfoFile( m_trashDir + "/info/" + fileId + ".trashinfo", origFilePath );
 
     QFileInfo files( m_trashDir + "/files/" + fileId );
@@ -564,6 +608,10 @@ void TestTrash::trashDirectory( const QString& origPath, const QString& fileId )
     // test
     ok = KIO::NetAccess::move( u, "trash:/" );
     assert( ok );
+    if ( origPath.startsWith( "/tmp" ) && m_tmpIsWritablePartition ) {
+        kdDebug() << " TESTS SKIPPED" << endl;
+        return;
+    }
     checkInfoFile( m_trashDir + "/info/" + fileId + ".trashinfo", origPath );
 
     QFileInfo filesDir( m_trashDir + "/files/" + fileId );
@@ -616,6 +664,10 @@ void TestTrash::delRootFile()
     assert( !file.exists() );
     QFileInfo info( m_trashDir + "/info/fileFromHome.trashinfo" );
     assert( !info.exists() );
+
+    // trash it again, we might need it later
+    const QString fileName = "fileFromHome";
+    trashFile( homeTmpDir() + fileName, fileName );
 }
 
 void TestTrash::delFileInDirectory()
@@ -649,6 +701,10 @@ void TestTrash::delDirectory()
     assert( !file.exists() );
     QFileInfo info( m_trashDir + "/info/trashDirFromHome.trashinfo" );
     assert( !info.exists() );
+
+    // trash it again, we'll need it later
+    QString dirName = "trashDirFromHome";
+    trashDirectory( homeTmpDir() + dirName, dirName );
 }
 
 void TestTrash::statRoot()
@@ -671,7 +727,7 @@ void TestTrash::statRoot()
 void TestTrash::statFileInRoot()
 {
     kdDebug() << k_funcinfo << endl;
-    KURL url( "trash:/0-fileFromOther" );
+    KURL url( "trash:/0-fileFromHome" );
     KIO::UDSEntry entry;
     bool ok = KIO::NetAccess::stat( url, entry, 0 );
     assert( ok );
@@ -682,7 +738,7 @@ void TestTrash::statFileInRoot()
     assert( item.isReadable() );
     assert( !item.isWritable() );
     assert( !item.isHidden() );
-    assert( item.name() == "fileFromOther" );
+    assert( item.name() == "fileFromHome" );
     assert( !item.acceptsDrops() );
 }
 
@@ -706,7 +762,7 @@ void TestTrash::statDirectoryInRoot()
 void TestTrash::statSymlinkInRoot()
 {
     kdDebug() << k_funcinfo << endl;
-    KURL url( "trash:/0-symlinkFromOther" );
+    KURL url( "trash:/0-symlinkFromHome" );
     KIO::UDSEntry entry;
     bool ok = KIO::NetAccess::stat( url, entry, 0 );
     assert( ok );
@@ -716,7 +772,7 @@ void TestTrash::statSymlinkInRoot()
     assert( item.isReadable() );
     assert( !item.isWritable() );
     assert( !item.isHidden() );
-    assert( item.name() == "symlinkFromOther" );
+    assert( item.name() == "symlinkFromHome" );
     assert( !item.acceptsDrops() );
 }
 
@@ -745,6 +801,8 @@ void TestTrash::copyFromTrash( const QString& fileId, const QString& destPath, c
     KURL dest;
     dest.setPath( destPath );
 
+    assert( KIO::NetAccess::exists( src, true, (QWidget*)0 ) );
+
     // A dnd would use copy(), but we use copyAs to ensure the final filename
     //kdDebug() << k_funcinfo << "copyAs:" << src << " -> " << dest << endl;
     KIO::Job* job = KIO::copyAs( src, dest );
@@ -762,8 +820,8 @@ void TestTrash::copyFromTrash( const QString& fileId, const QString& destPath, c
 void TestTrash::copyFileFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "fileFromOther_copied";
-    copyFromTrash( "fileFromOther", destPath );
+    const QString destPath = otherTmpDir() + "fileFromHome_copied";
+    copyFromTrash( "fileFromHome", destPath );
     assert( QFileInfo( destPath ).isFile() );
     assert( QFileInfo( destPath ).size() == 12 );
 }
@@ -780,16 +838,16 @@ void TestTrash::copyFileInDirectoryFromTrash()
 void TestTrash::copyDirectoryFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "trashDirFromOther_copied";
-    copyFromTrash( "trashDirFromOther", destPath );
+    const QString destPath = otherTmpDir() + "trashDirFromHome_copied";
+    copyFromTrash( "trashDirFromHome", destPath );
     assert( QFileInfo( destPath ).isDir() );
 }
 
 void TestTrash::copySymlinkFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "symlinkFromOther_copied";
-    copyFromTrash( "symlinkFromOther", destPath );
+    const QString destPath = otherTmpDir() + "symlinkFromHome_copied";
+    copyFromTrash( "symlinkFromHome", destPath );
     assert( QFileInfo( destPath ).isSymLink() );
 }
 
@@ -801,6 +859,8 @@ void TestTrash::moveFromTrash( const QString& fileId, const QString& destPath, c
     KURL dest;
     dest.setPath( destPath );
 
+    assert( KIO::NetAccess::exists( src, true, (QWidget*)0 ) );
+
     // A dnd would use move(), but we use moveAs to ensure the final filename
     KIO::Job* job = KIO::moveAs( src, dest );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
@@ -817,17 +877,21 @@ void TestTrash::moveFromTrash( const QString& fileId, const QString& destPath, c
 void TestTrash::moveFileFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "fileFromOther_restored";
-    moveFromTrash( "fileFromOther", destPath );
+    const QString destPath = otherTmpDir() + "fileFromHome_restored";
+    moveFromTrash( "fileFromHome", destPath );
     assert( QFileInfo( destPath ).isFile() );
     assert( QFileInfo( destPath ).size() == 12 );
+
+    // trash it again for later
+    const QString fileName = "fileFromHome";
+    trashFile( homeTmpDir() + fileName, fileName );
 }
 
 void TestTrash::moveFileInDirectoryFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
     const QString destPath = otherTmpDir() + "testfile_restored";
-    copyFromTrash( "trashDirFromOther", destPath, "testfile" );
+    copyFromTrash( "trashDirFromHome", destPath, "testfile" );
     assert( QFileInfo( destPath ).isFile() );
     assert( QFileInfo( destPath ).size() == 12 );
 }
@@ -835,9 +899,13 @@ void TestTrash::moveFileInDirectoryFromTrash()
 void TestTrash::moveDirectoryFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "trashDirFromOther_restored";
-    moveFromTrash( "trashDirFromOther", destPath );
+    const QString destPath = otherTmpDir() + "trashDirFromHome_restored";
+    moveFromTrash( "trashDirFromHome", destPath );
     assert( QFileInfo( destPath ).isDir() );
+
+    // trash it again, we'll need it later
+    QString dirName = "trashDirFromHome";
+    trashDirectory( homeTmpDir() + dirName, dirName );
 }
 
 void TestTrash::trashDirectoryOwnedByRoot()
@@ -873,8 +941,8 @@ void TestTrash::trashDirectoryOwnedByRoot()
 void TestTrash::moveSymlinkFromTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    const QString destPath = otherTmpDir() + "symlinkFromOther_restored";
-    moveFromTrash( "symlinkFromOther", destPath );
+    const QString destPath = otherTmpDir() + "symlinkFromHome_restored";
+    moveFromTrash( "symlinkFromHome", destPath );
     assert( QFileInfo( destPath ).isSymLink() );
 }
 
