@@ -15,7 +15,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
@@ -46,6 +46,8 @@
 #include <kapplication.h>
 #include <kservice.h>
 #include <kservicegroup.h>
+#include <kmultipledrag.h>
+#include <kurldrag.h>
 
 #include "treeview.h"
 #include "treeview.moc"
@@ -664,8 +666,14 @@ QStringList TreeView::dirList(const QString& rPath)
 
 bool TreeView::acceptDrag(QDropEvent* e) const
 {
-    return QString(e->format()).contains("application/x-kmenuedit-internal") &&
-           (e->source() == const_cast<TreeView *>(this));
+    if (e->provides("application/x-kmenuedit-internal") &&
+           (e->source() == const_cast<TreeView *>(this)))
+       return true;
+    KURL::List urls;
+    if (KURLDrag::decode(e, urls) && (urls.count() == 1) && 
+        urls[0].isLocalFile() && urls[0].path().endsWith(".desktop"))
+       return true;
+    return false;
 }
 
 
@@ -724,18 +732,62 @@ void TreeView::slotDropped (QDropEvent * e, QListViewItem *parent, QListViewItem
 {
    if(!e) return;
 
-   if (e->source() != this) return; // Only internal drags are supported atm
-
+   // get destination folder
    TreeItem *parentItem = static_cast<TreeItem*>(parent);
+   QString folder = parentItem ? parentItem->directory() : QString::null;
+   MenuFolderInfo *parentFolderInfo = parentItem ? parentItem->folderInfo() : m_rootFolder;
+
+   if (e->source() != this) 
+   {
+     // External drop
+     KURL::List urls;
+     if (!KURLDrag::decode(e, urls) || (urls.count() != 1) || !urls[0].isLocalFile())
+        return;
+     QString path = urls[0].path();
+     if (!path.endsWith(".desktop"))
+        return;
+
+     QString menuId;
+     QString result = createDesktopFile(path, &menuId, &m_newMenuIds);
+     KDesktopFile orig_df(path);
+     KDesktopFile *df = orig_df.copyTo(result);
+     df->deleteEntry("Categories"); // Don't set any categories!
+
+     KService *s = new KService(df);
+     s->setMenuId(menuId);
+
+     MenuEntryInfo *entryInfo = new MenuEntryInfo(s, df);
+
+     QString oldCaption = entryInfo->caption;
+     QString newCaption = parentFolderInfo->uniqueItemCaption(oldCaption, oldCaption);
+     entryInfo->setCaption(newCaption);
+
+     // Add file to menu
+     // m_menuFile->addEntry(folder, menuId);
+     m_menuFile->pushAction(MenuFile::ADD_ENTRY, folder, menuId);
+
+     // create the TreeItem
+     if(parentItem)
+        parentItem->setOpen(true);
+
+     // update fileInfo data
+     parentFolderInfo->add(entryInfo);
+
+     TreeItem *newItem = createTreeItem(parentItem, after, entryInfo, true);
+
+     setSelected ( newItem, true);
+     itemSelected( newItem);
+
+     m_drag = 0;
+     setLayoutDirty(parentItem);
+     return;
+   }
 
    // is there content in the clipboard?
    if (!m_drag) return;
 
    if (m_dragItem == after) return; // Nothing to do
 
-   // get destination folder
-   QString folder = parentItem ? parentItem->directory() : QString::null;
-   MenuFolderInfo *parentFolderInfo = parentItem ? parentItem->folderInfo() : m_rootFolder;
    int command = m_drag;
    if (command == MOVE_FOLDER)
    {
@@ -884,9 +936,12 @@ void TreeView::startDrag()
 
 QDragObject *TreeView::dragObject()
 {
+    m_dragPath = QString::null;
     TreeItem *item = (TreeItem*)selectedItem();
     if(item == 0) return 0;
 
+    KMultipleDrag *drag = new KMultipleDrag( this );
+
     if (item->isDirectory())
     {
        m_drag = MOVE_FOLDER;
@@ -898,6 +953,16 @@ QDragObject *TreeView::dragObject()
        m_drag = MOVE_FILE;
        m_dragInfo = 0;
        m_dragItem = item;
+       QString menuId = item->menuId();
+       m_dragPath = item->entryInfo()->service->desktopEntryPath();
+       if (!m_dragPath.isEmpty())
+          m_dragPath = locate("apps", m_dragPath);
+       if (!m_dragPath.isEmpty())
+       {
+          KURL url;
+          url.setPath(m_dragPath);
+          drag->addDragObject( new KURLDrag(url, 0));
+       }
     }
     else
     {
@@ -906,10 +971,10 @@ QDragObject *TreeView::dragObject()
        m_dragItem = item;
     }
 
-    QStoredDrag *d = new QStoredDrag("application/x-kmenuedit-internal", this);
+    drag->addDragObject( new QStoredDrag("application/x-kmenuedit-internal", 0));
     if ( item->pixmap(0) )
-        d->setPixmap(*item->pixmap(0));
-    return d;
+        drag->setPixmap(*item->pixmap(0));
+    return drag;
 }
 
 void TreeView::slotRMBPressed(QListViewItem*, const QPoint& p)
