@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <errno.h>
@@ -57,9 +57,11 @@ static const char version[] = "2.0";
 
 QCString clientApp::startup_id_str;
 bool clientApp::m_ok;
+bool s_interactive = true;
 
 static const KCmdLineOptions options[] =
 {
+   { "noninteractive", I18N_NOOP("Non interactive use: no message boxes"), 0},
    { "commands", I18N_NOOP("Show available commands"), 0},
    { "+command", I18N_NOOP("Command (see --commands)"), 0},
    { "+[URL(s)]", I18N_NOOP("Arguments for command"), 0},
@@ -273,6 +275,22 @@ static QCString konqyToReuse( const QString& url, const QString& mimetype, const
     return ret;
 }
 
+void clientApp::sendASNChange()
+{
+    KStartupInfoId id;
+    id.initId( startup_id_str );
+    KStartupInfoData data;
+    data.addPid( 0 );   // say there's another process for this ASN with unknown PID
+    data.setHostname(); // ( no need to bother to get this konqy's PID )
+    Display* dpy = qt_xdisplay();
+    if( dpy == NULL ) // we may be running without QApplication here
+        dpy = XOpenDisplay( NULL );
+    if( dpy != NULL )
+        KStartupInfo::sendChangeX( dpy, id, data );
+    if( dpy != NULL && dpy != qt_xdisplay())
+        XCloseDisplay( dpy );
+}
+
 bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, const QString & mimetype)
 {
     kdDebug( 1202 ) << "clientApp::createNewWindow " << url.url() << " mimetype=" << mimetype << endl;
@@ -319,9 +337,9 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, co
             "windowCanBeUsedForTab()", data, foundApp, foundObj, false, 3000 ) )
         {
             DCOPRef ref( foundApp, foundObj );
-            DCOPReply reply = ref.call( "newTab", url.url(), tempFile );
+            DCOPReply reply = ref.call( "newTabASN", url.url(), startup_id_str, tempFile );
             if ( reply.isValid() ) {
-                KStartupInfo::appStarted();
+                sendASNChange();
                 return true;
             }
       }
@@ -333,18 +351,7 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, co
         kdDebug( 1202 ) << "clientApp::createNewWindow using existing konqueror" << endl;
         KonquerorIface_stub konqy( appId, "KonquerorIface" );
         konqy.createNewWindowASN( url.url(), mimetype, startup_id_str, tempFile );
-        KStartupInfoId id;
-        id.initId( startup_id_str );
-        KStartupInfoData data;
-        data.addPid( 0 );   // say there's another process for this ASN with unknown PID
-        data.setHostname(); // ( no need to bother to get this konqy's PID )
-        Display* dpy = qt_xdisplay();
-        if( dpy == NULL ) // we may be running without QApplication here
-            dpy = XOpenDisplay( NULL );
-        if( dpy != NULL )
-            KStartupInfo::sendChangeX( dpy, id, data );
-        if( dpy != NULL && dpy != qt_xdisplay())
-            XCloseDisplay( dpy );
+        sendASNChange();
     }
     else
     {
@@ -404,18 +411,7 @@ bool clientApp::openProfile( const QString & profileName, const QString & url, c
   else
       konqy.createBrowserWindowFromProfileAndURLASN( profile, profileName, url, mimetype, startup_id_str );
   sleep(2); // Martin Schenk <martin@schenk.com> says this is necessary to let the server read from the socket
-  KStartupInfoId id;
-  id.initId( startup_id_str );
-  KStartupInfoData sidata;
-  sidata.addPid( 0 );   // say there's another process for this ASN with unknown PID
-  sidata.setHostname(); // ( no need to bother to get this konqy's PID )
-  Display* dpy = qt_xdisplay();
-  if( dpy == NULL ) // we may be running without QApplication here
-      dpy = XOpenDisplay( NULL );
-  if( dpy != NULL )
-      KStartupInfo::sendChangeX( dpy, id, sidata );
-  if( dpy != NULL && dpy != qt_xdisplay())
-      XCloseDisplay( dpy );
+  sendASNChange();
   return true;
 }
 
@@ -446,6 +442,9 @@ bool clientApp::doIt()
   int argc = args->count();
   checkArgumentCount(argc, 1, 0);
 
+  if ( !args->isSet( "ninteractive" ) ) {
+      s_interactive = false;
+  }
   QCString command = args->arg(0);
 
   // read ASN env. variable for non-KApp cases
@@ -512,7 +511,6 @@ bool clientApp::doIt()
     }
     else if ( argc == 2 )
     {
-      KFileOpenWithHandler fowh;
       KRun * run = new KRun( args->url(1) );
       QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
       QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
@@ -523,9 +521,9 @@ bool clientApp::doIt()
     {
       KURL::List urls;
       urls.append( args->url(1) );
-      KService::Ptr serv = (*KTrader::self()->query( QString::fromLocal8Bit(args->arg(2)) ).begin());
-      if (!serv) return 1;
-      KFileOpenWithHandler fowh;
+      const KTrader::OfferList offers = KTrader::self()->query( QString::fromLocal8Bit( args->arg( 2 ) ), QString::fromLatin1( "Application" ), QString::null, QString::null );
+      if (offers.isEmpty()) return 1;
+      KService::Ptr serv = offers.first();
       return KRun::run( *serv, urls );
     }
   }
@@ -537,6 +535,8 @@ bool clientApp::doIt()
       srcLst.append( args->url(i) );
 
     KIO::Job * job = KIO::move( srcLst, args->url(argc - 1) );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -565,11 +565,13 @@ bool clientApp::doIt()
        return m_ok;
     QString dst =
        KFileDialog::getSaveFileName( (argc<2) ? (QString::null) : (args->url(1).filename()) );
-    if (dst == QString::null)
+    if (dst.isEmpty()) // cancelled
        return m_ok; // AK - really okay?
     KURL dsturl;
     dsturl.setPath( dst );
     KIO::Job * job = KIO::copy( srcLst, dsturl );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -582,6 +584,8 @@ bool clientApp::doIt()
       srcLst.append( args->url(i) );
 
     KIO::Job * job = KIO::copy( srcLst, args->url(argc - 1) );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -618,7 +622,7 @@ bool clientApp::doIt()
 
 void clientApp::slotResult( KIO::Job * job )
 {
-  if (job->error())
+  if (job->error() && s_interactive)
     job->showErrorDialog();
   m_ok = !job->error();
   quit();
