@@ -290,10 +290,7 @@ void KEBApp::setActionsEnabled(SelcAbilities sa) {
                 toEnable << "edit_paste";
 
         if( sa.multiSelect || (sa.singleSelect && !sa.root && (sa.group || !sa.urlIsEmpty) && !sa.separator))
-            toEnable << "testlink" ;
-
-        if( sa.multiSelect || (sa.singleSelect && !sa.root && !sa.urlIsEmpty && !sa.group && !sa.separator))
-            toEnable  << "updatefavicon";
+            toEnable << "testlink" << "updatefavicon";
 
         if( sa.itemSelected)
             toEnable << "showintoolbar";
@@ -345,7 +342,6 @@ void ActionsImpl::slotCopy() {
     QValueList<KBookmark> bookmarks
         = ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
     KBookmarkDrag* data = KBookmarkDrag::newDrag(bookmarks, 0 /* not this ! */);
-    kapp->clipboard()->setData(data, QClipboard::Selection);
     kapp->clipboard()->setData(data, QClipboard::Clipboard);
 }
 
