@@ -1,4 +1,4 @@
-// -*- mode:cperl; cperl-indent-level:4; cperl-continued-statement-offset:4; indent-tabs-mode:nil -*-
+// -*- indent-tabs-mode:nil -*-
 // vim: set ts=4 sts=4 sw=4 et:
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "listview.h"
@@ -25,6 +25,7 @@
 #include "bookmarkinfo.h"
 #include "commands.h"
 #include "testlink.h"
+#include "settings.h"
 
 #include <stdlib.h>
 
@@ -32,6 +33,7 @@
 #include <qpopupmenu.h>
 #include <qpainter.h>
 #include <qheader.h>
+#include <qvaluevector.h>
 
 #include <klocale.h>
 #include <dcopclient.h>
@@ -46,6 +48,7 @@
 #include <kmessagebox.h>
 #include <klineedit.h>
 #include <krun.h>
+#include <klistviewsearchline.h>
 
 #include <kbookmarkdrag.h>
 #include <kbookmarkmanager.h>
@@ -57,11 +60,12 @@ ListView* ListView::s_self = 0;
 int ListView::s_myrenamecolumn = -1;
 KEBListViewItem *ListView::s_myrenameitem = 0;
 
-QStringList ListView::s_selected_addresses; // UGLY
-bool ListView::s_listview_is_dirty = false;
+QStringList ListView::s_selected_addresses;
+QString ListView::s_current_address;
 
-ListView::ListView() {
-    m_splitView = KEBApp::self()->splitView();
+ListView::ListView()
+    : m_needToFixUp(false)
+{
 }
 
 ListView::~ListView() {
@@ -70,101 +74,214 @@ ListView::~ListView() {
 
 void ListView::createListViews(QSplitter *splitter) {
     s_self = new ListView();
-    self()->m_folderListView = self()->m_splitView ? new KEBListView(splitter, true) : 0;
     self()->m_listView = new KEBListView(splitter, false);
     splitter->setSizes(QValueList<int>() << 100 << 300);
 }
 
 void ListView::initListViews() {
     self()->m_listView->init();
-    if (m_splitView)
-        self()->m_folderListView->init();
 }
 
 void ListView::updateListViewSetup(bool readonly) {
     self()->m_listView->readonlyFlagInit(readonly);
-    if (m_splitView)
-        self()->m_folderListView->readonlyFlagInit(readonly);
-}
-
-void ListView::setInitialAddress(QString address) {
-    m_last_selection_address = address;
 }
 
 void ListView::connectSignals() {
     m_listView->makeConnections();
-    if (m_splitView)
-        m_folderListView->makeConnections();
 }
 
-QValueList<KBookmark> ListView::itemsToBookmarks(QPtrList<KEBListViewItem>* items) const {
-    QValueList<KBookmark> bookmarks;
-    for (QPtrListIterator<KEBListViewItem> it(*items); it.current() != 0; ++it)
-        bookmarks.append(KBookmark(it.current()->bookmark()));
+bool lessAddress(QString a, QString b)
+{
+    if(a == b)
+         return false;
+
+    QString error("ERROR");
+    if(a == error)
+        return false;
+    if(b == error)
+        return true;
+
+    a += "/";
+    b += "/";
+
+    uint aLast = 0;
+    uint bLast = 0;
+    uint aEnd = a.length();
+    uint bEnd = b.length();
+    // Each iteration checks one "/"-delimeted part of the address
+    // "" is treated correctly
+    while(true)
+    {
+        // Invariant: a[0 ... aLast] == b[0 ... bLast]
+        if(aLast + 1 == aEnd) //The last position was the last slash
+            return true; // That means a is shorter than b
+        if(bLast +1 == bEnd)
+            return false;
+
+        uint aNext = a.find("/", aLast + 1);
+        uint bNext = b.find("/", bLast + 1);
+
+        bool okay;
+        uint aNum = a.mid(aLast + 1, aNext - aLast - 1).toUInt(&okay);
+        if(!okay)
+            return false;
+        uint bNum = b.mid(bLast + 1, bNext - bLast - 1).toUInt(&okay);
+        if(!okay)
+            return true;
+
+        if(aNum != bNum)
+            return aNum < bNum;
+
+        aLast = aNext;
+        bLast = bNext;
+    }
+}
+
+bool operator<(const KBookmark & first, const KBookmark & second) //FIXME Using internal represantation
+{
+    return lessAddress(first.address(), second.address());
+}
+
+
+
+QValueList<KBookmark> ListView::itemsToBookmarks(const QMap<KEBListViewItem *, bool> & items) const
+{
+    QValueList<KBookmark> bookmarks; //TODO optimize by using a QValueVector
+    QMap<KEBListViewItem *, bool>::const_iterator it = items.begin();
+    QMap<KEBListViewItem *, bool>::const_iterator end = items.end();
+    for( ; it!=end; ++it)
+    {
+        if(it.key() != m_listView->rootItem() )
+            bookmarks.push_back( it.key()->bookmark() );
+    }
+    qHeapSort(bookmarks);
     return bookmarks;
 }
 
-#define VALID_ITEM(item)                                            \
-    ( !item->isEmptyFolderPadder()                                  \
-   && (item->bookmark().hasParent() || item->parent() == 0) )
-
-#define VALID_FIRST(items) ( (items->count() > 0) && VALID_ITEM(items->first()) )
-
-QPtrList<KEBListViewItem>* ListView::selectedItems() const {
-    static QPtrList<KEBListViewItem>* s_selected_items_cache = 0;
-    if (!s_selected_items_cache || s_listview_is_dirty) {
-        if (!s_selected_items_cache)
-            s_selected_items_cache = new QPtrList<KEBListViewItem>();
-        else
-            s_selected_items_cache->clear();
-
-        for (QListViewItemIterator it(m_listView); it.current() != 0; ++it) 
-        {
-            if(VALID_ITEM( static_cast<KEBListViewItem *>(it.current())) && it.current()->isSelected())
-                s_selected_items_cache->append(static_cast<KEBListViewItem *>(it.current()));
-        }
-    }
-    s_listview_is_dirty = false;
-    return s_selected_items_cache;
+void ListView::invalidate(const QString & address)
+{
+    invalidate(getItemAtAddress(address));
 }
 
-void ListView::deselectAllButParent(KEBListViewItem *item) {
+void ListView::invalidate(QListViewItem * item)
+{
+    if(item->isSelected())
+    {
+        m_listView->setSelected(item, false);
+        m_needToFixUp = true;
+    }
+
+    if(m_listView->currentItem() == item)
+    {
+        // later overiden by fixUpCurrent
+        m_listView->setCurrentItem(m_listView->rootItem()); 
+        m_needToFixUp = true;
+    }
+
+    QListViewItem * child = item->firstChild();
+    while(child)
+    {
+        //invalidate(child);
+        child = child->nextSibling();
+    }
+}
+
+void ListView::fixUpCurrent(const QString & address)
+{
+    if(!m_needToFixUp)
+        return;
+    m_needToFixUp = false;
+
+    QListViewItem * item;
+    if(mSelectedItems.count() != 0)
+    {
+        QString least = mSelectedItems.begin().key()->bookmark().address();
+        QMap<KEBListViewItem *, bool>::iterator it, end;
+        end = mSelectedItems.end();
+        for(it = mSelectedItems.begin(); it != end; ++it)
+            if( lessAddress(it.key()->bookmark().address(), least))
+                least = it.key()->bookmark().address();
+        item = getItemAtAddress(least);
+    }
+    else
+        item  = getItemAtAddress(address);
+    m_listView->setSelected( item, true );
+    m_listView->setCurrentItem( item );
+}
+
+
+void ListView::selected(KEBListViewItem * item, bool s)
+{
+    Q_ASSERT(item->bookmark().hasParent() || item == m_listView->rootItem());
+    QMap<KEBListViewItem *, bool>::iterator it;
+
+    if(s)
+        mSelectedItems[item] = item;
+    else
+        if((it = mSelectedItems.find(item)) != mSelectedItems.end())
+            mSelectedItems.remove(it);
+
+    KEBApp::self()->updateActions();
+
+    if (mSelectedItems.count() != 1)
+    {
+        KEBApp::self()->bkInfo()->showBookmark(KBookmark());
+        return;
+    }
+    //FIXME do it once somewhere
+    if (!KEBApp::self()->bkInfo()->connected()) {
+        connect(KEBApp::self()->bkInfo(), SIGNAL( updateListViewItem() ),
+                                          SLOT( slotBkInfoUpdateListViewItem() ));
+        KEBApp::self()->bkInfo()->setConnected(true);
+    }
+
+    KEBApp::self()->bkInfo()->showBookmark(firstSelected()->bookmark());
+    firstSelected()->modUpdate();
+}
+
+KEBListViewItem * ListView::firstSelected() const
+{
+    if(mSelectedItems.isEmpty())
+        return 0L;
+    else
+        return mSelectedItems.begin().key();
+}
+
+void ListView::deselectAllChildren(KEBListViewItem *item) 
+{
     KEBListViewItem* child = static_cast<KEBListViewItem *>(item->firstChild());
     while(child)
     {
-        if (VALID_ITEM(child) && child->isSelected())
-            child->listView()->setSelected(child, false);
-        if(child->childCount())
-            deselectAllButParent(child);
+        if (child)
+        {
+            if(child->isSelected())
+                child->listView()->setSelected(child, false); //calls deselectAllChildren
+            else
+                deselectAllChildren(child);
+        }
+        child->repaint();
         child = static_cast<KEBListViewItem *>(child->nextSibling());
     }
 }
 
-void ListView::updateSelectedItems() {
-
-    m_listView->setSelected(m_listView->firstChild(), false); //Deselect root ("Bookmarks")
-
-    // adjust the current selection
-    QListViewItemIterator it(m_listView);
-    for ( ; it.current() != 0; ++it) {
-        if( !VALID_ITEM( static_cast<KEBListViewItem *>(it.current())) 
-            || !it.current()->isSelected() )
-            continue;
-        // don't bother looking into it if its not a folder
-        if (it.current()->childCount() == 0)
-            continue;
-        deselectAllButParent(static_cast<KEBListViewItem *>(it.current()));
-    }
+QValueList<QString> ListView::selectedAddresses()
+{
+    QValueList<QString> addresses;
+    QValueList<KBookmark> bookmarks = itemsToBookmarks( selectedItemsMap() );
+    QValueList<KBookmark>::const_iterator it, end;
+    end = bookmarks.end();
+    for( it = bookmarks.begin(); it != end; ++it)
+        addresses.append( (*it).address() );
+    return addresses;
 }
 
-#define NOT_FOLDER_OR_EMPTY(item) (!item->isEmptyFolderPadder() && (item->childCount() == 0))
 
 QValueList<KBookmark> ListView::selectedBookmarksExpanded() const {
     QValueList<KBookmark> bookmarks;
     for (QListViewItemIterator it(m_listView); it.current() != 0; ++it) {
-        if (!it.current()->isSelected() 
-         || !VALID_ITEM(static_cast<KEBListViewItem *>(it.current()))
-         || it.current() == m_listView->rootItem())
+        if (!it.current()->isSelected())
+            continue;
+        if(it.current() == m_listView->firstChild()) // root case
             continue;
         if (it.current()->childCount() == 0) // non folder case
             bookmarks.append(static_cast<KEBListViewItem *>(it.current())->bookmark());
@@ -174,15 +291,14 @@ QValueList<KBookmark> ListView::selectedBookmarksExpanded() const {
     return bookmarks;
 }
 
+
 void ListView::selectedBookmarksExpandedHelper(KEBListViewItem * item, QValueList<KBookmark> & bookmarks) const
 {
     KEBListViewItem* child = static_cast<KEBListViewItem *>(item->firstChild());
     while( child )
     {
-        if (NOT_FOLDER_OR_EMPTY(child))
-        {
+        if (!child->isEmptyFolderPadder() && (child->childCount() == 0))
             bookmarks.append(child->bookmark());
-        }
         if(child->childCount())
             selectedBookmarksExpandedHelper(child, bookmarks);
         child = static_cast<KEBListViewItem *>(child->nextSibling());
@@ -192,41 +308,35 @@ void ListView::selectedBookmarksExpandedHelper(KEBListViewItem * item, QValueLis
 QValueList<KBookmark> ListView::allBookmarks() const {
     QValueList<KBookmark> bookmarks;
     for (QListViewItemIterator it(m_listView); it.current() != 0; ++it)
-        if (NOT_FOLDER_OR_EMPTY(static_cast<KEBListViewItem *>(it.current())))
+    {
+        KEBListViewItem * item = static_cast<KEBListViewItem *>(it.current());
+        if (!item->isEmptyFolderPadder() && (item->childCount() == 0))
             bookmarks.append(static_cast<KEBListViewItem *>(it.current())->bookmark());
+    }
     return bookmarks;
 }
 
 // DESIGN - make + "/0" a kbookmark:: thing?
 
-QString ListView::userAddress() const {
-    if (selectedItems()->isEmpty()) {
-        // FIXME - maybe a in view one?
-        //       - else we could get /0
-        //       - in view?
-        return "/0";
-    }
-
-    KEBListViewItem *item = selectedItems()->first();
-    if (item->isEmptyFolderPadder())
-        item = static_cast<KEBListViewItem*>(item->parent());
-
-    KBookmark current = item->bookmark();
-    if (!current.hasParent())
-        return "/0";
-
+QString ListView::userAddress() const 
+{
+    KBookmark current = firstSelected()->bookmark();
     return (current.isGroup()) 
-        ? (current.address() + "/0")
+        ? (current.address() + "/0") //FIXME internal represantation used
         : KBookmark::nextAddress(current.address());
 }
 
-void ListView::setCurrent(KEBListViewItem *item) {
-    // for the moment listview(1) is all that matters
+void ListView::setCurrent(KEBListViewItem *item, bool select) {
     m_listView->setCurrentItem(item);
-    m_listView->ensureItemVisible(item);
+    if(select)
+    {
+        m_listView->clearSelection();
+        m_listView->setSelected(item, true);
+    }
 }
 
 KEBListViewItem* ListView::getItemAtAddress(const QString &address) const {
+    //FIXME uses internal represantation of bookmark address
     QListViewItem *item = m_listView->rootItem();
 
     QStringList addresses = QStringList::split('/',address); // e.g /5/10/2
@@ -250,32 +360,25 @@ void ListView::setOpen(bool open) {
 SelcAbilities ListView::getSelectionAbilities() const {
     SelcAbilities sa = { false, false, false, false, false, false, false, false, false };
 
-    if (VALID_FIRST(selectedItems())) {
-        KBookmark nbk = selectedItems()->first()->bookmark();
+    if (mSelectedItems.count() > 0) {
+        KBookmark nbk = firstSelected()->bookmark();
         sa.itemSelected   = true;
         sa.group          = nbk.isGroup();
         sa.separator      = nbk.isSeparator();
         sa.urlIsEmpty     = nbk.url().isEmpty();
-        sa.root           = (selectedItems()->first() == m_listView->rootItem());
+        sa.root           = (firstSelected() == m_listView->rootItem());
+        sa.multiSelect    = (mSelectedItems.count() > 1);
         sa.singleSelect   = (!sa.multiSelect && sa.itemSelected);
-        sa.multiSelect    = (selectedItems()->count() > 1);
-        sa.tbShowState    = CmdGen::self()->shownInToolbar(nbk);
+        sa.tbShowState    = CmdGen::shownInToolbar(nbk);
     }
 
     sa.notEmpty = (m_listView->rootItem()->childCount() > 0);
-    
+
     return sa;
 }
 
-void ListView::handleDropped(KEBListView *lv, QDropEvent *e, QListViewItem *newParent, QListViewItem *itemAfterQLVI) {
-    bool inApp = (e->source() == m_listView->viewport())
-        || (m_folderListView && e->source() == m_folderListView->viewport());
-    bool toOther = e->source() != lv->viewport();
-
-    Q_UNUSED(toOther);
-
-    if (m_splitView)
-        return;
+void ListView::handleDropped(KEBListView *, QDropEvent *e, QListViewItem *newParent, QListViewItem *itemAfterQLVI) {
+    bool inApp = (e->source() == m_listView->viewport());
 
     // drop before root item
     if (!newParent)
@@ -288,16 +391,16 @@ void ListView::handleDropped(KEBListView *lv, QDropEvent *e, QListViewItem *newP
         ? (static_cast<KEBListViewItem *>(newParent)->bookmark().address() + "/0")
         : (KBookmark::nextAddress(itemAfter->bookmark().address()));
 
-    KMacroCommand *mcmd = 0;
+    KEBMacroCommand *mcmd = 0;
 
     if (!inApp) {
-        mcmd = CmdGen::self()->insertMimeSource(i18n("Drop Items"), e, newAddress);
+        mcmd = CmdGen::insertMimeSource(i18n("Drop Items"), e, newAddress);
 
     } else {
-        if (!VALID_FIRST(selectedItems()) || (selectedItems()->first() == itemAfterQLVI))
+        if (!(mSelectedItems.count() > 0) || (firstSelected() == itemAfterQLVI))
             return;
         bool copy = (e->action() == QDropEvent::Copy);
-        mcmd = CmdGen::self()->itemsMoved(selectedItems(), newAddress, copy);
+        mcmd = CmdGen::itemsMoved(selectedItemsMap(), newAddress, copy);
     }
 
     CmdHistory::self()->didCommand(mcmd);
@@ -305,51 +408,57 @@ void ListView::handleDropped(KEBListView *lv, QDropEvent *e, QListViewItem *newP
 
 void ListView::updateStatus(QString url) {
     m_listView->updateByURL(url);
-    handleSelectionChanged(0);
 }
 
-void ListView::updateListView() {
-    s_selected_addresses.clear();
-    QPtrList<KEBListViewItem> *selcItems = selectedItems();
-    for (QPtrListIterator<KEBListViewItem> it(*selcItems); it.current() != 0; ++it)
-        if (it.current()->bookmark().hasParent())
-            s_selected_addresses << it.current()->bookmark().address();
+void ListView::updateListView() 
+{
+    // this is upper border of the visible are
     int lastCurrentY = m_listView->contentsY();
+
+    //Save selected items (restored in fillWithGroup)
+    s_selected_addresses.clear();
+    QMap<KEBListViewItem *, bool>::const_iterator it, end;
+    it = mSelectedItems.begin();
+    end = mSelectedItems.end();
+    for ( ; it != end; ++it)
+        s_selected_addresses << it.key()->bookmark().address();
+    if(m_listView->currentItem())
+    {
+        KEBListViewItem * item = static_cast<KEBListViewItem*>(m_listView->currentItem());
+        if(item->isEmptyFolderPadder())
+            s_current_address = static_cast<KEBListViewItem*>(item->parent())->bookmark().address();
+        else
+            s_current_address = item->bookmark().address();
+    }
+    else
+        s_current_address = QString::null;
+
     updateTree();
-    if (selectedItems()->isEmpty() && m_listView->currentItem())
-        m_listView->setSelected(m_listView->currentItem(), true);
-    m_listView->ensureVisible(0, lastCurrentY, 0, 0);
-    m_listView->ensureVisible(0, lastCurrentY + m_listView->visibleHeight(), 0, 0);
+    m_searchline->updateSearch();
+
+    // ensureVisible wants to have the midpoint of the new visible area
+    m_listView->ensureVisible(0, lastCurrentY + m_listView->visibleHeight() / 2, 0, m_listView->visibleHeight() / 2 );
 }
 
-KEBListViewItem *s_lazySettingCurrentItem;
-
-void ListView::updateTree(bool updateSplitView) {
+void ListView::updateTree() {
     KBookmarkGroup root = CurrentMgr::self()->mgr()->root();
-    if (m_splitView)
-        root = CurrentMgr::bookmarkAt(m_currentSelectedRootAddress).toGroup();
     fillWithGroup(m_listView, root);
-    if (m_splitView && updateSplitView)
-        fillWithGroup(m_folderListView, CurrentMgr::self()->mgr()->root());
-    s_listview_is_dirty = true;
-    if (s_lazySettingCurrentItem)
-        setCurrent(s_lazySettingCurrentItem);
-    s_lazySettingCurrentItem = 0;
 }
 
 void ListView::fillWithGroup(KEBListView *lv, KBookmarkGroup group, KEBListViewItem *parentItem) {
     KEBListViewItem *lastItem = 0;
-    if (!parentItem) {
+    if (!parentItem) 
+    {
         lv->clear();
-        if (!m_splitView || lv->isFolderList()) {
-            KEBListViewItem *tree = new KEBListViewItem(lv, group);
-            fillWithGroup(lv, group, tree);
-            tree->QListViewItem::setOpen(true);
-            return;
-        }
+        KEBListViewItem *tree = new KEBListViewItem(lv, group);
+        fillWithGroup(lv, group, tree);
+        tree->QListViewItem::setOpen(true);
+        if (s_selected_addresses.contains(tree->bookmark().address()))
+            lv->setSelected(tree, true);
+        if(!s_current_address.isNull() && s_current_address == tree->bookmark().address())
+            lv->setCurrentItem(tree);
+        return;
     }
-    if (m_splitView && !lv->isFolderList())
-        lastItem = new KEBListViewItem(lv, lastItem, group);
     for (KBookmark bk = group.first(); !bk.isNull(); bk = group.next(bk)) {
         KEBListViewItem *item = 0;
         if (bk.isGroup()) {
@@ -357,16 +466,16 @@ void ListView::fillWithGroup(KEBListView *lv, KBookmarkGroup group, KEBListViewI
             item = (parentItem)
                 ? new KEBListViewItem(parentItem, lastItem, grp)
                 : new KEBListViewItem(lv, lastItem, grp);
-            if (!(m_splitView && !lv->isFolderList())) {
-                fillWithGroup(lv, grp, item);
-                if (grp.isOpen())
-                    item->QListViewItem::setOpen(true);
-                if (!m_splitView && grp.first().isNull())
-                    new KEBListViewItem(item, item); // empty folder
-            }
+            fillWithGroup(lv, grp, item);
+            if (grp.isOpen())
+                item->QListViewItem::setOpen(true);
+            if (grp.first().isNull())
+                new KEBListViewItem(item, item); // empty folder
             lastItem = item;
 
-        } else if (!(lv->isFolderList() && m_splitView)) {
+        }
+        else
+        {
             item = (parentItem)   
                 ? ( (lastItem)
                         ? new KEBListViewItem(parentItem, lastItem, bk)
@@ -378,10 +487,8 @@ void ListView::fillWithGroup(KEBListView *lv, KBookmarkGroup group, KEBListViewI
         }
         if (s_selected_addresses.contains(bk.address()))
             lv->setSelected(item, true);
-	    // TODO this is far too exact, we need to do partial + best match system
-        QString correctedCurrentAddr = CurrentMgr::self()->correctAddress(m_last_selection_address);
-        if (bk.address() == correctedCurrentAddr)
-            s_lazySettingCurrentItem = item;
+        if(!s_current_address.isNull() && s_current_address == bk.address())
+            lv->setCurrentItem(item);
     }
 }
 
@@ -394,62 +501,17 @@ void ListView::handleMoved(KEBListView *) {
      */
 }
 
-void ListView::handleCurrentChanged(KEBListView *lv, QListViewItem *item) {
-    if (!item)
-        return;
-
-    KEBListViewItem *currentItem = static_cast<KEBListViewItem *>(item);
-
-    // note, for some reason just doing count() > 0 means that the
-    // selectedItems list seems to follow the old current. *realy wierd*
-    if (selectedItems()->count() > 1 && VALID_FIRST(selectedItems()))
-        m_last_selection_address = selectedItems()->first()->bookmark().address();
-    else if (VALID_ITEM(currentItem))
-        m_last_selection_address = currentItem->bookmark().address();
-
-    if (m_splitView && lv == m_folderListView) {
-        m_folderListView->setSelected(item, true);
-        QString addr = currentItem->bookmark().address();
-        if (addr != m_currentSelectedRootAddress) {
-            m_currentSelectedRootAddress = addr;
-            updateTree(false);
-        }
-    }
-}
-
-void ListView::handleSelectionChanged(KEBListView *) {
-    s_listview_is_dirty = true;
-    updateSelectedItems();
-    KEBApp::self()->updateActions();
-
-    if (selectedItems()->count() != 1
-      || !VALID_FIRST(selectedItems())
-    ) {
-        KEBApp::self()->bkInfo()->showBookmark(KBookmark());
-        return;
-    }
-    if (!KEBApp::self()->bkInfo()->connected()) {
-        connect(KEBApp::self()->bkInfo(), SIGNAL( updateListViewItem() ),
-                                          SLOT( slotBkInfoUpdateListViewItem() ));
-        KEBApp::self()->bkInfo()->setConnected(true);
-    }
-
-    KEBApp::self()->bkInfo()->showBookmark(selectedItems()->first()->bookmark());
-    selectedItems()->first()->modUpdate();
-}
-
 void ListView::slotBkInfoUpdateListViewItem() {
     // its not possible that the selection changed inbetween as 
     // handleSelectionChanged is the one that sets up bkInfo() 
     // to emit this signal, otoh. maybe this can cause various
     // differing responses.
     // kdDebug() << "slotBkInfoUpdateListViewItem()" << endl;
-    KEBApp::self()->setModifiedFlag(true);
-    KEBListViewItem *i = selectedItems()->first();
+    KEBListViewItem *i = firstSelected();
     Q_ASSERT(i);
     KBookmark bk = i->bookmark();
     i->setText(KEBListView::NameColumn, bk.fullText());
-    i->setText(KEBListView::UrlColumn, bk.url().url());
+    i->setText(KEBListView::UrlColumn, bk.url().pathOrURL());
     QString commentStr = NodeEditCommand::getNodeText(bk, QStringList() << "desc");
     i->setText(KEBListView::CommentColumn, commentStr);
 }
@@ -483,8 +545,11 @@ void ListView::handleItemRenamed(KEBListView *lv, QListViewItem *item, const QSt
         }
 
     } else if (column == KEBListView::UrlColumn && !lv->isFolderList()) {
-        if (bk.url() != newText)
-            cmd = new EditCommand(bk.address(), EditCommand::Edition("href", newText), i18n("URL"));
+        if (bk.url().pathOrURL() != newText)
+        {
+            KURL u = KURL::fromPathOrURL(newText);
+            cmd = new EditCommand(bk.address(), EditCommand::Edition("href", u.url(0, 106)), i18n("URL"));
+        }
 
     } else if (column == KEBListView::CommentColumn && !lv->isFolderList()) {
         if (NodeEditCommand::getNodeText(bk, "desc") != newText)
@@ -495,9 +560,7 @@ void ListView::handleItemRenamed(KEBListView *lv, QListViewItem *item, const QSt
 
 // used by f2 and f3 shortcut slots - see actionsimpl
 void ListView::rename(int column) {
-    // TODO - check which listview has focus
-    assert( VALID_FIRST(selectedItems()) );
-    m_listView->rename(selectedItems()->first(), column);
+    m_listView->rename(firstSelected(), column);
 }
 
 void ListView::clearSelection() {
@@ -510,13 +573,6 @@ void ListView::startRename(int column, KEBListViewItem *item) {
 }
 
 void ListView::renameNextCell(bool fwd) {
-    // this needs to take special care
-    // of the current listview focus!
-    // but for the moment we just default
-    // to using the item listview
-    // in fact, because the two are so 
-    // different they each need to be 
-    // handled almost completely differently...
     KEBListView *lv = m_listView;
     while (1) {
         if (fwd && s_myrenamecolumn < KEBListView::CommentColumn) {
@@ -550,8 +606,9 @@ void ListView::renameNextCell(bool fwd) {
 
 class KeyPressEater : public QObject {
     public:
-        KeyPressEater( QWidget *parent = 0, const char *name = 0 ) { 
-            m_allowedToTab = true; 
+        KeyPressEater( QWidget *parent = 0, const char *name = 0 )
+            : QObject(parent, name) {
+            m_allowedToTab = true;
         }
     protected:
         bool eventFilter(QObject *, QEvent *);
@@ -580,14 +637,12 @@ bool KeyPressEater::eventFilter(QObject *, QEvent *pe) {
 
 void KEBListView::loadColumnSetting() 
 {
-    KConfig config("keditbookmarksrc", false, false);
-    config.setGroup("Columns");
-    header()->resizeSection(KEBListView::NameColumn, config.readNumEntry("Name", 300));
-    header()->resizeSection(KEBListView::UrlColumn, config.readNumEntry("URL", 300));
-    header()->resizeSection(KEBListView::CommentColumn, config.readNumEntry("Comment", 300));
-    header()->resizeSection(KEBListView::StatusColumn, config.readNumEntry("Status", 300));
+    header()->resizeSection(KEBListView::NameColumn, KEBSettings::name());
+    header()->resizeSection(KEBListView::UrlColumn, KEBSettings::uRL());
+    header()->resizeSection(KEBListView::CommentColumn, KEBSettings::comment());
+    header()->resizeSection(KEBListView::StatusColumn, KEBSettings::status());
 #ifdef DEBUG_ADDRESSES
-    header()->resizeSection(KEBListView::AddressColumn, config.readNumEntry("Address", 300));
+    header()->resizeSection(KEBListView::AddressColumn, KEBSettings::address());
 #endif
     m_widthsDirty = false;
 }
@@ -595,15 +650,14 @@ void KEBListView::loadColumnSetting()
 void KEBListView::saveColumnSetting () 
 {
     if (m_widthsDirty) {
-        KConfig config("keditbookmarksrc", false, false);
-        config.setGroup("Columns");
-        config.writeEntry("Name", header()->sectionSize(KEBListView::NameColumn));
-        config.writeEntry("URL", header()->sectionSize(KEBListView::UrlColumn));
-        config.writeEntry("Comment", header()->sectionSize(KEBListView::CommentColumn));
-        config.writeEntry("Status", header()->sectionSize(KEBListView::StatusColumn));
+        KEBSettings::setName( header()->sectionSize(KEBListView::NameColumn));
+        KEBSettings::setURL( header()->sectionSize(KEBListView::UrlColumn));
+        KEBSettings::setComment( header()->sectionSize(KEBListView::CommentColumn));
+        KEBSettings::setStatus( header()->sectionSize(KEBListView::StatusColumn));
 #ifdef DEBUG_ADDRESSES
-        config.writeEntry("Address", header()->sectionSize(KEBListView::AddressColumn));
+        KEBSettings::setAddress( header()->sectionSize(KEBListView::AddressColumn));
 #endif
+        KEBSettings::writeConfig();
     }
 }
 
@@ -641,10 +695,6 @@ void KEBListView::init() {
 void KEBListView::makeConnections() {
     connect(this, SIGNAL( moved() ),
             SLOT( slotMoved() ));
-    connect(this, SIGNAL( selectionChanged() ),
-            SLOT( slotSelectionChanged() ));
-    connect(this, SIGNAL( currentChanged(QListViewItem *) ),
-            SLOT( slotCurrentChanged(QListViewItem *) ));
     connect(this, SIGNAL( contextMenu(KListView *, QListViewItem*, const QPoint &) ),
             SLOT( slotContextMenu(KListView *, QListViewItem *, const QPoint &) ));
     connect(this, SIGNAL( itemRenamed(QListViewItem *, const QString &, int) ),
@@ -664,10 +714,6 @@ void KEBListView::readonlyFlagInit(bool readonly) {
 
 void KEBListView::slotMoved() 
 { ListView::self()->handleMoved(this); }
-void KEBListView::slotSelectionChanged() 
-{ ListView::self()->handleSelectionChanged(this); }
-void KEBListView::slotCurrentChanged(QListViewItem *a) 
-{ ListView::self()->handleCurrentChanged(this, a); }
 void KEBListView::slotContextMenu(KListView *a, QListViewItem *b, const QPoint &c) 
 { ListView::self()->handleContextMenu(this, a,b,c); }
 void KEBListView::slotItemRenamed(QListViewItem *a, const QString &b, int c) 
@@ -714,14 +760,8 @@ bool KEBListView::acceptDrag(QDropEvent * e) const {
 }
 
 QDragObject *KEBListView::dragObject() {
-    if (!VALID_FIRST(ListView::self()->selectedItems())) {
-        // we handle empty folders here as a special
-        // case for drag & drop in order to allow 
-        // for pasting into a "empty folder"
-        return (QDragObject*)0;
-    }
     QValueList<KBookmark> bookmarks = 
-        ListView::self()->itemsToBookmarks(ListView::self()->selectedItems());
+        ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
     KBookmarkDrag *drag = KBookmarkDrag::newDrag(bookmarks, viewport());
     const QString iconname = 
         (bookmarks.size() == 1) ? bookmarks.first().icon() : QString("bookmark");
@@ -731,12 +771,52 @@ QDragObject *KEBListView::dragObject() {
 
 /* -------------------------------------- */
 
+bool KEBListViewItem::parentSelected(QListViewItem * item)
+{
+    QListViewItem *root = item->listView()->firstChild();
+    for( QListViewItem *parent = item->parent(); parent ; parent = parent->parent())
+        if (parent->isSelected() && parent != root)
+            return true;
+    return false;
+}
+
+void KEBListViewItem::setSelected(bool s)
+{
+    if( isEmptyFolderPadder())
+    {
+        parent()->setSelected(true);
+        return;
+    }
+
+    if(listView()->firstChild() == this)
+    {
+        ListView::self()->selected(this, s);
+        QListViewItem::setSelected( s );
+        return;
+    }
+
+    if(s == false)
+    {
+        ListView::self()->selected(this, false);
+        QListViewItem::setSelected( false );
+        ListView::deselectAllChildren( this ); //repaints
+    }
+    else if(parentSelected(this))
+        return;
+    else
+    {
+        ListView::self()->selected(this, true);
+        QListViewItem::setSelected( true );
+        ListView::deselectAllChildren(this);
+    }
+}
+
 void KEBListViewItem::normalConstruct(const KBookmark &bk) {
 #ifdef DEBUG_ADDRESSES
     setText(KEBListView::AddressColumn, bk.address());
 #endif
     setText(KEBListView::CommentColumn, NodeEditCommand::getNodeText(bk, "desc"));
-    bool shown = CmdGen::self()->shownInToolbar(bk);
+    bool shown = CmdGen::shownInToolbar(bk);
     setPixmap(0, SmallIcon(shown ? QString("bookmark_toolbar") : bk.icon()));
     // DESIGN - modUpdate badly needs a redesign
     modUpdate();
@@ -770,25 +850,25 @@ KEBListViewItem::KEBListViewItem(KEBListViewItem *parent, QListViewItem *after,
 
 // bookmark (first of its group)
 KEBListViewItem::KEBListViewItem(KEBListViewItem *parent, const KBookmark & bk)
-    : QListViewItem(parent, bk.fullText(), bk.url().prettyURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
+    : QListViewItem(parent, bk.fullText(), bk.url().pathOrURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
     normalConstruct(bk);
 }
 
 // bookmark (after another)
 KEBListViewItem::KEBListViewItem(KEBListViewItem *parent, QListViewItem *after, const KBookmark &bk)
-    : QListViewItem(parent, after, bk.fullText(), bk.url().prettyURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
+    : QListViewItem(parent, after, bk.fullText(), bk.url().pathOrURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
     normalConstruct(bk);
 }
 
 // root bookmark (first of its group)
 KEBListViewItem::KEBListViewItem(QListView *parent, const KBookmark & bk)
-    : QListViewItem(parent, bk.fullText(), bk.url().prettyURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
+    : QListViewItem(parent, bk.fullText(), bk.url().pathOrURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
     normalConstruct(bk);
 }
 
 // root bookmark (after another)
 KEBListViewItem::KEBListViewItem(QListView *parent, QListViewItem *after, const KBookmark &bk)
-    : QListViewItem(parent, after, bk.fullText(), bk.url().prettyURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
+    : QListViewItem(parent, after, bk.fullText(), bk.url().pathOrURL()), m_bookmark(bk), m_emptyFolderPadder(false) {
     normalConstruct(bk);
 }
 
@@ -815,18 +895,7 @@ void KEBListViewItem::boldStyle(QPainter *p) {
 
 void KEBListViewItem::paintCell(QPainter *p, const QColorGroup &ocg, int col, int w, int a) {
     QColorGroup cg(ocg);
-
-    bool parentSelected = false;
-    KEBListViewItem *allTheWayToTheRoot = this;
-    while (allTheWayToTheRoot 
-            = static_cast<KEBListViewItem *>(allTheWayToTheRoot->parent()), 
-           allTheWayToTheRoot) {
-        if (allTheWayToTheRoot->isSelected() 
-	 && allTheWayToTheRoot != listView()->firstChild() )
-            parentSelected  = true;
-    };
-
-    if (parentSelected) {
+    if (parentSelected(this)) {
         int base_h, base_s, base_v;
         cg.background().hsv(&base_h, &base_s, &base_v);
 
