@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef HISTORY_SETTINGS_H
@@ -25,7 +25,7 @@
 
 #include <dcopobject.h>
 
-class KonqSidebarHistorySettings : public QObject, virtual public DCOPObject
+class KonqSidebarHistorySettings : public QObject, public DCOPObject
 {
     K_DCOP
     Q_OBJECT
