@@ -50,6 +50,7 @@ public:
     void              setColorTable(const ColorEntry table[]);
 
     void setScrollbarLocation(int loc);
+    int  getScrollbarLocation() { return scrollLoc; }
     enum { SCRNONE=0, SCRLEFT=1, SCRRIGHT=2 };
 
     void setScroll(int cursor, int lines);
@@ -99,6 +100,7 @@ public:
     void setSelection(const QString &t);
 
     virtual void setFont(const QFont &);
+    QFont getVTFont() { return font(); }
     void setVTFont(const QFont &);
 
     void setMouseMarks(bool on);
@@ -301,7 +303,7 @@ private:
     bool m_isIMSel;
 
     QRgb blend_color;
-    
+ 
 private slots:
     void drop_menu_activated(int item);
     void swapColorTable();
