@@ -67,19 +67,25 @@ private:
 		QColor inactiveBlend;
 		QColor activeTitleBtnBg;
 		QColor inactiveTitleBtnBg;
+                QColor alternateBackground;
 	} usrColors, currentColors;
 	// first, the KDE 2 default color values
 	QColor widget;
-	QColor kde31Beige;
-	QColor emerald1;
-	QColor emerald2;
+	QColor kde34Blue;
+        QColor inactiveBackground;
+        QColor activeBackground;
 	QColor button;
 	QColor link;
 	QColor visitedLink;
+        QColor activeBlend;
+        QColor activeTitleBtnBg;
+        QColor inactiveTitleBtnBg;
+        QColor inactiveForeground;
+        QColor alternateBackground;
 
 	QListViewItem * kde;
 	QListViewItem * classic;
-	QListViewItem * plastik;
+	QListViewItem * keramik;
 	QListViewItem * cde;
 	QListViewItem * win;
 	QListViewItem * platinum;
