@@ -58,6 +58,8 @@ void KSysInfo::initXInfo() {
 	m_xvendor = !dpy ? QString::null : (QString)ServerVendor(dpy);
 	// XFree-Inc?
 	m_xfree_inc = m_xvendor.contains("XFree86");
+	// X.org ?
+	m_xorg = m_xvendor.contains("X.Org");
 	// release-number
 	m_xrelease = !dpy ? 0 : VendorRelease(dpy);
 	// RENDER-support
@@ -78,6 +80,10 @@ bool KSysInfo::isXfromXFreeInc() {
 	return m_xfree_inc;
 }
 
+bool KSysInfo::isXfromXOrg() {
+	return m_xorg;
+}
+
 int KSysInfo::getXRelease() {
 	return m_xrelease;
 }
@@ -102,9 +108,15 @@ void KSysInfo::initFontFamilies() {
 		if ( (font.contains("Arial [") || font=="Arial") && normal_priority < 15 ) {
 			m_normal_font = font;
 			normal_priority = 15;
+		} else if ( font.contains("Vera Sans") && normal_priority < 12 ) {
+			m_normal_font = font;
+			normal_priority = 12;
 		} else if ( (font.contains("Luxi Sans") || font.contains("Lucidux Sans")) && normal_priority < 10 ) {
 			m_normal_font = font;
 			normal_priority = 10;
+		} else if ( font.contains("Helmet") && normal_priority < 7 ) {
+			m_normal_font = font;
+			normal_priority = 7;
 		} else if ( font.contains("Nimbus Sans") && normal_priority < 5 ) {
 			m_normal_font = font;
 			normal_priority = 5;
