@@ -33,37 +33,8 @@ Copyright (C) 2000 Matthias Ettrich <ettrich@kde.org>
 typedef QValueList<QCString> QCStringList;
 class KSMListener;
 class KSMConnection;
-class KSMClient
-{
-public:
-    KSMClient( SmsConn );
-    ~KSMClient();
+class KSMClient;
 
-    void registerClient( const char* previousId  = 0 );
-    SmsConn connection() const { return smsConn; }
-
-    void resetState();
-    uint saveYourselfDone : 1;
-    uint pendingInteraction : 1;
-    uint waitForPhase2 : 1;
-    uint wasPhase2 : 1;
-
-    QPtrList<SmProp> properties;
-    SmProp* property( const char* name ) const;
-
-    QString program() const;
-    QStringList restartCommand() const;
-    QStringList discardCommand() const;
-    int restartStyleHint() const;
-    QString userId() const;
-    const char* clientId() { return id ? id : ""; }
-
-private:
-    const char* id;
-    SmsConn smsConn;
-};
-
-#ifndef NO_LEGACY_SESSION_MANAGEMENT
 enum SMType { SM_ERROR, SM_WMCOMMAND, SM_WMSAVEYOURSELF };
 struct SMData
     {
@@ -73,7 +44,6 @@ struct SMData
     QString wmclass1, wmclass2;
     };
 typedef QMap<WId,SMData> WindowMap;
-#endif
 
 class KSMServer : public QObject, public KSMServerInterface
 {
@@ -86,6 +56,8 @@ public:
     KSMServer( const QString& windowManager, bool only_local );
     ~KSMServer();
 
+    static KSMServer* self();
+
     void* watchConnection( IceConn iceConn );
     void removeConnection( KSMConnection* conn );
 
@@ -112,6 +84,9 @@ public:
                    KApplication::ShutdownType sdtype,
                    KApplication::ShutdownMode sdmode );
 
+    virtual void suspendStartup();
+    virtual void resumeStartup();
+
 public slots:
     void cleanUp();
 
@@ -127,7 +102,9 @@ private slots:
 
     void autoStart();
     void autoStart2();
-    void restoreNextInternal();
+    void tryRestoreNext();
+    void restoreNext();
+    void startupSuspendTimeout();
 
 private:
     void handlePendingInteractions();
@@ -148,8 +125,8 @@ private:
     void executeCommand( const QStringList& command );
     
     bool isWM( const KSMClient* client ) const;
+    void setupXIOErrorHandler();
 
-#ifndef NO_LEGACY_SESSION_MANAGEMENT
     void performLegacySessionSave();
     void storeLegacySession( KConfig* config );
     void restoreLegacySession( KConfig* config );
@@ -158,7 +135,6 @@ private:
     QString windowWmClientMachine(WId w);
     WId windowWmClientLeader(WId w);
     QCString windowSessionId(WId w, WId leader);
-#endif
 
     // public dcop interface
     void logout( int, int, int );
@@ -177,6 +153,7 @@ private:
     bool saveSession;
     int wmPhase1WaitingCount;
     int saveType;
+    int startupSuspendCount;
 
     KApplication::ShutdownType shutdownType;
     KApplication::ShutdownMode shutdownMode;
@@ -193,6 +170,7 @@ private:
     QString xonCommand;
     int logoutSoundEvent;
     QTimer knotifyTimeoutTimer;
+    QTimer startupSuspendTimeoutTimer;
 
     // ksplash interface
     void upAndRunning( const QString& msg );
@@ -205,9 +183,7 @@ private:
     
     QStringList excludeApps;
 
-#ifndef NO_LEGACY_SESSION_MANAGEMENT
     WindowMap legacyWindows;
-#endif
 };
 
 #endif
