@@ -25,9 +25,12 @@ public:
 
 k_dcop:
   virtual ASYNC upAndRunning( QString ) = 0;
+  virtual ASYNC setMaxProgress(int) = 0;
+  virtual ASYNC setProgress(int) = 0;
   virtual ASYNC setStartupItemCount( int count ) = 0;
   virtual ASYNC programStarted( QString programIcon, QString programName, QString description ) = 0;
   virtual ASYNC startupComplete() = 0;
+  virtual ASYNC close() = 0;
 };
 
 #endif
