@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 	KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>.
 	Please do not commit any changes without consulting me first. Thanks!
@@ -101,7 +101,7 @@ int ProcessLVI::compare( QListViewItem *item, int col, bool ascending ) const
 }
 
 ProcessList::ProcessList(QWidget *parent, const char* name)
-	: QListView(parent, name)
+	: KListView(parent, name)
 {
 	iconCache.setAutoDelete(true);
 
@@ -243,7 +243,7 @@ ProcessList::ProcessList(QWidget *parent, const char* name)
 	setAllColumnsShowFocus(true);
 	setTreeStepSize(17);
 	setSorting(sortColumn, increasing);
-	setSelectionMode(Multi);
+	setSelectionMode(QListView::Extended);
 
 	// Create popup menu for RMB clicks on table header
 	headerPM = new QPopupMenu();
@@ -281,6 +281,21 @@ ProcessList::getSelectedPIds()
 	return (selectedPIds);
 }
 
+const QStringList&
+ProcessList::getSelectedAsStrings()
+{
+	selectedAsStrings.clear();
+	// iterate through all items of the listview and find selected processes
+	QListViewItemIterator it(this);
+	QString spaces;
+	for ( ; it.current(); ++it )
+		if (it.current()->isSelected()) {
+			spaces.fill(QChar(' '), 7 - it.current()->text(1).length());
+			selectedAsStrings.append("(PID: " + it.current()->text(1) + ")" + spaces + " " + it.current()->text(0));
+		}	
+
+	return (selectedAsStrings);
+}
 bool
 ProcessList::update(const QString& list)
 {
@@ -326,21 +341,21 @@ ProcessList::update(const QString& list)
 	else
 		buildList();
 
-  QListViewItemIterator it( this );
-  while ( it.current() ) {
-    if ( itemPos( it.current() ) == currItemPos ) {
-      setCurrentItem( it.current() );
-      break;
-    }
-    ++it;
-  }
+	QListViewItemIterator it( this );
+	while ( it.current() ) {
+		if ( itemPos( it.current() ) == currItemPos ) {
+			setCurrentItem( it.current() );
+			break;
+		}
+		++it;
+	}
 
 	verticalScrollBar()->setValue(vpos);
 	horizontalScrollBar()->setValue(hpos);
 
 	// Re-enable painting, and force an update.
-	viewport()->setUpdatesEnabled(true);
 	setUpdatesEnabled(true);
+	viewport()->setUpdatesEnabled(true);
 
 	triggerUpdate();
 
@@ -768,7 +783,8 @@ ProcessList::handleRMBPressed(QListViewItem* lvi, const QPoint& p, int col)
 	for (int i = 0; i < columns(); ++i)
 		if (columnWidth(i) == 0)
 			hiddenPM->insertItem(header()->label(i), i + 100);
-	processPM->insertItem(i18n("Show Column"), hiddenPM);
+	if(columns())
+	  processPM->insertItem(i18n("Show Column"), hiddenPM);
 
 	processPM->insertSeparator();
 
@@ -821,7 +837,7 @@ ProcessList::handleRMBPressed(QListViewItem* lvi, const QPoint& p, int col)
 		break;
 	case 1:
 	case 2:
-		selectAll(id & 1);
+		selectAllItems(id & 1);
 		break;
 	case 3:
 	case 4:
@@ -841,7 +857,7 @@ ProcessList::handleRMBPressed(QListViewItem* lvi, const QPoint& p, int col)
 		int reniceVal;
 		if ((reniceVal = reniceDlg->exec()) != 40) {
 			delete reniceDlg;
-			emit reniceProcess(currentPId, reniceVal);
+			emit reniceProcess(selectedPIds, reniceVal);
     }
 		}
 		break;
@@ -867,7 +883,7 @@ ProcessList::handleRMBPressed(QListViewItem* lvi, const QPoint& p, int col)
 					selectedPIds.count())
 				.arg(signalPM->text(id));
 			int answ;
-			switch(answ = KMessageBox::questionYesNo(this, msg))
+			switch(answ = KMessageBox::questionYesNo(this, msg, QString::null, i18n("Send"), KStdGuiItem::cancel()))
 			{
 			case KMessageBox::Yes:
 			{
@@ -893,7 +909,7 @@ ProcessList::handleRMBPressed(QListViewItem* lvi, const QPoint& p, int col)
 }
 
 void
-ProcessList::selectAll(bool select)
+ProcessList::selectAllItems(bool select)
 {
 	selectedPIds.clear();
 
