@@ -14,9 +14,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    $Id: SensorShellAgent.cc,v 1.4 2003/03/07 22:08:24 waba Exp $
 */
 
 #include <stdlib.h>
@@ -33,7 +32,7 @@
 using namespace KSGRD;
 
 SensorShellAgent::SensorShellAgent( SensorManager *sm )
-  : SensorAgent( sm )
+  : SensorAgent( sm ), mDaemon( 0 )
 {
 }
 
@@ -41,7 +40,7 @@ SensorShellAgent::~SensorShellAgent()
 {
   if ( mDaemon ) {
     mDaemon->writeStdin( "quit\n", strlen( "quit\n" ) );
-    delete mDaemon;
+    delete (KShellProcess*)mDaemon;
 		mDaemon = 0;
   }
 }
@@ -117,11 +116,13 @@ void SensorShellAgent::errMsgRcvd( KProcess*, char *buffer, int buflen )
                 << endl << buf << endl;
 }
 
-void SensorShellAgent::daemonExited( KProcess* )
+void SensorShellAgent::daemonExited( KProcess *process )
 {
   setDaemonOnLine( false );
   sensorManager()->hostLost( this );
   sensorManager()->disengage( this );
+
+  process->deleteLater();
 }
 
 bool SensorShellAgent::writeMsg( const char *msg, int len )
