@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>. Please do
     not commit any changes without consulting me first. Thanks!
@@ -30,6 +30,7 @@
 #include <dcopobject.h>
 #include <kapplication.h>
 #include <kmainwindow.h>
+#include <dnssd/servicebrowser.h>
 
 #include <ksgrd/SensorClient.h>
 
@@ -84,6 +85,7 @@ class TopLevel : public KMainWindow, public KSGRD::SensorClient, public DCOPObje
     void editToolbars();
     void editStyle();
     void slotNewToolbarConfig();
+    void serviceAdded(DNSSD::RemoteService::Ptr srv);
 
   private:
     void setSwapInfo( long, long, const QString& );
@@ -95,6 +97,8 @@ class TopLevel : public KMainWindow, public KSGRD::SensorClient, public DCOPObje
 
     SensorBrowser* mSensorBrowser;
     Workspace* mWorkSpace;
+    
+    DNSSD::ServiceBrowser* mServiceBrowser;
 
     bool mDontSaveSession;
     int mTimerId;
