@@ -22,8 +22,18 @@ class KWinInterface : virtual public DCOPObject
     virtual void nextDesktop() = 0;
     virtual void previousDesktop() = 0;
     virtual void circulateDesktopApplications() = 0;
+    // kompmgr stuff
+    virtual void startKompmgr() = 0;
+    virtual void stopKompmgr() = 0;
+    virtual bool kompmgrIsRunning() = 0;
+    virtual void setOpacity(unsigned long winId, unsigned int opacityPercent) = 0;
+    virtual void setShadowSize(unsigned long winId, unsigned int shadowSizePercent) = 0;
+    virtual void setUnshadowed(unsigned long winId) = 0;
 
     k_dcop_signals:
+    
+    virtual void kompmgrStarted() = 0;
+    virtual void kompmgrStopped() = 0;
 
     // never emitted  
     virtual void dcopResetAllClients();
