@@ -248,6 +248,7 @@ QRect Workspace::clientArea( clientAreaOption opt, const QPoint& p, int desktop
                 return desktopwidget->geometry();
         case WorkArea:
             return warea;
+        case FullXineramaArea:
         case FullArea:
             return desktopwidget->geometry();
         case ScreenArea:
@@ -595,7 +596,8 @@ void Workspace::cascadeDesktop()
 // TODO XINERAMA this probably is not right for xinerama
     Q_ASSERT( block_stacking_updates == 0 );
     ClientList::ConstIterator it(stackingOrder().begin());
-    bool re_init_cascade_at_first_client = true;
+    initPositioning->reinitCascading( currentDesktop());
+    QRect area = clientArea( PlacementArea, QPoint( 0, 0 ), currentDesktop());
     for (; it != stackingOrder().end(); ++it)
         {
         if((!(*it)->isOnDesktop(currentDesktop())) ||
@@ -603,10 +605,7 @@ void Workspace::cascadeDesktop()
            ((*it)->isOnAllDesktops())              ||
            (!(*it)->isMovable()) )
             continue;
-        initPositioning->placeCascaded(*it, QRect(), re_init_cascade_at_first_client);
-        //CT is an if faster than an attribution?
-        if (re_init_cascade_at_first_client)
-          re_init_cascade_at_first_client = false;
+        initPositioning->placeCascaded(*it, area);
         }
     }
 
@@ -882,8 +881,6 @@ void Client::checkWorkspacePosition()
         }
     if( isDock())
         return;
-    if( isOverride())
-        return; // I wish I knew what to do here :(
     if( isTopMenu())
         {
         if( workspace()->managingTopMenus())
@@ -1011,6 +1008,13 @@ QSize Client::adjustedSize( const QSize& frame, Sizemode mode ) const
     return sizeForClientSize( wsize, mode, false );
     }
 
+// this helper returns proper size even if the window is shaded
+// see also the comment in Client::setGeometry()
+QSize Client::adjustedSize() const
+    {
+    return sizeForClientSize( clientSize());
+    }
+
 /*!
   Calculate the appropriate frame size for the given client size \a
   wsize.
@@ -1023,6 +1027,11 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
     {
     int w = wsize.width();
     int h = wsize.height();
+    if( w < 1 || h < 1 )
+        {
+        kdWarning() << "sizeForClientSize() with empty size!" << endl;
+        kdWarning() << kdBacktrace() << endl;
+        }
     if (w<1) w = 1;
     if (h<1) h = 1;
 
@@ -1165,8 +1174,6 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
                 ASPECT_CHECK_GROW_H
                 break;
                 }
-            case SizemodeShaded:
-                break;
             }
 #undef ASPECT_CHECK_SHRINK_H_GROW_W
 #undef ASPECT_CHECK_SHRINK_W_GROW_H
@@ -1189,10 +1196,7 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
         w += border_left + border_right;
         h += border_top + border_bottom;
         }
-    QSize ret = rules()->checkSize( QSize( w, h ));
-    if ( mode == SizemodeShaded && wsize.height() == 0 )
-        ret.setHeight( noframe ? 0 : border_top + border_bottom );
-    return ret;
+    return rules()->checkSize( QSize( w, h ));
     }
 
 /*!
@@ -1253,7 +1257,7 @@ void Client::getWmNormalHints()
         xSizeHint.win_gravity = NorthWestGravity;
     if( isManaged())
         { // update to match restrictions
-        QSize new_size = adjustedSize( size());
+        QSize new_size = adjustedSize();
         if( new_size != size() && !isFullScreen())
             resizeWithChecks( new_size );
         }
@@ -1395,7 +1399,7 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             || ns != size())
             {
             QRect orig_geometry = geometry();
-            ++block_geometry;
+            GeometryUpdatesPostponer blocker( this );
             move( new_pos );
             plainResize( ns );
             setGeometry( QRect( calculateGravitation( false, gravity ), size()));
@@ -1404,8 +1408,6 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen()
                 && area.contains( orig_geometry ))
                 keepInArea( area );
-            --block_geometry;
-            setGeometry( geometry(), ForceGeometrySet );
 
             // this is part of the kicker-xinerama-hack... it should be
             // safe to remove when kicker gets proper ExtendedStrut support;
@@ -1430,7 +1432,7 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
         if( ns != size())  // don't restore if some app sets its own size again
             {
             QRect orig_geometry = geometry();
-            ++block_geometry;
+            GeometryUpdatesPostponer blocker( this );
             int save_gravity = xSizeHint.win_gravity;
             xSizeHint.win_gravity = gravity;
             resizeWithChecks( ns );
@@ -1440,8 +1442,6 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen()
                 && area.contains( orig_geometry ))
                 keepInArea( area );
-            --block_geometry;
-            setGeometry( geometry(), ForceGeometrySet );
             }
         }
     // No need to send synthetic configure notify event here, either it's sent together
@@ -1451,6 +1451,16 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
 
 void Client::resizeWithChecks( int w, int h, ForceGeometry_t force )
     {
+    if( shade_geometry_change )
+        assert( false );
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdWarning() << "Shaded geometry passed for size:" << endl;
+            kdWarning() << kdBacktrace() << endl;
+            }
+        }
     int newx = x();
     int newy = y();
     QRect area = workspace()->clientArea( WorkArea, this );
@@ -1544,7 +1554,7 @@ bool Client::isMovable() const
     {
     if( !motif_may_move || isFullScreen())
         return false;
-    if( isSpecialWindow() && !isOverride() && !isSplash() && !isToolbar()) // allow moving of splashscreens :)
+    if( isSpecialWindow() && !isSplash() && !isToolbar()) // allow moving of splashscreens :)
         return false;
     if( maximizeMode() == MaximizeFull && !options->moveResizeMaximizedWindows() )
         return false;
@@ -1560,7 +1570,7 @@ bool Client::isResizable() const
     {
     if( !motif_may_resize || isFullScreen())
         return false;
-    if(( isSpecialWindow() || isSplash() || isToolbar()) && !isOverride())
+    if( isSpecialWindow() )
         return false;
     if( maximizeMode() == MaximizeFull && !options->moveResizeMaximizedWindows() )
         return false;
@@ -1605,39 +1615,86 @@ bool Client::isMaximizable() const
  */
 void Client::setGeometry( int x, int y, int w, int h, ForceGeometry_t force )
     {
+    // this code is also duplicated in Client::plainResize()
+    // Ok, the shading geometry stuff. Generally, code doesn't care about shaded geometry,
+    // simply because there are too many places dealing with geometry. Those places
+    // ignore shaded state and use normal geometry, which they usually should get
+    // from adjustedSize(). Such geometry comes here, and if the window is shaded,
+    // the geometry is used only for client_size, since that one is not used when
+    // shading. Then the frame geometry is adjusted for the shaded geometry.
+    // This gets more complicated in the case the code does only something like
+    // setGeometry( geometry()) - geometry() will return the shaded frame geometry.
+    // Such code is wrong and should be changed to handle the case when the window is shaded,
+    // for example using Client::clientSize().
+    if( shade_geometry_change )
+        ; // nothing
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdDebug() << "Shaded geometry passed for size:" << endl;
+            kdDebug() << kdBacktrace() << endl;
+            }
+        else
+            {
+            client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+            h = border_top + border_bottom;
+            }
+        }
+    else
+        {
+        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+        }
     if( force == NormalGeometrySet && frame_geometry == QRect( x, y, w, h ))
         return;
-    h = checkShadeGeometry( w, h );
     frame_geometry = QRect( x, y, w, h );
-    if( !isShade())
-        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
-    else
-        client_size = QSize( w - border_left - border_right, client_size.height());
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
-        resizeDecoration( QSize( w, h ));
-        if( !isShade())
-            {
-            QSize cs = clientSize();
-            XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
-                cs.width(), cs.height());
-    	    // FRAME tady poradi tak, at neni flicker
-            XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
-            }
-        if( shape())
-            updateShape();
-        // SELI TODO won't this be too expensive?
-        updateWorkareaDiffs();
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
+    resizeDecoration( QSize( w, h ));
+    XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
+//     resizeDecoration( QSize( w, h ));
+    if( !isShade())
+        {
+        QSize cs = clientSize();
+        XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
+            cs.width(), cs.height());
+        XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
+        }
+    if( shape())
+        updateShape();
+    // SELI TODO won't this be too expensive?
+    updateWorkareaDiffs();
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 void Client::plainResize( int w, int h, ForceGeometry_t force )
     {
+    // this code is also duplicated in Client::setGeometry(), and it's also commented there
+    if( shade_geometry_change )
+        ; // nothing
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdDebug() << "Shaded geometry passed for size:" << endl;
+            kdDebug() << kdBacktrace() << endl;
+            }
+        else
+            {
+            client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+            h = border_top + border_bottom;
+            }
+        }
+    else
+        {
+        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+        }
     if( QSize( w, h ) != rules()->checkSize( QSize( w, h )))
         {
         kdDebug() << "forced size fail:" << QSize( w,h ) << ":" << rules()->checkSize( QSize( w, h )) << endl;
@@ -1645,49 +1702,29 @@ void Client::plainResize( int w, int h, ForceGeometry_t force )
         }
     if( force == NormalGeometrySet && frame_geometry.size() == QSize( w, h ))
         return;
-    h = checkShadeGeometry( w, h );
     frame_geometry.setSize( QSize( w, h ));
-    if( !isShade())
-        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
-    else
-        client_size = QSize( w - border_left - border_right, client_size.height());
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XResizeWindow( qt_xdisplay(), frameId(), w, h );
-        resizeDecoration( QSize( w, h ));
-        if( !isShade())
-            {
-            QSize cs = clientSize();
-            XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
-                cs.width(), cs.height());
-            XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
-            }
-        if( shape())
-            updateShape();
-        updateWorkareaDiffs();
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
-    }
-
-// There may be cases when an application requests resizing while shaded,
-// and even KWin itself may do so somewhere (too many places to check :-/ ).
-// If the requested geometry doesn't fit shaded geometry, adjust the height
-// of the requested geometry and return it.
-int Client::checkShadeGeometry( int w, int h )
-    {
-    // check that the frame is not resized to full size when it should be shaded
-    if( isShade() && !shade_geometry_change && h != border_top + border_bottom )
+    resizeDecoration( QSize( w, h ));
+    XResizeWindow( qt_xdisplay(), frameId(), w, h );
+//     resizeDecoration( QSize( w, h ));
+    if( !isShade())
         {
-        kdDebug() << "Fixing shaded geometry:" << this << endl;
-        // adjust the client size to match the newly requested geometry
-        client_size = adjustedSize( QSize( w, h ));
-//        checkMaximizeGeometry(); // doesn't work, actual setting of geometry changes this again
-        h = border_top + border_bottom;
+        QSize cs = clientSize();
+        XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
+            cs.width(), cs.height());
+        XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
         }
-    return h;
+    if( shape())
+        updateShape();
+    updateWorkareaDiffs();
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 /*!
@@ -1699,16 +1736,42 @@ void Client::move( int x, int y, ForceGeometry_t force )
         return;
     frame_geometry.moveTopLeft( QPoint( x, y ));
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XMoveWindow( qt_xdisplay(), frameId(), x, y );
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
+    XMoveWindow( qt_xdisplay(), frameId(), x, y );
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 
+void Client::postponeGeometryUpdates( bool postpone )
+    {
+    if( postpone )
+        {
+        if( postpone_geometry_updates == 0 )
+            pending_geometry_update = false;
+        ++postpone_geometry_updates;
+        }
+    else
+        {
+        if( --postpone_geometry_updates == 0 )
+            {
+            if( pending_geometry_update )
+                {
+                if( isShade())
+                    setGeometry( QRect( pos(), adjustedSize()), ForceGeometrySet );
+                else
+                    setGeometry( geometry(), ForceGeometrySet );
+                pending_geometry_update = false;
+                }
+            }
+        }
+    }
+
 void Client::maximize( MaximizeMode m )
     {
     setMaximize( m & MaximizeVertical, m & MaximizeHorizontal );
@@ -1744,7 +1807,7 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     if( !adjust && max_mode == old_mode )
         return;
 
-    ++block_geometry; // TODO GeometryBlocker class?
+    GeometryUpdatesPostponer blocker( this );
 
     // maximing one way and unmaximizing the other way shouldn't happen
     Q_ASSERT( !( vertical && horizontal )
@@ -1753,12 +1816,12 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     QRect clientArea = workspace()->clientArea( MaximizeArea, this );
 
     // save sizes for restoring, if maximalizing
-    if( !( y() == clientArea.top() && height() == clientArea.height()))
+    if( !adjust && !( y() == clientArea.top() && height() == clientArea.height()))
         {
         geom_restore.setTop( y());
         geom_restore.setHeight( height());
         }
-    if( !( x() == clientArea.left() && width() == clientArea.width()))
+    if( !adjust && !( x() == clientArea.left() && width() == clientArea.width()))
         {
         geom_restore.setLeft( x());
         geom_restore.setWidth( width());
@@ -1776,6 +1839,21 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     if( decoration != NULL ) // decorations may turn off some borders when maximized
         decoration->borders( border_left, border_right, border_top, border_bottom );
 
+    // restore partial maximizations
+    if ( old_mode==MaximizeFull && max_mode==MaximizeRestore )
+        {
+        if ( maximizeModeRestore()==MaximizeVertical )
+	    {
+	    max_mode = MaximizeVertical;
+	    maxmode_restore = MaximizeRestore;
+	    }
+	if ( maximizeModeRestore()==MaximizeHorizontal )
+	    {
+	    max_mode = MaximizeHorizontal;
+	    maxmode_restore = MaximizeRestore;
+	    }	
+	}
+    
     switch (max_mode)
         {
 
@@ -1790,11 +1868,11 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                     }
                 else
                     setGeometry( QRect(QPoint( geom_restore.x(), clientArea.top()),
-                              adjustedSize(QSize( geom_restore.width(), clientArea.height()), SizemodeFixedH )));
+                              adjustedSize(QSize( geom_restore.width(), clientArea.height()), SizemodeFixedH )), ForceGeometrySet);
                 }
             else
                 setGeometry( QRect(QPoint(x(), clientArea.top()),
-                              adjustedSize(QSize(width(), clientArea.height()), SizemodeFixedH )));
+                              adjustedSize(QSize(width(), clientArea.height()), SizemodeFixedH )), ForceGeometrySet);
             info->setState( NET::MaxVert, NET::Max );
             break;
             }
@@ -1810,11 +1888,11 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                     }
                 else
                     setGeometry( QRect( QPoint(clientArea.left(), geom_restore.y()),
-                              adjustedSize(QSize(clientArea.width(), geom_restore.height()), SizemodeFixedW )));
+                              adjustedSize(QSize(clientArea.width(), geom_restore.height()), SizemodeFixedW )), ForceGeometrySet);
                 }
             else
                 setGeometry( QRect( QPoint(clientArea.left(), y()),
-                              adjustedSize(QSize(clientArea.width(), height()), SizemodeFixedW )));
+                              adjustedSize(QSize(clientArea.width(), height()), SizemodeFixedW )), ForceGeometrySet);
             info->setState( NET::MaxHoriz, NET::Max );
             break;
             }
@@ -1848,16 +1926,23 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                 if( geom_restore.height() > 0 )
                     restore.moveTop( geom_restore.y());
                 }
-            setGeometry( restore );
+            setGeometry( restore, ForceGeometrySet );
             info->setState( 0, NET::Max );
             break;
             }
 
         case MaximizeFull:
             {
+            if( !adjust )
+                {
+                if( old_mode & MaximizeVertical )
+                    maxmode_restore = MaximizeVertical;
+                if( old_mode & MaximizeHorizontal )
+    	            maxmode_restore = MaximizeHorizontal;
+                }
             QSize adjSize = adjustedSize(clientArea.size(), SizemodeMax );
             QRect r = QRect(clientArea.topLeft(), adjSize);
-            setGeometry( r );
+            setGeometry( r, ForceGeometrySet );
             info->setState( NET::Max, NET::Max );
             break;
             }
@@ -1865,9 +1950,6 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
             break;
         }
 
-    --block_geometry;
-    setGeometry( geometry(), ForceGeometrySet );
-
     updateAllowedActions();
     if( decoration != NULL )
         decoration->maximizeChange();
@@ -1884,7 +1966,10 @@ void Client::resetMaximize()
     updateAllowedActions();
     if( decoration != NULL )
         decoration->borders( border_left, border_right, border_top, border_bottom );
-    setGeometry( geometry(), ForceGeometrySet );
+    if( isShade())
+        setGeometry( QRect( pos(), sizeForClientSize( clientSize())), ForceGeometrySet );
+    else
+        setGeometry( geometry(), ForceGeometrySet );
     if( decoration != NULL )
         decoration->maximizeChange();
     }
@@ -1934,7 +2019,7 @@ bool Client::isFullScreenable( bool fullscreen_hack ) const
     if( !rules()->checkFullScreen( true ))
         return false;
     if( fullscreen_hack )
-        return isNormalWindow() || isOverride();
+        return isNormalWindow();
     if( rules()->checkStrictGeometry( false ))
         {
         // the app wouldn't fit exactly fullscreen geometry due its strict geometry requirements
@@ -1973,18 +2058,21 @@ void Client::setFullScreen( bool set, bool user )
     fullscreen_mode = set ? FullScreenNormal : FullScreenNone;
     if( was_fs == isFullScreen())
         return;
-    StackingUpdatesBlocker blocker( workspace());
-    ++block_geometry;
+    StackingUpdatesBlocker blocker1( workspace());
+    GeometryUpdatesPostponer blocker2( this );
     workspace()->updateClientLayer( this ); // active fullscreens get different layer
     info->setState( isFullScreen() ? NET::FullScreen : 0, NET::FullScreen );
     updateDecoration( false, false );
-    if( isFullScreen())
+    // XINERAMA
+    if( isFullScreen() && xSizeHint.width > workspace()->clientArea(FullScreenArea, this).width() )
+        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
+    else if( isFullScreen() && xSizeHint.height > workspace()->clientArea(FullScreenArea, this).height() )
+        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
+    else if( isFullScreen())
         setGeometry( workspace()->clientArea( FullScreenArea, this ));
     else
         {
-        if( maximizeMode() != MaximizeRestore )
-            changeMaximize( false, false, true ); // adjust size
-        else if( !geom_fs_restore.isNull())
+        if( !geom_fs_restore.isNull())
             setGeometry( QRect( geom_fs_restore.topLeft(), adjustedSize( geom_fs_restore.size())));
         // TODO isShaded() ?
         else
@@ -1992,8 +2080,6 @@ void Client::setFullScreen( bool set, bool user )
             setGeometry( workspace()->clientArea( MaximizeArea, this ));
             }
         }
-    --block_geometry;
-    setGeometry( geometry(), ForceGeometrySet );
     updateWindowRules();
     }
 
