@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
  */
@@ -75,6 +75,7 @@
 #define KWIN_SHADEHOVER            "ShadeHover"
 #define KWIN_SHADEHOVER_INTERVAL   "ShadeHoverInterval"
 #define KWIN_FOCUS_STEALING        "FocusStealingPreventionLevel"
+#define KWIN_HIDE_UTILITY          "HideUtilityWindowsForInactive"
 
 // kwm config keywords
 #define KWM_ELECTRIC_BORDER                  "ElectricBorders"
@@ -590,7 +591,7 @@ KAdvancedConfig::KAdvancedConfig (bool _standAlone, KConfig *_config, QWidget *p
     focusStealingLabel->setBuddy( focusStealing );
     focusStealingLayout->addWidget( focusStealingLabel );
     focusStealingLayout->addWidget( focusStealing, AlignLeft );
-    wtstr = i18n( "This option specifies how much KWin will try to prevent unwanted focus stealing "
+    wtstr = i18n( "<p>This option specifies how much KWin will try to prevent unwanted focus stealing "
                   "caused by unexpected activation of new windows. (Note: This feature does not "
                   "work with the Focus Under Mouse or Focus Strictly Under Mouse focus policies.)"
                   "<ul>"
@@ -605,11 +606,21 @@ KAdvancedConfig::KAdvancedConfig (bool _standAlone, KConfig *_config, QWidget *p
                   "or if they belong to the currently active application. This setting is probably "
                   "not really usable when not using mouse focus policy.</li>"
                   "<li><em>Extreme:</em> All windows must be explicitly activated by the user.</li>"
-                  "</ul>" );
+                  "</ul></p>"
+                  "<p>Windows that are prevented from stealing focus are marked as demanding attention, "
+                  "which by default means their taskbar entry will be highlighted. This can be changed "
+                  "in the Notifications control module.</p>" );
     QWhatsThis::add( focusStealing, wtstr );
     QWhatsThis::add( focusStealingLabel, wtstr );
-
     connect(focusStealing, SIGNAL(activated(int)), SLOT(changed()));
+    
+    hideUtilityWindowsForInactive = new QCheckBox( i18n( "Hide utility windows for inactive applications" ), this );
+    QWhatsThis::add( hideUtilityWindowsForInactive,
+        i18n( "When turned on, utility windows (tool windows, torn-off menus,...) of inactive applications will be"
+              " hidden and will be shown only when the application becomes active. Note that applications"
+              " have to mark the windows with the proper window type for this feature to work." ));
+    connect(hideUtilityWindowsForInactive, SIGNAL(toggled(bool)), SLOT(changed()));
+    lay->addWidget( hideUtilityWindowsForInactive );
 
     lay->addStretch();
     load();
@@ -643,6 +654,10 @@ void KAdvancedConfig::setFocusStealing(int l) {
     focusStealing->setCurrentItem(l);
 }
 
+void KAdvancedConfig::setHideUtilityWindowsForInactive(bool s) {
+    hideUtilityWindowsForInactive->setChecked( s );
+}
+
 void KAdvancedConfig::load( void )
 {
     config->setGroup( "Windows" );
@@ -657,6 +672,7 @@ void KAdvancedConfig::load( void )
 //    setFocusStealing( config->readNumEntry(KWIN_FOCUS_STEALING, 2 ));
     // TODO default to low for now
     setFocusStealing( config->readNumEntry(KWIN_FOCUS_STEALING, 1 ));
+    setHideUtilityWindowsForInactive( config->readBoolEntry( KWIN_HIDE_UTILITY, true ));
 
     emit KCModule::changed(false);
 }
@@ -680,6 +696,7 @@ void KAdvancedConfig::save( void )
     config->writeEntry(KWM_ELECTRIC_BORDER_DELAY,getElectricBorderDelay());
 
     config->writeEntry(KWIN_FOCUS_STEALING, focusStealing->currentItem());
+    config->writeEntry(KWIN_HIDE_UTILITY, hideUtilityWindowsForInactive->isChecked());
 
     if (standAlone)
     {
@@ -701,6 +718,7 @@ void KAdvancedConfig::defaults()
 //    setFocusStealing(2);
     // TODO default to low for now
     setFocusStealing(1);
+    setHideUtilityWindowsForInactive( true );
     emit KCModule::changed(true);
 }
 
@@ -825,6 +843,7 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, QWidget *paren
 
     placementCombo = new QComboBox(false, windowsBox);
     placementCombo->insertItem(i18n("Smart"), SMART_PLACEMENT);
+    placementCombo->insertItem(i18n("Maximizing"), MAXIMIZING_PLACEMENT);
     placementCombo->insertItem(i18n("Cascade"), CASCADE_PLACEMENT);
     placementCombo->insertItem(i18n("Random"), RANDOM_PLACEMENT);
     placementCombo->insertItem(i18n("Centered"), CENTERED_PLACEMENT);
@@ -839,6 +858,9 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, QWidget *paren
                  " will appear on the desktop."
                  " <ul>"
                  " <li><em>Smart</em> will try to achieve a minimum overlap of windows</li>"
+                 " <li><em>Maximizing</em> will try to maximize every window to fill the whole screen."
+                 " It might be useful to selectively affect placement of some windows using"
+                 " the window-specific settings.</li>"
                  " <li><em>Cascade</em> will cascade the windows</li>"
                  " <li><em>Random</em> will use a random position</li>"
                  " <li><em>Centered</em> will place the window centered</li>"
@@ -1045,12 +1067,13 @@ void KMovingConfig::load( void )
         setPlacement(CENTERED_PLACEMENT);
     else if( key == "ZeroCornered")
         setPlacement(ZEROCORNERED_PLACEMENT);
-
+    else if( key == "Maximizing")
+        setPlacement(MAXIMIZING_PLACEMENT);
     else
         setPlacement(SMART_PLACEMENT);
 //  }
 
-    setMoveResizeMaximized(config->readBoolEntry(KWIN_MOVE_RESIZE_MAXIMIZED, true));
+    setMoveResizeMaximized(config->readBoolEntry(KWIN_MOVE_RESIZE_MAXIMIZED, false));
 
     int v;
 
@@ -1092,6 +1115,8 @@ void KMovingConfig::save( void )
         config->writeEntry(KWIN_PLACEMENT, "Centered");
     else if (v == ZEROCORNERED_PLACEMENT)
         config->writeEntry(KWIN_PLACEMENT, "ZeroCornered");
+    else if (v == MAXIMIZING_PLACEMENT)
+        config->writeEntry(KWIN_PLACEMENT, "Maximizing");
 //CT 13mar98 manual and interactive placement
 //   else if (v == MANUAL_PLACEMENT)
 //     config->writeEntry(KWIN_PLACEMENT, "Manual");
@@ -1134,7 +1159,7 @@ void KMovingConfig::defaults()
     setResizeOpaque(RESIZE_TRANSPARENT);
     setGeometryTip(false);
     setPlacement(SMART_PLACEMENT);
-    setMoveResizeMaximized(true);
+    setMoveResizeMaximized(false);
 
     //copied from kcontrol/konq/kwindesktop, aleXXX
     setWindowSnapZone(KWM_WNDW_SNAP_ZONE_DEFAULT);
@@ -1198,16 +1223,10 @@ KTranslucencyConfig::KTranslucencyConfig (bool _standAlone, KConfig *_config, QW
   QVBoxLayout *vLay = new QVBoxLayout (tGroup,KDialog::marginHint(), KDialog::spacingHint());
   vLay->addSpacing(11); // to get the proper gb top offset
   
-//  QHBoxLayout *hLay = new QHBoxLayout(vLay);
-//  QLabel *label0 = new QLabel(i18n("Apply translucency on"),tGroup);
-//  hLay->addWidget(label0);
-//  transMode = new QComboBox(tGroup);
-//  transMode->insertItem ("i18n(The whole window)");
-//  transMode->insertItem ("i18n(The titlebar only)");
-//  transMode->insertItem ("i18n(The content only)");
-//  hLay->addWidget(transMode);
+  onlyDecoTranslucent = new QCheckBox(i18n("Apply translucency only to decoration"),tGroup);
+  vLay->addWidget(onlyDecoTranslucent);
   
-//  vLay->addSpacing(11);
+  vLay->addSpacing(11);
   
   QGridLayout *gLay = new QGridLayout(vLay,4,2,KDialog::spacingHint());
   gLay->setColStretch(1,1);
@@ -1249,7 +1268,7 @@ KTranslucencyConfig::KTranslucencyConfig (bool _standAlone, KConfig *_config, QW
   vLay->addWidget(disableARGB);
 
   vLay->addStretch();
-  tabW->addTab(tGroup, i18n("Translucency"));
+  tabW->addTab(tGroup, i18n("Opacity"));
 
   QWidget *sGroup = new QWidget(tabW);
 //   sGroup->setCheckable(TRUE);
@@ -1318,10 +1337,10 @@ KTranslucencyConfig::KTranslucencyConfig (bool _standAlone, KConfig *_config, QW
   fadeOnOpacityChange = new QCheckBox(i18n("Fade between opacity changes"),eGroup);
   fadeInSpeed = new KIntNumInput(100, eGroup);
   fadeInSpeed->setRange(1,100);
-  fadeInSpeed->setLabel("Fade-in speed:");
+  fadeInSpeed->setLabel(i18n("Fade-in speed:"));
   fadeOutSpeed = new KIntNumInput(100, eGroup);
   fadeOutSpeed->setRange(1,100);
-  fadeOutSpeed->setLabel("Fade-out speed:");
+  fadeOutSpeed->setLabel(i18n("Fade-out speed:"));
   vLay3->addWidget(fadeInWindows);
   vLay3->addWidget(fadeOnOpacityChange);
   vLay3->addWidget(fadeInSpeed);
@@ -1342,7 +1361,7 @@ KTranslucencyConfig::KTranslucencyConfig (bool _standAlone, KConfig *_config, QW
   connect(dockWindowTransparency, SIGNAL(toggled(bool)), dockWindowOpacity, SLOT(setEnabled(bool)));
 
   connect(useTranslucency, SIGNAL(toggled(bool)), SLOT(changed()));
-//  connect(transMode, SIGNAL(activated(int)), SLOT(changed()));
+  connect(onlyDecoTranslucent, SIGNAL(toggled(bool)), SLOT(changed()));
   connect(activeWindowTransparency, SIGNAL(toggled(bool)), SLOT(changed()));
   connect(inactiveWindowTransparency, SIGNAL(toggled(bool)), SLOT(changed()));
   connect(movingWindowTransparency, SIGNAL(toggled(bool)), SLOT(changed()));
@@ -1383,7 +1402,6 @@ KTranslucencyConfig::KTranslucencyConfig (bool _standAlone, KConfig *_config, QW
 
   // handle kompmgr restarts if necessary
   connect(useTranslucency, SIGNAL(toggled(bool)), SLOT(resetKompmgr()));
-//  connect(transMode, SIGNAL(activated(int)), SLOT(resetKompmgr()));
   connect(disableARGB, SIGNAL(toggled(bool)), SLOT(resetKompmgr()));
   connect(useShadows, SIGNAL(toggled(bool)), SLOT(resetKompmgr()));
   connect(inactiveWindowShadowSize, SIGNAL(valueChanged(int)), SLOT(resetKompmgr()));
@@ -1414,10 +1432,11 @@ void KTranslucencyConfig::load( void )
   activeWindowTransparency->setChecked(config->readBoolEntry("TranslucentActiveWindows",false));
   inactiveWindowTransparency->setChecked(config->readBoolEntry("TranslucentInactiveWindows",true));
   movingWindowTransparency->setChecked(config->readBoolEntry("TranslucentMovingWindows",false));
-  removeShadowsOnMove->setChecked(config->readBoolEntry("RemoveShadowsOnMove",FALSE));
-  removeShadowsOnResize->setChecked(config->readBoolEntry("RemoveShadowsOnResize",FALSE));
+  removeShadowsOnMove->setChecked(config->readBoolEntry("RemoveShadowsOnMove",false));
+  removeShadowsOnResize->setChecked(config->readBoolEntry("RemoveShadowsOnResize",false));
   dockWindowTransparency->setChecked(config->readBoolEntry("TranslucentDocks",true));
   keepAboveAsActive->setChecked(config->readBoolEntry("TreatKeepAboveAsActive",true));
+  onlyDecoTranslucent->setChecked(config->readBoolEntry("OnlyDecoTranslucent",false));
 
   activeWindowOpacity->setValue(config->readNumEntry("ActiveWindowOpacity",100));
   inactiveWindowOpacity->setValue(config->readNumEntry("InactiveWindowOpacity",75));
@@ -1425,9 +1444,9 @@ void KTranslucencyConfig::load( void )
   dockWindowOpacity->setValue(config->readNumEntry("DockOpacity",80));
 
   int ass, iss, dss;
-  dss = config->readNumEntry("DockShadowSize", 50);
-  ass = config->readNumEntry("ActiveWindowShadowSize", 200);
-  iss = config->readNumEntry("InactiveWindowShadowSize", 100);
+  dss = config->readNumEntry("DockShadowSize", 33);
+  ass = config->readNumEntry("ActiveWindowShadowSize", 133);
+  iss = config->readNumEntry("InactiveWindowShadowSize", 67);
 
   activeWindowOpacity->setEnabled(activeWindowTransparency->isChecked());
   inactiveWindowOpacity->setEnabled(inactiveWindowTransparency->isChecked());
@@ -1437,9 +1456,6 @@ void KTranslucencyConfig::load( void )
   KConfig conf_(QDir::homeDirPath() + "/.xcompmgrrc");
   conf_.setGroup("xcompmgr");
   
-//  QString modeString = conf_.readEntry("TransMode","All");
-//  transMode->setCurrentItem(!modeString.compare("Content")?2:!modeString.compare("Title")?1:0);
-  
   disableARGB->setChecked(conf_.readBoolEntry("DisableARGB",FALSE));
 
   useShadows->setChecked(conf_.readEntry("Compmode","CompClientShadows").compare("CompClientShadows") == 0);
@@ -1490,11 +1506,14 @@ void KTranslucencyConfig::save( void )
   // (speed reasons, so the shadow matrix hasn't to be recreated for every window)
   // we set inactive windows to 100%, the radius to the inactive window value and adjust the multiplicators for docks and active windows
   // this way the user can set the three values without caring about the radius/multiplicator stuff
-  config->writeEntry("DockShadowSize",(int)(100.0*dockWindowShadowSize->value()/inactiveWindowShadowSize->value()));
-  config->writeEntry("ActiveWindowShadowSize",(int)(100.0*activeWindowShadowSize->value()/inactiveWindowShadowSize->value()));
-  config->writeEntry("InctiveWindowShadowSize",100);
+   // additionally we find a value between big and small values to have a more smooth appereance
+   config->writeEntry("DockShadowSize",(int)(200.0 * dockWindowShadowSize->value() / (activeWindowShadowSize->value() + inactiveWindowShadowSize->value())));
+   config->writeEntry("ActiveWindowShadowSize",(int)(200.0 * activeWindowShadowSize->value() / (activeWindowShadowSize->value() + inactiveWindowShadowSize->value())));
+   config->writeEntry("InactiveWindowShadowSize",(int)(200.0 * inactiveWindowShadowSize->value() / (activeWindowShadowSize->value() + inactiveWindowShadowSize->value())));
+   
   config->writeEntry("RemoveShadowsOnMove",removeShadowsOnMove->isChecked());
   config->writeEntry("RemoveShadowsOnResize",removeShadowsOnResize->isChecked());
+  config->writeEntry("OnlyDecoTranslucent", onlyDecoTranslucent->isChecked());
   config->writeEntry("ResetKompmgr",resetKompmgr_);
 
   KConfig *conf_ = new KConfig(QDir::homeDirPath() + "/.xcompmgrrc");
@@ -1504,7 +1523,6 @@ void KTranslucencyConfig::save( void )
   conf_->writeEntry("DisableARGB",disableARGB->isChecked());
   conf_->writeEntry("ShadowOffsetY",-1*shadowTopOffset->value());
   conf_->writeEntry("ShadowOffsetX",-1*shadowLeftOffset->value());
-//  conf_->writeEntry("TransMode",transMode->currentItem()==0?"All":transMode->currentItem()==1?"Title":"Content");
 
 
   int r, g, b;
@@ -1512,7 +1530,7 @@ void KTranslucencyConfig::save( void )
   QString hex;
   hex.sprintf("0x%02X%02X%02X", r,g,b);
   conf_->writeEntry("ShadowColor",hex);
-  conf_->writeEntry("ShadowRadius",inactiveWindowShadowSize->value());
+  conf_->writeEntry("ShadowRadius",(activeWindowShadowSize->value() + inactiveWindowShadowSize->value()) / 2);
   conf_->writeEntry("FadeWindows",fadeInWindows->isChecked());
   conf_->writeEntry("FadeTrans",fadeOnOpacityChange->isChecked());
   conf_->writeEntry("FadeInStep",fadeInSpeed->value()/1000.0);
@@ -1535,7 +1553,7 @@ void KTranslucencyConfig::defaults()
     if (!kompmgrAvailable_)
         return;
   useTranslucency->setChecked(false);
-//  transMode->setCurrentItem(0);
+  onlyDecoTranslucent->setChecked(false);
   activeWindowTransparency->setChecked(false);
   inactiveWindowTransparency->setChecked(true);
   movingWindowTransparency->setChecked(false);
