@@ -43,7 +43,8 @@ bool Client::manage( Window w, bool isMapped )
     grabXServer();
 
     // from this place on, manage() mustn't return false
-    block_geometry = 1;
+    postpone_geometry_updates = 1;
+    pending_geometry_update = true; // force update when finishing with geometry changes
 
     embedClient( w, attr );
 
@@ -103,6 +104,7 @@ bool Client::manage( Window w, bool isMapped )
     setCaption( cap_normal, true );
 
     detectNoBorder();
+    detectShapable();
     fetchIconicName();
     getWMHints(); // needs to be done before readTransient() because of reading the group
     modal = ( info->state() & NET::Modal ) != 0; // needs to be valid before handling groups
@@ -163,7 +165,7 @@ bool Client::manage( Window w, bool isMapped )
                  it != mainclients.end();
                  ++it )
                 {
-                if( (*it)->isSpecialWindow() && !(*it)->isOverride())
+                if( (*it)->isSpecialWindow())
                     continue; // don't consider toolbars etc when placing
                 maincl = *it;
                 if( (*it)->isOnCurrentDesktop())
@@ -249,7 +251,7 @@ bool Client::manage( Window w, bool isMapped )
         ; // force using placement policy
     else
         usePosition = true;
-    if( !rules()->checkIgnorePosition( !usePosition ))
+    if( !rules()->checkIgnoreGeometry( !usePosition ))
         {
         bool ignorePPosition = ( options->ignorePositionClasses.contains(QString::fromLatin1(resourceClass())));
 
@@ -425,7 +427,7 @@ bool Client::manage( Window w, bool isMapped )
     if( isTopMenu()) // they're shown in Workspace::addClient() if their mainwindow
         hideClient( true ); // is the active one
 
-    if ( isShown( true ) && !doNotShow )
+    if( isShown( true ) && !doNotShow )
         {
         if( isDialog())
             Notify::raise( Notify::TransNew );
@@ -443,46 +445,34 @@ bool Client::manage( Window w, bool isMapped )
         if( !isOnCurrentDesktop() && !isMapped && !session && ( allow || workspace()->sessionSaving()))
             workspace()->setCurrentDesktop( desktop());
 
-        if( isOnCurrentDesktop())
-            {
-            setMappingState( NormalState );
+        if( workspace()->showingDesktop())
+            workspace()->resetShowingDesktop( false );
 
-            if( isMapped )
+        if( isOnCurrentDesktop() && !isMapped && !allow )
+            workspace()->restackClientUnderActive( this );
+        else
+            workspace()->raiseClient( this );
+
+        updateVisibility();
+
+        if( !isMapped )
+            {
+            if( allow && isOnCurrentDesktop())
                 {
-                workspace()->raiseClient( this );
-                rawShow();
+                if( !isSpecialWindow())
+                    if ( options->focusPolicyIsReasonable() && wantsTabFocus() )
+                        workspace()->requestFocus( this );
                 }
             else
                 {
-                if( allow )
-                    {
-                    workspace()->raiseClient( this );
-                    rawShow();
-                    if( !isSpecialWindow() || isOverride())
-                        if ( options->focusPolicyIsReasonable() && wantsTabFocus() )
-                            workspace()->requestFocus( this );
-                    }
-                else
-                    {
-                    workspace()->restackClientUnderActive( this );
-                    rawShow();
-                    if( !session && ( !isSpecialWindow() || isOverride()))
+                if( !session && !isSpecialWindow())
                         demandAttention();
-                    }
                 }
             }
-        else
-            {
-            virtualDesktopChange();
-            workspace()->raiseClient( this );
-            if( !session && !isMapped )
-                demandAttention();
-            }
         }
-    else if( !doNotShow ) // !isShown()
+    else if( !doNotShow ) // if( !isShown( true ) && !doNotShow )
         {
-        rawHide();
-        setMappingState( IconicState );
+        updateVisibility();
         }
     else // doNotShow
         { // SELI HACK !!!
@@ -507,6 +497,8 @@ bool Client::manage( Window w, bool isMapped )
     ungrabXServer();
     
     client_rules.discardTemporary();
+    applyWindowRules(); // just in case
+    workspace()->discardUsedWindowRules( this, false ); // remove ApplyNow rules
     updateWindowRules(); // was blocked while !isManaged()
 
 // TODO there's a small problem here - isManaged() depends on the mapping state,
