@@ -183,6 +183,8 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         Client* topClientOnDesktop( int desktop, bool unconstrained = false ) const;
         Client* findDesktop( bool topmost, int desktop ) const;
         void sendClientToDesktop( Client* c, int desktop, bool dont_activate );
+        void windowToPreviousDesktop( Client* c );
+        void windowToNextDesktop( Client* c );
 
     // KDE4 remove me - and it's also in the DCOP interface :(
         void showWindowMenuAt( unsigned long id, int x, int y );
@@ -209,6 +211,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         SessionInfo* takeSessionInfo( Client* );
         WindowRules findWindowRules( const Client*, bool );
         void rulesUpdated();
+        void discardUsedWindowRules( Client* c, bool withdraw );
 
     // dcop interface
         void cascadeDesktop();
@@ -221,6 +224,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         QString desktopName( int desk ) const;
         void setDesktopLayout(int o, int x, int y);
+        void setShowingDesktop( bool showing );
+        void resetShowingDesktop( bool keep_hidden );
+        bool showingDesktop() const;
 
         bool isNotManaged( const QString& title );  // ### setter or getter ?
 
@@ -248,6 +254,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         bool forcedGlobalMouseGrab() const;
         void clientShortcutUpdated( Client* c );
         bool shortcutAvailable( const KShortcut& cut, Client* ignore = NULL ) const;
+        bool globalShortcutsDisabled() const;
+        void disableGlobalShortcuts( bool disable );
+        void disableGlobalShortcutsForClient( bool disable );
 
         void sessionSaveStarted();
         void sessionSaveDone();
@@ -329,6 +338,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void slotWindowToDesktopDown();
 
         void slotMouseEmulation();
+        void slotDisableGlobalShortcuts();
 
         void slotSettingsChanged( int category );
 
@@ -361,6 +371,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void gotTemporaryRulesMessage( const QString& );
         void cleanupTemporaryRules();
         void writeWindowRules();
+        void kipcMessage( int id, int data );
         // kompmgr
         void setPopupClientOpacity(int v);
         void resetClientOpacity();
@@ -471,7 +482,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         void loadSessionInfo();
         void loadWindowRules();
-        void editWindowRules( Client* );
+        void editWindowRules( Client* c, bool whole_app );
 
         QPtrList<SessionInfo> session;
         QValueList<Rules*> rules;
@@ -499,6 +510,10 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         ClientList focus_chain;
         ClientList should_get_focus; // last is most recent
         ClientList attention_chain;
+        
+        bool showing_desktop;
+        ClientList showing_desktop_clients;
+        int block_showing_desktop;
 
         GroupList groups;
 
@@ -532,6 +547,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         KGlobalAccel *client_keys;
         ShortcutDialog* client_keys_dialog;
         Client* client_keys_client;
+        KGlobalAccel *disable_shortcuts_keys;
+        bool global_shortcuts_disabled;
+        bool global_shortcuts_disabled_for_client;
 
         WId root;
 
@@ -622,7 +640,7 @@ class StackingUpdatesBlocker
     };
 
 // NET WM Protocol handler class
-class RootInfo : public NETRootInfo3
+class RootInfo : public NETRootInfo4
     {
     private:
         typedef KWinInternal::Client Client;  // because of NET::Client
@@ -639,6 +657,7 @@ class RootInfo : public NETRootInfo3
         virtual void gotPing(Window w, Time timestamp);
         virtual void restackWindow(Window w, RequestSource source, Window above, int detail, Time timestamp);
         virtual void gotTakeActivity(Window w, Time timestamp, long flags );
+        virtual void changeShowingDesktop( bool showing );
     private:
         Workspace* workspace;
     };
@@ -738,6 +757,16 @@ inline bool Workspace::forcedGlobalMouseGrab() const
     return forced_global_mouse_grab;
     }
 
+inline bool Workspace::showingDesktop() const
+    {
+    return showing_desktop;
+    }
+
+inline bool Workspace::globalShortcutsDisabled() const
+    {
+    return global_shortcuts_disabled || global_shortcuts_disabled_for_client;
+    }
+
 template< typename T >
 inline Client* Workspace::findClient( T predicate )
     {
