@@ -87,7 +87,7 @@ bool XKBExtension::setXkbOptions(const QString& options, bool resetOld)
 }
 
 bool XKBExtension::setLayout(const QString& model, 
-		const QString& layout, const char* variant, unsigned int group,
+		const QString& layout, const QString& variant, unsigned int group,
 		const char* baseGr)
 {
     if (/*rule.isEmpty() ||*/ model.isEmpty() || layout.isEmpty())
