@@ -53,6 +53,19 @@ static QString lookupLocalized(const QDict<char> &dict, const QString& text)
   return QString::null;
 }
 
+static QListViewItem* copyLVI(const QListViewItem* src, QListView* parent)
+{
+    QListViewItem* ret = new QListViewItem(parent);
+    for(int i = 0; i < 3; i++)
+    {
+        ret->setText(i, src->text(i));
+        if ( src->pixmap(i) )
+            ret->setPixmap(i, *src->pixmap(i));
+    }
+    
+    return ret;
+}
+
 OptionListItem::OptionListItem( OptionListItem *parent, const QString &text,
     Type tt, const QString &optionName )
     : QCheckListItem( parent, text, tt ), m_OptionName( optionName )
@@ -146,16 +159,24 @@ void LayoutConfig::updateStickyLimit()
     widget->spinStickyDepth->setMaxValue( (layoutsCnt<=2) ? 2 : layoutsCnt - 1);
 }
 
+const QString LayoutConfig::itemVariant(QListViewItem *item)
+{
+    return m_variants[QString::number((ptrdiff_t)item, 16)];
+}
+
 void LayoutConfig::add()
 {
     QListViewItem* sel = widget->listLayoutsSrc->selectedItem();
     if( sel == 0 )
 	return;
     
-    widget->listLayoutsSrc->takeItem(sel);
-    widget->listLayoutsDst->insertItem(sel);
+    // Create a copy of the sel widget, as one might add the same layout more
+    // than one time, with different variants.
+    QListViewItem* toadd = copyLVI(sel, widget->listLayoutsDst);
+    
+    widget->listLayoutsDst->insertItem(toadd);
     if( widget->listLayoutsDst->childCount() > 1 )
-	sel->moveItem(widget->listLayoutsDst->lastItem());
+	toadd->moveItem(widget->listLayoutsDst->lastItem());
 // disabling temporary: does not work reliable in Qt :(
 //    widget->listLayoutsSrc->setSelected(sel, true);
 //    layoutSelChanged(sel);
@@ -178,8 +199,7 @@ void LayoutConfig::remove()
     if( sel->itemAbove() )
 	newSel = sel->itemAbove();
 
-    widget->listLayoutsDst->takeItem(sel);
-    widget->listLayoutsSrc->insertItem(sel);
+    delete sel;
     if( newSel )
 	widget->listLayoutsSrc->setSelected(newSel, true);
     layoutSelChanged(newSel);
@@ -222,7 +242,7 @@ void LayoutConfig::variantChanged()
     }
 
     QString kbdLayout = lookupLocalized( m_rules->layouts(), selLayout->text(1) );
-    m_variants.replace(kbdLayout, widget->comboVariant->currentText().latin1());
+    m_variants.replace(QString::number((ptrdiff_t)selLayout, 16), widget->comboVariant->currentText());
 }
 
 void LayoutConfig::latinChanged()
@@ -277,15 +297,14 @@ void LayoutConfig::layoutSelChanged(QListViewItem *sel)
         return;
     }
 
-    char* variant = m_variants[kbdLayout];
     widget->comboVariant->insertStringList(vars);
 
-    if( variant ) {
-      widget->comboVariant->setCurrentText(variant);
+    if( ! itemVariant(sel).isNull() ) {
+      widget->comboVariant->setCurrentText(itemVariant(sel));
     }
     else {
       widget->comboVariant->setCurrentItem(0);
-      m_variants.insert(kbdLayout, widget->comboVariant->currentText().latin1());
+      m_variants.insert(QString::number((ptrdiff_t)sel, 16), widget->comboVariant->currentText().latin1());
     }
 
     updateLayoutCommand();
@@ -382,7 +401,6 @@ void LayoutConfig::updateLayoutCommand()
 
   if( sel ) {
     QString kbdLayout = lookupLocalized(m_rules->layouts(), sel->text(1));
-//    char* variant = m_variants[kbdLayout];
     QString variant = widget->comboVariant->currentText();
 
     setxkbmap = "setxkbmap"; //-rules " + m_rule
@@ -432,32 +450,30 @@ void LayoutConfig::load()
   QString m_name = m_rules->models()[model];
   widget->comboModel->setCurrentText(i18n(m_name.local8Bit()));
 
-  QString layout = config->readEntry("Layout", "us");
-  QString l_name = m_rules->layouts()[layout];
+  QString layvar = config->readEntry("Layout", "us(basic)");
+  QString l_name = m_rules->layouts()[m_rules->getLayout(layvar)];
 
   QStringList otherLayouts = config->readListEntry("Additional");
   if( !l_name.isEmpty() ) {
-    otherLayouts.prepend(layout);
+    otherLayouts.prepend(layvar);
   }
 // to optimize we should have gone from it.end to it.begin
   for ( QStringList::Iterator it = otherLayouts.begin(); it != otherLayouts.end(); ++it ) {
     QListViewItemIterator src_it( widget->listLayoutsSrc );
     for ( ; src_it.current(); ++src_it ) {
 	QListViewItem* srcItem = src_it.current();
-	if ( *it == lookupLocalized(m_rules->layouts(), src_it.current()->text(1)) ) {
-    	    widget->listLayoutsSrc->takeItem(srcItem);
-    	    widget->listLayoutsDst->insertItem(srcItem);
-	    srcItem->moveItem(widget->listLayoutsDst->lastItem());
+	if ( m_rules->getLayout(*it) == lookupLocalized(m_rules->layouts(), src_it.current()->text(1)) ) {
+    	    QListViewItem* newItem = copyLVI(srcItem, widget->listLayoutsDst);
+    	    widget->listLayoutsDst->insertItem(newItem);
+	    newItem->moveItem(widget->listLayoutsDst->lastItem());
+	    
+	    m_variants[QString::number((ptrdiff_t)newItem, 16)] = 
+	      m_rules->getVariant(*it, m_rules->getLayout(*it));
 	    break;
 	}
     }
   }
 
-// reading variants
-  QStringList vars = config->readListEntry("Variants");
-  m_rules->parseVariants(vars, m_variants);
-//  m_rules->parseVariants( vars, m_variants );
-
   QStringList incs = config->readListEntry( "Includes" );
   m_rules->parseVariants( incs, m_includes, false );
 
@@ -625,7 +641,12 @@ void LayoutConfig::save()
   if( item ) {
     layout = lookupLocalized(m_rules->layouts(), item->text(1));
     if( !layout.isEmpty() )
-	config->writeEntry("Layout", layout);
+    {
+        if ( !itemVariant(item).isNull() )
+            config->writeEntry("Layout", layout + "(" + itemVariant(item) + ")");
+        else
+	    config->writeEntry("Layout", layout);
+    }
     item = item->nextSibling();
   }
   else {
@@ -633,8 +654,10 @@ void LayoutConfig::save()
   }
 
   while (item) {
-	QString layout = lookupLocalized(m_rules->layouts(), item->text(1));
-	otherLayouts.append(layout);
+      QString layout = lookupLocalized(m_rules->layouts(), item->text(1));
+      if ( !itemVariant(item).isNull() )
+          layout += "(" + itemVariant(item) + ")";
+      otherLayouts.append(layout);
       item = item->nextSibling();
   }
   config->writeEntry("Additional", otherLayouts);
@@ -652,19 +675,10 @@ void LayoutConfig::save()
         inc += ")";
         incList.append(  inc );
     }
-    if( m_variants[ layout ] && m_variants[ layout ][ 0 ] != '\0' )
-    {
-	QString var = layout;
-	var += "(";
-	var += m_variants[ layout ];
-	var += ")";
-	varList.append( var );
-    }
 
     item = item->nextSibling();
   }
   config->writeEntry( "Includes", incList );
-  config->writeEntry("Variants", varList);
 
   config->writeEntry("Use", widget->chkEnable->isChecked());
   config->writeEntry("ShowSingle", widget->chkShowSingle->isChecked());
@@ -745,7 +759,7 @@ extern "C"
 
     if( key ) {
         int delay_ = config->readNumEntry("RepeatDelay", 250);
-        int rate_ = config->readNumEntry("RepeatRate", 30);
+        double rate_ = config->readDoubleNumEntry("RepeatRate", 30);
         set_repeatrate(delay_, rate_);
     }
 
