@@ -69,13 +69,15 @@ private:
   LayoutConfigWidget* widget;
 
   QDict<OptionListItem> m_optionGroups;
-  QDict<char> m_variants;
+  QMap<QString, QString> m_variants;
   QDict<char> m_includes;
 //  QString m_rule;
   KeyRules *m_rules;
 
   QWidget* makeOptionsTab();
   void updateStickyLimit();
+  
+  const QString itemVariant(QListViewItem *item);
 };
 
 
