@@ -10,11 +10,18 @@
 #include "pixmap.h"
 
 
+QDict<QPixmap> LayoutIcon::pixmaps(80);
+const QString LayoutIcon::flagTemplate("l10n/%1/flag.png");
+bool LayoutIcon::cacheWithFlags = true;
 
-const QPixmap& findPixmap(const QString& code_)
+
+const QPixmap&
+LayoutIcon::findPixmap(const QString& code_, bool showFlag)
 {
-  static QDict<QPixmap> pixmaps(80);
-  static const QString flagTemplate("l10n/%1/flag.png");
+    if( showFlag != cacheWithFlags ) {
+	cacheWithFlags = showFlag;
+	pixmaps.clear();
+    }
 
   QPixmap* pm = pixmaps[code_];
   if( pm )
@@ -24,7 +31,7 @@ const QPixmap& findPixmap(const QString& code_)
     pm = new QPixmap(21, 14);
     pm->fill(Qt::white);
 
-  QPainter p(pm);
+    QPainter p(pm);
   
     QFont font("sans");
     font.setPixelSize(10);
@@ -34,20 +41,36 @@ const QPixmap& findPixmap(const QString& code_)
     p.drawText(2, 1, pm->width(), pm->height()-2, Qt::AlignCenter, "err");
     p.setPen(Qt::blue);
     p.drawText(1, 0, pm->width(), pm->height()-2, Qt::AlignCenter, "err");
-  pixmaps.insert(code_, pm);
+    pixmaps.insert(code_, pm);
 
-  return *pm;
+    return *pm;
   }
 
 
   QString code = code_.mid(0, code_.find(QRegExp("[()+]")));	//co_foo,co(foo),co+foo
   code = code.mid(code.find('/') + 1);	//NEC/jp
 
-
   QString flag;
   int pos = code.length();
   if( code_ == "ar" )	// Arabic - not argentina
     flag = locate("locale", flagTemplate.arg("C"));
+  else
+  if( code_ == "sr" || code_ == "cs")	// Serbian language - Yugoslavia
+    flag = locate("locale", flagTemplate.arg("yu"));
+  else
+  if( code_ == "bs" )	// Bosnian language - Bosnia
+    flag = locate("locale", flagTemplate.arg("ba"));
+  else
+  if( code_ == "la" )	// Latin America
+    flag = locate("locale", flagTemplate.arg("C"));
+  else
+  if( code_ == "lo" )	// Lao
+    flag = locate("locale", flagTemplate.arg("la"));
+  else
+  if( code_ == "ml" || code_ == "dev" || code_ == "gur" 
+	|| code_ == "guj" || code_ == "kan" || code_ == "ori" 
+	|| code_ == "tel" || code_ == "tml" || code_ == "ben" ) // some Indian languages
+    flag = locate("locale", flagTemplate.arg("in"));
   else {
     flag = locate("locale", flagTemplate.arg(code.lower()));
     if (flag.isEmpty()) {
@@ -71,13 +94,14 @@ const QPixmap& findPixmap(const QString& code_)
   if (flag.isEmpty())
     flag = locate("locale", flagTemplate.arg("C"));
 
-  if (flag.isEmpty()) {		// could not find "C" flag - pretty bad!
+  if (flag.isEmpty() || !showFlag ) {
     pm = new QPixmap(21, 14);
     pm->fill(Qt::white);
   }
   else
     pm = new QPixmap(flag);
 
+
   QImage image = pm->convertToImage();
   for (int y=0; y<image.height(); y++)
     for(int x=0; x<image.width(); x++)
@@ -227,4 +251,13 @@ const QPixmap& findPixmap(const QString& code_)
   //these seem to be new in XFree86 4.4.0
    I18N_NOOP("Uzbek");
    I18N_NOOP("Faroese");
+
+  //these seem to be new in XOrg 6.8.2
+   I18N_NOOP("Dzongkha / Tibetan");
+   I18N_NOOP("Hungarian (US)");
+   I18N_NOOP("Irish");
+   I18N_NOOP("Israeli (phonetic)");
+   I18N_NOOP("Serbian (Cyrillic)");
+   I18N_NOOP("Serbian (Latin)");
+   I18N_NOOP("Swiss");
 #endif
