@@ -17,13 +17,13 @@
 
 #include "rules.h"
 
-const char* X11DirList[2] = {"/usr/X11R6/lib/X11/", "/usr/local/X11R6/lib/X11/"};
-const char* rulesFileList[2] = {"xkb/rules/xfree86", "xkb/rules/xorg"};
+const char* X11DirList[6] = {"/usr/share/X11/", "/usr/local/share/X11/", "/usr/X11R6/lib/X11/", "/usr/local/X11R6/lib/X11/", "/usr/lib/X11/", "/usr/local/lib/X11/"};
+const char* rulesFileList[2] = {"xkb/rules/xorg", "xkb/rules/xfree86"};
 
 KeyRules::KeyRules():
     m_layouts(90)
 {
-  for(int ii=0; ii<2; ii++)
+  for(int ii=0; ii<6; ii++)
     if( QDir(X11DirList[ii]).exists() ) {
 	X11_DIR = X11DirList[ii];
 	break;
@@ -34,13 +34,16 @@ KeyRules::KeyRules():
     return;
   }
 
+// AR: 2006.01.18: removed this check as it does not work on some SuSE's
+// - will assume XFree 4.3 or later (BUG: 117468)
+
   // ugly check for XFree 4.3 or older
-  if( QDir(X11_DIR + "xkb/symbols/pc").exists() ) {
+//  if( QDir(X11_DIR + "xkb/symbols/pc").exists() ) {
 	m_xfree43 = true;
-  }
-  else {
-	m_xfree43 = false;
-  }
+//  }
+//  else {
+//	m_xfree43 = false;
+//  }
 
   QString rulesFile;
   Display *dpy = qt_xdisplay();
@@ -290,28 +293,68 @@ KeyRules::getVariants(const QString& layout)
     return *result;
 }
 
+/**
+ * @brief Gets the single layout part of a layout(variant) string
+ * @param[in] layvar String in form layout(variant) to parse
+ * @return The layout found in the string
+ */
+const QString KeyRules::getLayout(const QString &layvar) const
+{
+  static const char* LAYOUT_PATTERN = "[a-z0-9_-]*";
+  QString varLine = layvar.stripWhiteSpace();
+  QRegExp rx(LAYOUT_PATTERN);
+  int pos = rx.search(varLine, 0);
+  int len = rx.matchedLength();
+  // check for errors
+  if( pos < 0 || len < 2 )
+    return QString::null;
+  return varLine.mid(pos, len);
+}
+
+/**
+ * @brief Gets the single variant part of a layout(variant) string
+ * @param[in] layvar String in form layout(variant) to parse
+ * @return The variant found in the string, no check is performed
+ */
+const QString KeyRules::getVariant(const QString &layvar) const
+{
+  static const char* VARIANT_PATTERN = "\\([a-z0-9_-]*\\)";
+  QString varLine = layvar.stripWhiteSpace();
+  QRegExp rx(VARIANT_PATTERN);
+  int pos = rx.search(varLine, 0);
+  int len = rx.matchedLength();
+  // check for errors
+  if( pos < 2 || len < 2 )
+    return QString::null;
+  return varLine.mid(pos+1, len-2);
+}
+
+/**
+ * @brief Gets the single variant part of a layout(variant) string
+ * @param[in] layvar String in form layout(variant) to parse
+ * @param[in] layout When validating the variant, use this as layout to check
+ *   against. Use QString::null to avoid validating.
+ * @return The variant found in the string, or "basic" if the variant does not
+ *   exists and layout is non-null.
+ */
+const QString KeyRules::getVariant(const QString &layvar, const QString &layout)
+{
+  QString variant_ = getVariant(layvar);
+  
+  if ( ! layout.isNull() )
+  {
+    QStringList addVars = getVariants(layout);
+    if( !addVars.contains(variant_) )
+      variant_ = "basic";
+  }
+  return variant_;
+}
+
 void KeyRules::parseVariants(const QStringList& vars, QDict<char>& variants, bool chkVars)
 {
-  static const char* LAYOUT_PATTERN = "[a-z0-9_]*";
-  static const char* VARIANT_PATTERN = "\\([a-z0-9_]*\\)";
   for (QStringList::ConstIterator it = vars.begin(); it != vars.end(); ++it)
   {
-      QString varLine = (*it).stripWhiteSpace();
-      QRegExp rx(LAYOUT_PATTERN);
-      int pos = rx.search(varLine, 0);
-      int len = rx.matchedLength();
-  // check for errors
-      if( pos < 0 || len < 2 )
-        continue;
-      QString layout = varLine.mid(pos, len);
-      rx.setPattern(VARIANT_PATTERN);
-      pos = rx.search(varLine, pos+len);
-      len = rx.matchedLength();
-  // check for errors
-      if( pos < 2 || len < 2 )
-        continue;
-      QString variant_ = varLine.mid(pos+1, len-2);
-
+      QString layout = getLayout(*it), variant_ = getVariant(*it, QString::null);
 
       QStringList addVars = getVariants(layout);
       if( !chkVars || (!variant_.isEmpty() && addVars.contains(variant_)) )
