@@ -10,14 +10,18 @@ class KeyRules
 {
 public:
 
-  KeyRules(QString rule="xfree86");
+  KeyRules();
 
   const QDict<char> &models() const { return m_models; };
   const QDict<char> &layouts() const { return m_layouts; };
   const QDict<char> &options() const { return m_options; };
 
+  const QString getLayout(const QString &layvar) const;
+  const QString getVariant(const QString &layvar) const;
+  const QString getVariant(const QString &layvar, const QString &layout);
+
   void parseVariants(const QStringList& vars, QDict<char>& variants, bool chkVars=true);
-  static QStringList rules(QString path = QString::null);
+//  static QStringList rules(QString path = QString::null);
 
   QStringList getVariants(const QString& layout);
   unsigned int getGroup(const QString& layout, const char* baseGr);
@@ -42,6 +46,8 @@ private:
   QStringList m_oldLayouts;
   QStringList m_nonLatinLayouts;
   bool m_xfree43;
+  
+  QString X11_DIR;	// pseudo-constant
 };
 
 
